/*
 *  QIFConfirmation.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-2000. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 */

// --------------------------------------------------------------------------
//
// Function:
//
//    Puts the information from the QIFDownLoadForm page into a cached
//    GtAccountHistory object and verifies the input data from the form.
//
//    If everything verifies and the OK button is selected, the QIFDownLoadSuccess
//    servlet is run to get the account history information from the host system.
//
//    Control is returned to the QIFDownLoadForm when the CANCEL button or the
//    'QIF down load form' link is selected.
//
// Http Request Parameters:
//
//    The following request parameter name is expected as input by this servlet:
//       ah    - object ID of the cached GtAccountHistory object
//
//    If the user clicks the 'OK' button, the following parameter name is posted
//    to the QIFDownLoadSuccess page by this servlet:
//       ah - object id of the GtAccountHistory instance that was cached
//            on this confirm page.
//
//    If an error occurs due to the field validation, the following parameter names
//    are posted to the QIFDownLoadForm page by this servlet:
//       1. phase - flag indicationg the redisplay of the input form to correct the
//                  the errors. This has a value - "REDISPLAY".
//       2. errMsg - error message describing the cause of the error .
//       3. ah - object id of the GtAccountHistory instance that was cached
//                     on this confirm page.
//
//    If the user clicks the 'CANCEL' button or the 'QIF down load form' link,
//    the following parameter names are posted to the QIFDownLoadForm page by this servlet:
//       1. phase - flag indicationg the redisplay of the input form to correct input.
//                  This has a value - "REDISPLAY".
//       2. ah - object id of the GtAccountHistory instance that was cached
//                     on this confirm page.
// --------------------------------------------------------------------------

import java.io.PrintWriter;
import java.io.IOException;
import java.net.URLEncoder;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import com.ibm.ifs.gold.auto.GtAccountHistory;
import com.ibm.ifs.servlets.IfsHttpServlet;
import com.ibm.ifs.gold.IfsSession;
import com.ibm.ifs.gold.IfsSystem;
import com.ibm.ifs.gold.IfsSecurity;
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.GsCurrency;
import com.ibm.ifs.gold.GsDate;

//#################################################
//             QIFConfirmation
//
// Servlet Class which displays the confirmation of the QIF download operation.
//#################################################
public class QIFConfirmation extends IfsHttpServlet
{
   // Use this flag to debug this servlet
   public static boolean debugOn = false;

   //#################################################
   //             doPost
   //
   // Processes the HTTP POST method request.
   //#################################################
   public void doPost (HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      //----------------
      // Declare and initialize local variables
      //
      setDefaultHeader(response);   // Need to set character set before getWriter()
      PrintWriter   out         = response.getWriter();
      IfsSession    ifsSession  = new IfsSession(request, response);
      String        validateMsg = "OK";
      String        oid         = "";
      String        tfrMemo     = (String)getRequestParameter(request,"reqMemo");
      StringBuffer  strBuf      = null;
      String        tempPhase   = "REDISPLAY";

      try
      {
         //---------------
         // Get a reference to the GtAccountHistory object using the object identifier
         // passed via the ah request parameter.  If object does not exist,
         // it is created in the session cache.
         //
         GtAccountHistory ah = (GtAccountHistory)
                               createInstance(ifsSession,
                                              "com.ibm.ifs.gold.auto.GtAccountHistory",
                                              "ah");
         try
         {
            //---------------
            // Set the fields of the GtAccountHistory object instance from the
            // corresponding request parameters values in the HTTP request.
            //
            setFields(ifsSession,"ah",ah);
         }
         catch (Exception e)
         {
            debug(response, "setFields has thrown an exception" + e.toString());
            throw e;
         }

         //---------------
         // Get the object ID of the cached GtAccountHistory instance object - ah.
         //
         oid = getOid(ifsSession,ah);

         //---------------
         // Normally the validation methods are registered in the initIfs method of
         // IfsSystem class.  Check to see if a "validate" method has been registered for
         // GtAccountHistory transaction.
         //
         if (!ah.hasRegisteredMethod("validate"))
         {
            // No validation method was registered for GtAccountHistory. Register the
            // validateQIF method from QIFValidator.java for the "validate" method.
            try
            {
               debug(response,"Registering validate message in QIFConfirmation class. ");
               ah.registerMethod("validate",
                                 QIFValidator.class.getMethod("validateQIF",
                                                               new Class[]{GtAccountHistory.class}));
            }
            catch (Exception e)
            {
               debug(response,"Exception occured while registering the validate method in QIFConfirmation class.");
            }
         }

         //---------------
         // Validate the input fields. If any errors are found, go back to the
         // form page to correct the errors.
         //
         try
         {
            validateMsg = ah.validate();

            if ( (validateMsg != null) && (validateMsg.equals("OK") == false) )
            {
               // Validation of input failed.  Redisplay the input form with the
               // error message from the validation routine.
               strBuf = new StringBuffer();
               strBuf.append("QIFDownLoadForm?phase=REDISPLAY&ah=");
               strBuf.append(oid);
               strBuf.append("&errMsg=");
               strBuf.append(URLEncoder.encode(validateMsg));
               callPage(response,strBuf.toString());
            }
         }
         catch (Exception e)
         {
            debug(response, "After validation block: exception = " + e.toString());
            throw e;
         }
      }
      catch (Throwable e)
      {
         strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doGet method of QIFDownLoadForm Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }

      //---------------
      // Start creating the HTML data for this servlet and add a copyright statement.
      //
      out.println("<HTML> <HEAD>");
      addCopyright(response);

      //---------------
      // Create HTML page title data.
      //
      out.println("<TITLE>");
      out.println("WebBank Interactive Banking: FILE DOWNLOAD - QIF DOWNLOAD OF ACCOUNT INFORMATION");
      out.println("</TITLE>");
      out.println("<META NAME=\"Author\" VALUE=\"IBM Corporation\">");
      out.println("<META NAME=\"Application\" VALUE=\"Customizable Web Server Example Pages\">");
      out.println("</HEAD>");

      //---------------
      // Start the page body and add background color, webank logo, banner....
      //
      out.println("<BODY BGCOLOR=\"#FFFFFF\" BACKGROUND=\"\">");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
      out.println("<TR><TD><IMG SRC=/images/3dlogo.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
      out.println("<BR>");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0>");
      out.println("<TR><TD><IMG SRC=/images/3dbanner.gif ALIGN=TOP BORDER=0></TD></TR></TABLE>");
      out.println("<BR>");

      //---------------
      // Display standard title and instructions
      //
      out.println("<P><TABLE WIDTH=100% BORDER=0 CELLSPACING=0 CELLPADDING=0><TR>");
      out.println("<TD VALIGN=BASELINE ALIGN=CENTER><FONT SIZE=+2 COLOR=#000000>FILE DOWNLOAD - ");
      out.println("</FONT><FONT SIZE=+2 COLOR=#000080>QIF DOWNLOAD OF ACCOUNT INFORMATION</FONT></TD>");
      out.println("</TR></TABLE>");

      out.println("<IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=CENTER><BR>");
      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=3 CELLPADDING=2>");
      out.println("<TR> <TD WIDTH=90% BGCOLOR=#8EB5F0>&nbsp;</TD>");
      out.println("</TR></TABLE>");

      out.println("<TABLE WIDTH=100% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
      out.println("<TR><TD WIDTH=100% VALIGN=TOP><FONT>");
      out.println("The account information has been successfully formatted. To proceed with the download, click on <B>OK</B>. Otherwise, click on <B>Cancel</B>.");
      out.println("</FONT></TD></TR></TABLE>");

      out.println("<BR>NOTE : If OK is selected, a message will appear asking how to proceed with the download of the QIF information. In order to create the file, select the Save File option. The message disappears when the download is complete.");
      out.println("<BR><BR>After the information has been saved, select the <B>QIF_down_load_form</B> link to return to the QIF input form page, or select the <B>EXIT</B> link to log off.<BR><BR>");

      //---------------
      // Display OK button and if clicked display the down load success page.
      // The hidden name/value passes the object id of the cached object to the
      // down load success page.
      //
      out.println("<CENTER> <TABLE Width=30% BORDER=0 CELLSPACING=2 CELLPADDING=0>");
      out.println("<TR><FORM METHOD=post ACTION=\"QIFDownLoadSuccess\">");
      out.println("<TD WIDTH=50% ALIGN=CENTER>");
      out.println("<INPUT TYPE=HIDDEN NAME=ah VALUE=\"" + oid + "\">");
      out.println("<INPUT TYPE=image BORDER=0 NAME=ok SRC=/images/ok.gif WIDTH=85 HEIGHT=25>");
      out.println("</TD></FORM>");

      //---------------
      // Display CANCEL button and if clicked display the user input form.
      //
      out.println("<FORM METHOD=post ACTION=\"QIFDownLoadForm\">");
      out.println("<TD WIDTH=50% ALIGN=CENTER>");
      out.println("<INPUT TYPE=HIDDEN NAME=ah VALUE=\"" + oid + "\">");
      out.println("<INPUT TYPE=HIDDEN NAME=phase VALUE=\""+ tempPhase + "\">");
      out.println("<INPUT TYPE=image BORDER=0 NAME=cancel SRC=/images/cancel.gif WIDTH=85 HEIGHT=25>");
      out.println("</TD> </FORM>");

      out.println("</TR> </TABLE> </CENTER>");

      //---------------
      // Display Footer
      //
      out.println("<P><IMG SRC=/images/hr_2.gif WIDTH=100% HEIGHT=3 ALIGN=BOTTOM>");
      out.println("<P ALIGN=CENTER><FONT SIZE=-2>");
      out.println("<A HREF=QIFDownLoadForm?phase=" + tempPhase + "&ah=" +
                  oid + ">QIF down load form</A> | ");
      out.println("<A HREF=Logoff>EXIT</A></FONT></P>");

      //---------------
      // Done with the HTML
      //
      out.println("</BODY> </HTML>");

   } // end-of, doPost method


   //#################################################
   //             doGet
   //
   // We perform the same operations for Get request as for POST requests,
   // so this method just sends the request to the doPost method.
   //#################################################
   public void doGet(HttpServletRequest request, HttpServletResponse response)
   throws ServletException, IOException
   {
      try
      {
         doPost(request,response);
      }
      catch (Exception e)
      {
         StringBuffer strBuf = new StringBuffer("ErrorPage?Exception=");
         strBuf.append(URLEncoder.encode("Exception caught in doGet method of QIFConfirmation Servlet:"));
         strBuf.append(URLEncoder.encode(e.toString()));
         callPage(response,strBuf.toString());
      }
   } // end-of, doGet method

} // end-of, QIFConfirmation class
