/*
 * QIFValidator.java
 *
 * Licensed Materials - Property of IBM
 * 5648-A06
 * (c) Copyright IBM Corporation, 1996-1999. All rights reserved.
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM
 *
 */

//#################################################
//               QIFValidator
//
// The class defines static methods which are registered by IfsObjects to
// perform special validate functionality.
//
//#################################################
import com.ibm.ifs.gold.GsAccount;
import com.ibm.ifs.gold.GsDateFormat;
import com.ibm.ifs.gold.auto.GtAccountHistory;

public class QIFValidator
{
   // Use this flag to debug this servlet
   public static boolean debugOn = false;

   //#################################################
   //             validateQIF
   //
   //  This is an example validation method for GtAccountHistory transaction when
   //  doing a QIF download. Add or remove validation checks from the example as
   //  business rules require.
   //
   //  To use this method, register it using an IfsObject object with the
   //  GtAccountHistory class by the name of "validate".  Then invoke the validate
   //  method using the GtAccountHistory instance object (see QIFConfirmation.java).
   //
   //  This example makes the following validation checks:
   //
   //  1. When a date range is required, assure the starting date is less than or equal
   //     to the ending date.
   //  2. When an ending amount is specified a starting amount must also be specified
   //     and the starting amount must be less than or equal to ending amount.
   //  3. Starting check number must be less than or equal to ending check number.
   //
   //#################################################
   public static String validateQIF(GtAccountHistory ah)
   {
      StringBuffer strBuf = new StringBuffer("");
      GsAccount account = ah.getReqAccount();

      try
      {
         //---------------
         // Validate the starting and ending dates when the product type needs a
         // date range.
         //
         String reqPostingStartDate = "";
         String reqPostingEndDate = "";

         if ( (ah.getReqProductType() == null) || (ah.getReqProductType().equals("") ) )
         {
            try
            {
               // Using toString pattern of yyyy/MM/dd so we can do date compares.
               //
               reqPostingStartDate = ah.getReqPostingStartDate().toString(new GsDateFormat("yyyy/MM/dd"));
               if (reqPostingStartDate == null)
                  reqPostingStartDate = "";
            }
            catch (Exception e)
            {
               ah.debug("Exception getting Start Date in validateQIF - " + e.toString());
               reqPostingStartDate = "";
            }

            try
            {
               // Using toString pattern of yyyy/MM/dd so we can do date compares.
               //
               reqPostingEndDate = ah.getReqPostingEndDate().toString(new GsDateFormat("yyyy/MM/dd"));
               if (reqPostingEndDate == null)
                  reqPostingEndDate = "";
            }
            catch (Exception e)
            {
               ah.debug("Exception getting End Date in validateQIF - " + e.toString());
               reqPostingEndDate = "";
            }

            // Check for a valid starting date and/or a valid date range.
            //
            if (reqPostingStartDate.equals("") )
               strBuf.append("<BR>*** Select a time frame or enter a valid starting date or date range.");

            else if ( (reqPostingEndDate.equals("") == false ) &&
                      (reqPostingStartDate.compareTo(reqPostingEndDate) > 0) )
               strBuf.append("<BR>*** Starting date must be less then or equal to ending date.");
         }

         //---------------
         // Check for a valid amount range when an amount range is specified.  Rules are:
         // When ending amount is specified a starting amount must also be specified
         // and starting amount must be less than or equal to ending amount.
         //
         int lowAmountLength = 0;
         double dLowAmount = 0;

         int highAmountLength = 0;
         double dHighAmount = 0;

         // get the low amount information.
         try
         {
            if ( ah.getReqLowAmount() != null )
            {
               lowAmountLength = ah.getReqLowAmount().getDigits().length();
               dLowAmount = ah.getReqLowAmount().getNumeric();
            }
         }
         catch (Exception e)
         {
            ah.debug("validateQIF method lowAmount exception - " + e.toString());
            dLowAmount = 0;
         }

         // get the high amount information.
         try
         {
            if ( ah.getReqHighAmount() != null )
            {
               highAmountLength = ah.getReqHighAmount().getDigits().length();
               dHighAmount = ah.getReqHighAmount().getNumeric();
            }
         }
         catch (Exception e)
         {
            ah.debug("validateQIF method highAmount exception - " + e.toString());
            dHighAmount = 0;
         }

         // Create validate error message when amounts are invalid.
         //
         if ( (lowAmountLength == 0) && (highAmountLength > 0) )
            strBuf.append("<BR>*** Must specify starting amount when ending amount is specified.");

         else if ( (highAmountLength > 0) && (dLowAmount > dHighAmount ) )
            strBuf.append("<BR>*** Starting amount must be less then or equal to ending amount.");

         //---------------
         // Check for a valid check number range when check numbers are specified.
         // Rule is: Starting check number must be less than or
         // equal to ending check number.
         //
         try
         {
            if ( (ah.getReqHighCheckNumber() > 0) &&
                 (ah.getReqLowCheckNumber() > ah.getReqHighCheckNumber() ) )
               strBuf.append("<BR>*** Starting check number must be less then or equal to ending check number.");
         }
         catch (Exception e)
         {
            ah.debug("validateQIF method check number exception - " + e.toString());
         }
      }
      catch (Exception e)
      {
         ah.debug("Exception occurred in validateQIF method of QIFValidator class" + e.toString() );
      }

      // When no validation error messages are generated, return an "OK" string.
      //
      if (strBuf.toString().equals(""))
         strBuf.append("OK");
      else
         ah.debug("Validation failed in validateQIF method of QIFValidator class");

      return strBuf.toString();

   } // end-of, validateQIF method

} // end-of, QIFValidator class
