package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GsCurrency.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.
//    03/29/2000  ftc   d6545   Added NLS support, Javadocs
//    05/16/2000  ftc   d6717   Improved NLS Javadocs
//    05/25/2000  rhs   d6837   Added constructor which takes an IfsSession object.
//    11/07/2001  bjb   d7364   Fixed toString() method so it will compile
//                              with a JDK 1.3 compiler
//    02/05/2002  pdl   d7430   Fix calls to deprecated APIs
//    03/01/2002  pdl   d7465   Replace GsDecimalFormatSymbols with DecimalFormatSymbols
// ***************************************************************************

import java.math.BigDecimal;
import java.lang.Math;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.text.DecimalFormatSymbols;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import javax.servlet.http.HttpSession;

/**
* The GsCurrency class is used to represent IFS supported currencies.
* The class is NLS enabled. Currencies may be constructed and formatted
* in the Java supported Locales.  By the use of an associated class,
* GsCurrencyFormat, custom formats can be constructed.
* <P>
* In general, formatting is based on the convention that a particular locale L
* wants to see foriegn currencies formated in locale L's format with the exception
* of the use of the correct currency symbol.
* <P>
* Certain multiple-currency situations require the use of the correct method signatures.
* NLS capabilities and restrictions will be noted in the affected methods documentation.
*
* There are two types of NLS support offered through GsCurrency:
* <UL type = round>
*  <LI>1. If a user wants a GsCurrency object to be formatted according to the
*     current Locale (which may be changed during a session), then constructors
*     should be used that do not pass in a GsCurrencyFormat object. Also, direct
*     setting of the object's format via the setFormat method must be avoided.
*
*   <LI>2. If a user wants a GsCurrency object to be formatted a certain way regardless
*     of the current Locale, then constructors can be used that pass in the GsCurrencyFormat
*     object or the format can be set via the setFormat method.
* </UL>
*/

public class GsCurrency extends GsCurrencyS implements IComparator
{
   public static boolean debugOn = false;
   public static final int FRONT = 0;
   public static final int DEBIT = 0;
   public static final int CREDIT = 1;
   public static final int NO_DECIMAL = 0;
   public static final int EQUAL = 0;
   public static final int LESS = -1;
   public static final int GREATER = 1;
   public static final int MAX_LENGTH = 15;

   //Total length of all fields is 42
   //15(digit)+12(position)+12(debit)+3(isoCode)
   public static final int TOTAL_FIELDS_LENGTH = 42;
   public static final String CURRENCY_FORMAT = "__CurrencyFormat";
   private static final int CURRENCYSTYLE = 1;
   public static Hashtable registry = new Hashtable();
   private static Hashtable locales = new Hashtable();
   private String  text;
   private Double numeric;
   private GsCurrencyFormat format;

   /**
   * Null constructor.
   * This constructs a GsCurrency object without meaningful value.
   */
   public GsCurrency() {
     super();
   }

   /**
   * Constructor with an associated IfsSession object.
   * This constructs a GsCurrency object without meaningful value.
   */
   public GsCurrency(IfsSession sess)
   {
      super();
      setIfsSession(sess);
   }

   /**
   * Copy constructor.
   * This constructor can be used to copy an existing GsCurrency object.  If the original
   * object had an associated GsCurrencyFormat object, that is copied as well.
   * <P>
   * @param amount -- A GsCurrency object to be copied
   */
   public GsCurrency(GsCurrency amount) {
     String temp;
     setDigits((null==(temp = amount.getDigits())) ? null : new String(temp));
     setPrecision(amount.getPrecision());
     setDebitCredit(amount.getDebitCredit());
     setIsoCode((null==(temp = amount.getIsoCode())) ? null : new String(temp));
     setIfsSession(amount.getIfsSession());
     if (amount.format != null)
     {
       //specific format being used, call format copy constructor
       format = (GsCurrencyFormat) amount.format.clone();
     }
     return;
   }
   /**
   * GMJO/GMOB string constructor.
   * The alias parameter is not currently used, it is there for future consideration.
   * Auto-gen requires the parameter.
   * <P>
   * @param transaction -- an IFS transaction object
   * @param alias -- not used
   */
   public GsCurrency(IfsTxn transaction, String alias) {
     super(transaction);
     setIfsSession(transaction.getIfsSession());
     return;
   }
   /**
   * Gold data constructor.
   * Creates a GsCurrency object with a formatter (that is, a GsCurrencyFormat object)
   * based on the current locale and the isoCode.  The table below shows the result of a
   * toString() method of a GsCurrency object contructed with:
   * <pre>
   *       GsCurrency("0000123456789", 2, 1, isoCode, ifsSession)
   *
   *     current locale      isocode         toString()
   *    ----------------------------------------------------
   *     English (US)         USD          $1,234,567.89
   *     English (US)         FRF          F1,234,567.89
   *     French (France)      USD           1 234 567,89 $
   *     French (France)      FRF           1 234 567,89 F
   * </pre>
   * <P>
   * @param digits -- a String of digits without punctuation
   * @param precision -- number of decimal places to the right of the decimal point
   * @param debitCredit -- the logical sign of the value, 0 = debit, negative, 1 = credit, positive.
   * @param isoCode -- the three character String ISO currency code
   * @param session -- an IFS session object
   */
   public GsCurrency(String digits, int precision, int debitCredit, String isoCode, IfsSession session) {
     setDigits(digits);
     setPrecision(precision);
     setDebitCredit(debitCredit);
     setIsoCode(isoCode);
     setIfsSession(session);
     return;
   }
   /**
   * String constructor.
   * The String, amount, must be compliant with the current locale's currency pattern.
   * This means that only the current locales currency can be created with this contructor.
   * The use of the currency symbol is optional.  The table below shows the toString() of
   * some GsCurrency objects made with this constructor:
   *
   * <pre>
   *     current locale         amount         toString()
   *    ----------------------------------------------------
   *     English (US)       $1,234,567.89    $1,234,567.89
   *     English (US)        1,234,567.89    $1,234,567.89
   *     French (France)     1 234 567,89 F   1 234 567,89 F
   *     French (France)     1 234 567,89     1 234 567,89 F
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @param session -- an IFS session object
   */
   public GsCurrency(String amount, IfsSession session) throws InstantiationException {
     try
     {
       GsCurrency currency = GsCurrency.fromString(amount, session);
       if (null==currency) throw new InstantiationException("Could not create GsCurrency object from amount string of " + amount);
       setDigits(new String(currency.getDigits()));
       setPrecision(currency.getPrecision());
       setDebitCredit(currency.getDebitCredit());
       setIsoCode(new String(currency.getIsoCode()));
       setIfsSession(session);
       if (currency.format != null)
       {
         //specific format being used, clone it
         format = (GsCurrencyFormat) currency.format.clone();
       }
     }
     catch (Exception e)
     {
       throw new InstantiationException("Could not create GsCurrency object from amount string of " + amount);
     }
     return;
   }
   /**
   * String constructor.
   * The String, amount, must be compliant with the current locale's currency pattern except
   * that the currency symbol correponding to the isocode must be used if a currency symbol is used.
   * The table below shows the toString() of some GsCurrency objects made with this constructor:
   *
   * <pre>
   *     current locale         amount        isocode       toString()
   *    -----------------------------------------------------------------
   *     English (US)       $1,234,567.89      USD       $1,234,567.89
   *     English (US)        1,234,567.89      USD       $1,234,567.89
   *     English (US)       F1,234,567.89      FRF       F1,234,567.89
   *     English (US)        1,234,567.89      FRF       F1,234,567.89
   *     French (France)     1 234 567,89 $    USD        1 234 567,89 $
   *     French (France)     1 234 567,89      USD        1 234 567,89 $
   *     French (France)     1 234 567,89 F    FRF        1 234 567,89 F
   *     French (France)     1 234 567,89      FRF        1 234 567,89 F
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @param session -- an IFS session object
   * @param isoCode -- the three character String ISO currency code
   */
   public GsCurrency(String amount, IfsSession session, String isoCode) throws InstantiationException {
     try
     {
       GsCurrency currency = GsCurrency.fromString(amount, session, isoCode);
       if (null==currency) throw new InstantiationException("Could not create GsCurrency object from amount string of " + amount);
       setDigits(new String(currency.getDigits()));
       setPrecision(currency.getPrecision());
       setDebitCredit(currency.getDebitCredit());
       setIsoCode(new String(currency.getIsoCode()));
       setIfsSession(session);
       if (currency.format != null)
       {
         //specific format being used, clone it
         format = (GsCurrencyFormat) currency.format.clone();
       }
     }
     catch (Exception e)
     {
       throw new InstantiationException("Could not create GsCurrency object from amount string of " + amount);
     }
     return;
   }
   /**
   * addLocale adds an ISOcode/Locale pair to the GsCurrency locales table.  This table
   * is used to get formating information from a locale when only the ISO code is known.
   * <P>
   * @param isoCode -- the three character String ISO currency code
   * @param locale -- a Locale object that has the ISO code as it's currency
   */
   public void addLocale(String isoCode, Locale locale) {
     getLocales().put(isoCode, locale);
     return;
   }
 /**
 * private method to alter a currency format pattern to have new currency
 * symbol.
 * <P>
 * @return a new String of the altered pattern
 * @param locale -- a Locale object
 */
 private String alterPattern(Locale locale, String currencySymbol)

{
   GsCurrencyFormat tempFormat = new GsCurrencyFormat(locale);
   DecimalFormatSymbols sym = tempFormat.getDecimalFormatSymbols(); //d7465
   String pattern = ResourceBundle.getBundle("java.text.resources.LocaleElements",locale).
                                         getStringArray("NumberPatterns")[CURRENCYSTYLE];

   // alter pattern to have correct currency symbol
   int endIndex;
   int startIndex;
   for (int i = 0; i < 2; i++) //expect 2, one for positive and negative
   {
      endIndex = pattern.indexOf(sym.getCurrencySymbol());
      if(endIndex != -1)
      {
         startIndex = 0;
         StringBuffer tempPattern = new StringBuffer();
         tempPattern.append(pattern.substring(startIndex, endIndex));
         tempPattern.append(currencySymbol);
         startIndex = endIndex + sym.getCurrencySymbol().length();
         tempPattern.append(pattern.substring(startIndex, pattern.length()));
         pattern = tempPattern.toString();
      }
   }
   return pattern;
}
 /**
 * private method to alter a currency format if necessary
 * <P>
 * @param locale -- a Locale object
 * @param isoCode -- the three char ISO Currency Code
 */
 private GsCurrencyFormat alterFormat(GsCurrencyFormat format, Locale locale, String isoCode)
 {
    GsCurrencyFormat tempFormat = format;
    DecimalFormatSymbols sym = tempFormat.getDecimalFormatSymbols(); //d7465
    if (!sym.getInternationalCurrencySymbol().equals(isoCode))
    {
       if (isoCode.equals("EUR")) //special for EURO
       {
        Locale euroLoc = locale;
        if (locale.getVariant().indexOf("EURO") < 0)
        {
          String var = locale.getVariant();
          var = (var.length() > 0) ?
            (var + "_EURO") : "EURO";
          euroLoc = new Locale(locale.getLanguage(),
                          locale.getCountry(),
                          var);
        }
        tempFormat = new GsCurrencyFormat(euroLoc);
        sym = tempFormat.getDecimalFormatSymbols();      //d7465
        if (!sym.getCurrencySymbol().equals("\u20ac"))
        { //alter it
          tempFormat.setMinimumFractionDigits(2);
          tempFormat.setMaximumFractionDigits(2);
          sym.setCurrencySymbol("\u20ac"); // euro
          tempFormat.setDecimalFormatSymbols(sym);       //d7465
          //d7465 - moved applyPattern below setDecimalFormatSymbols
          tempFormat.applyPattern(alterPattern(euroLoc, "\u20ac")); 
        }
       } // if EUR
       else  // use reference Locale, if found, to alter current locale
       {
        Locale refLocale = (Locale) getLocales().get(isoCode);
        if (refLocale != null)
        {
          GsCurrencyFormat refFormat = new GsCurrencyFormat(refLocale);
          DecimalFormatSymbols refSym = refFormat.getDecimalFormatSymbols();     //d7465
          tempFormat.setMinimumFractionDigits(refFormat.getMinimumFractionDigits());
          tempFormat.setMaximumFractionDigits(refFormat.getMaximumFractionDigits());
          sym.setCurrencySymbol(refSym.getCurrencySymbol());
          sym.setInternationalCurrencySymbol(refSym.getInternationalCurrencySymbol());
          tempFormat.setDecimalFormatSymbols(sym);                               //d7465
          //d7465 - moved applyPattern below setDecimalFormatSymbols
          tempFormat.applyPattern(alterPattern(locale, refSym.getCurrencySymbol()));

        }
       }
    }
    return tempFormat;
 }
   /**
   * asSelfDefiningFieldDef returns GsGmSelfDefiningFieldDef object corresponding
   * to a GsCurrency.
   * <P>
   * @return A GsGmSelfDefiningFieldDef object
   */
   public static GsGmSelfDefiningFieldDef asSelfDefiningFieldDef() {
     GsGmSelfDefiningFieldDef definition = new GsGmSelfDefiningFieldDef();
     definition.setDataType(SD_TYPE_CHAR);
     definition.setLength(TOTAL_FIELDS_LENGTH);
     return definition;
   }
   /**
   * compare compares two GsCurrency objects. Comparision is without consideration
   * of the particular currency type and is not intended for multi-currency comparison.
   * <P>
   * @param currency1 -- a GsCurrency object
   * @param currency2 -- another GsCurrency object
   * @return 0 if equal, -1 if the c1 < c2, 1 if c1 > c2
   */
   public int compare(Object currency1, Object currency2) {
     double myValue, otherValue;
     if ((currency1 instanceof GsCurrency) && (currency2 instanceof GsCurrency))
     {
       return((myValue=((GsCurrency)currency1).getNumeric())==(otherValue=((GsCurrency)currency2).getNumeric())) ?
       EQUAL :
       (myValue<otherValue) ? LESS : GREATER;
     }
     return EQUAL;
   }
   /**
   * fromSelfDefiningFieldDef returns a GsCurrency object derived from the supplied
   * GsGmSelfDefiningFieldDef parameter.
   * <P>
   * @param definition -- a GsGmSelfDefiningFieldDef object
   * @return a GsCurrency object
   */
   public static GsCurrency fromSelfDefiningFieldDef(GsGmSelfDefiningFieldDef definition) {
     GsCurrency currency = new GsCurrency();
     currency.setDigits(IfsApi.trim(definition.getSdChar().substring(0,15),' '));
     currency.setPrecision(new Integer(IfsApi.chop(definition.getSdChar().substring(15,27),'0')).intValue());
     currency.setDebitCredit(new Integer(IfsApi.chop(definition.getSdChar().substring(27,39),'0')).intValue());
     currency.setIsoCode(IfsApi.chop(definition.getSdChar().substring(39,42),' '));
     return currency;
   }
   /**
   * This fromString method constructs a GsCurrency object from the amount using the
   * default server locale.
   * The String, amount, must be compliant with the default server locale's currency pattern.
   * Omission of the currency symbol is only allowed for USD currency.
   * The table below shows the toString() of some GsCurrency objects made with this method:
   *
   * <pre>
   *     default server locale         amount        toString()
   *    -----------------------------------------------------
   *     English (US)              $1,234,567.89    $1,234,567.89
   *     English (US)               1,234,567.89    $1,234,567.89
   *     French (France)            1 234 567,89 F   1 234 567,89 F
   *     French (France)            1 234 567,89     1 234 567,89 F
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @return a GsCurrency object
   */
   public static GsCurrency fromString(String amount) throws Exception {
     GsCurrency currency = new GsCurrency();
     try
     {
       if (currency.hasRegisteredMethod("fromString"))
       {
         return(GsCurrency) currency.invokeMethod("fromString", new Object[] { amount});

       }
       else
       {
         GsCurrencyFormat tempFormat = currency.getFormat();
         ParsePosition position = new ParsePosition(0);
         Number number = tempFormat.parse(amount, position);
         if (null!=number)
         {
            currency.setNumber(number);
            currency.setIsoCode(tempFormat.getDecimalFormatSymbols().getInternationalCurrencySymbol());  //d7465
         }
         else
            currency = null;

       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }
     return currency;
   }
   /**
   * This fromString method constructs a GsCurrency object from the amount using the
   * locale that is associated with the supplied GsCurrencyFormat.
   * The String, amount, must be compliant with the locale associated with the supplied
   * GsCurrencyFormat parameter.
   * Omission of the currency symbol is only allowed for USD currency.
   *
   * <pre>
   *     default server locale      format locale       amount        toString()
   *    -----------------------------------------------------------------------
   *     English (US)              English (US)      $1,234,567.89    $1,234,567.89
   *     English (US)              French (France)    1 234 567,89 F   1 234 567,89 F
   *     French (France)           English (US)      $1,234,567.89    $1,234,567.89
   *     French (France)           French (France)    1 234 567,89     1 234 567,89 F
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @param format -- a GsCurrencyFormat object created with the desired Locale
   * @return a GsCurrency object
   */
   public static GsCurrency fromString(String amount, GsCurrencyFormat format) throws Exception {
     GsCurrency currency = new GsCurrency();
     currency.format = format;
     try
     {
       if (currency.hasRegisteredMethod("fromStringWithFormat"))
       {
         return(GsCurrency) currency.invokeMethod("fromStringWithFormat", new Object[] { amount, format});

       }
       else
       {
         ParsePosition position = new ParsePosition(0);
         Number number = format.parse(amount, position);
         if (null!=number)
         {
            currency.setNumber(number);
            currency.setIsoCode(format.getDecimalFormatSymbols().getInternationalCurrencySymbol());  //d7465
         }
         else
            currency = null;
       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }
     return currency;
   }
   /**
   * This fromString method constructs a GsCurrency object from the amount using the
   * locale that has been currently set for the session.
   * The String, amount, must be compliant with that locale's currency pattern.
   * Omission of the currency symbol is only allowed for USD currency.
   * The table below shows the toString() of some GsCurrency objects made with this method:
   *
   * <pre>
   *     current session locale         amount        toString()
   *    -----------------------------------------------------
   *     English (US)              $1,234,567.89    $1,234,567.89
   *     English (US)               1,234,567.89    $1,234,567.89
   *     French (France)            1 234 567,89 F   1 234 567,89 F
   *     French (France)            1 234 567,89     1 234 567,89 F
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @param session -- an IFS session object
   * @return a GsCurrency object
   */
   public static GsCurrency fromString(String amount, IfsSession session) throws Exception {
     GsCurrency currency = new GsCurrency();
     currency.setIfsSession(session);
     try
     {
       if (currency.hasRegisteredMethod("fromStringWithSession"))
       {
         return(GsCurrency) currency.invokeMethod("fromStringWithSession", new Object[] { amount, session});

       }
       else
       {
         GsCurrencyFormat tempFormat = currency.getFormat();
         ParsePosition position = new ParsePosition(0);
         Number number = tempFormat.parse(amount, position);
         if (null!=number)
         {
            currency.setNumber(number);
            currency.setIsoCode(tempFormat.getDecimalFormatSymbols().getInternationalCurrencySymbol());  //d7465
         }
         else
            currency = null;

       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }
     return currency;
   }
   /**
   * This fromString method constructs a GsCurrency object from the amount using the
   * locale that has been currently set for the session but allowing other currencies to
   * be constructed as specified by the ISO code.
   * The String, amount, must be compliant with that locale's currency pattern.
   * Omission of the currency symbol is only allowed for USD currency.
   * The table below shows the toString() of some GsCurrency objects made with this method:
   *
   * <pre>
   *     current session locale         amount    isocode     toString()
   *    -------------------------------------------------------------
   *     English (US)              $1,234,567.89    USD     $1,234,567.89
   *     English (US)              F1,234,567.89    FRF     F1,234,567.89
   *     French (France)            1 234 567,89 F  FRF      1 234 567,89 F
   *     French (France)            1 234 567,89 $  USD      1 234 567,89 $
   * </pre>
   * <P>
   * @param amount -- a String representing the amount, punctuation is optional
   * @param session -- an IFS session object
   * @param isoCode -- the three character String ISO currency code
   * @return a GsCurrency object
   */
   public static GsCurrency fromString(String amount, IfsSession session, String isoCode) throws Exception {
     GsCurrency currency = new GsCurrency();
     currency.setIfsSession(session);
     currency.setIsoCode(isoCode);
     try
     {
       if (currency.hasRegisteredMethod("fromStringWithSessionWithIsoCode"))
       {
         return(GsCurrency) currency.invokeMethod("fromStringWithSessionWithIsoCode", new Object[] { amount, session, isoCode});

       }
       else
       {
         ParsePosition position = new ParsePosition(0);
         GsCurrencyFormat format = currency.getFormat();
         Number number = format.parse(amount, position);

         if (null!=number)
         {
            currency.setNumber(number);
         }
         else
            currency = null;

       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }
     return currency;
   }
   /**
   * getFormat returns the GsCurrencyFormat object currently associated with this
   * GsCurrency.  If one is not yet set, one is calculated based on the current Locale
   * and the ISO code if one is available.
   * <P>
   * @return a GsCurrencyFormat object
   */
   public GsCurrencyFormat getFormat()
   {
      if (format == null)
      {
         GsCurrencyFormat tempFormat = null;
         Locale tempLocale = getLocale();
         String isoCode = getIsoCode();

         Hashtable isoLocales = null;
         GsCurrencyFormat isoFormat = null;
         String isoCurrencySymbol = null;
         char localeDecimalSeparator = 0;
         int isoFlag = -1;
         if (isoCode != null)
         {
            isoLocales = getLocales();
            Locale isoLocale = (Locale) isoLocales.get(isoCode);
            if(isoLocale != null) {
                isoFormat = new GsCurrencyFormat(isoLocale);
                isoCurrencySymbol = isoFormat.getDecimalFormatSymbols().getCurrencySymbol(); //d7465

            }
            else if(!("EUR".equalsIgnoreCase(isoCode)))
                isoCode = null;

         }

         if (getIfsSession() != null)
         {
            // build cache key
            String cacheKey = CURRENCY_FORMAT;
            if (isoCode != null)
            {
               cacheKey = cacheKey + "_" + isoCode;
            }
            cacheKey = cacheKey + "_" + tempLocale.toString();

            // check for format object in cache
            tempFormat = (GsCurrencyFormat)getHttpSession().getAttribute(cacheKey);
            if (tempFormat == null)
            {
               tempFormat = new GsCurrencyFormat(tempLocale);
               if (isoCode != null)
               {
                  localeDecimalSeparator = tempFormat.getDecimalFormatSymbols().getMonetaryDecimalSeparator();  //d7465
                  try {
                        isoFlag = isoCurrencySymbol.indexOf(localeDecimalSeparator);
                  }
                  catch (Throwable e) { debug("isoFlag is " + isoFlag);}

                  if( -1 == isoFlag )
                        tempFormat = alterFormat(tempFormat, tempLocale, isoCode);
                  else
                        tempFormat = isoFormat;

               }
               getHttpSession().setAttribute(cacheKey, tempFormat);
            }
         }
         else  // IfsSession is null
         {
            tempFormat = new GsCurrencyFormat(tempLocale);
            if (isoCode != null)
            {
                  localeDecimalSeparator = tempFormat.getDecimalFormatSymbols().getMonetaryDecimalSeparator();
                  try {
                        isoFlag = isoCurrencySymbol.indexOf(localeDecimalSeparator);
                  }
                  catch (Throwable e) { debug("isoFlag is " + isoFlag);}

                  if( -1 == isoFlag )
                        tempFormat = alterFormat(tempFormat, tempLocale, isoCode);

                  else
                        tempFormat = isoFormat;

            }
         }
         return tempFormat;
      }
      return format;
   }
   /**
   * getLocales returns a Hashtable of ISOcode/Locale pairs.  This table
   * is used to get formating information from a locale when only the ISO code is known.
   * <P>
   * @return a Hashtable of ISOcode/Locale pairs
   */
   public Hashtable getLocales() {
     if (locales.isEmpty())
     {
       locales.put("ALL", new Locale("sq","AL"));
       locales.put("ARS", new Locale("es","AR"));
       locales.put("ATS", new Locale("de","AT"));
       locales.put("AUD", new Locale("en","AU"));
       locales.put("BEF", new Locale("fr","BE"));
       locales.put("BGL", new Locale("bg","BG"));
       locales.put("BOB", new Locale("es","BO"));
       locales.put("BRL", new Locale("pt","BR"));
       locales.put("BYB", new Locale("be","BY"));
       locales.put("CAD", new Locale("en","CA"));
       locales.put("CHF", new Locale("de","CH"));
       locales.put("CLP", new Locale("es","CL"));
       locales.put("CNY", new Locale("zh","CN"));
       locales.put("COP", new Locale("es","CO"));
       locales.put("CRC", new Locale("es","CR"));
       locales.put("CZK", new Locale("cs","CZ"));
       locales.put("DEM", new Locale("de","DE"));
       locales.put("DKK", new Locale("da","DK"));
       locales.put("DOP", new Locale("es","DO"));
       locales.put("ECS", new Locale("es","EC"));
       locales.put("EEK", new Locale("et","EE"));
       locales.put("EGP", new Locale("ar","EG"));
       locales.put("ESP", new Locale("es","ES"));
       locales.put("FIM", new Locale("fi","FI"));
       locales.put("FRF", new Locale("fr","FR"));
       locales.put("GBP", new Locale("en","GB"));
       locales.put("GRD", new Locale("el","GR"));
       locales.put("GTQ", new Locale("es","GT"));
       locales.put("HNL", new Locale("es","HN"));
       locales.put("HRK", new Locale("hr","HR"));
       locales.put("HUF", new Locale("hu","HU"));
       locales.put("IEP", new Locale("en","IE"));
       locales.put("ISK", new Locale("is","IS"));
       locales.put("ITL", new Locale("it","IT"));
       locales.put("JPY", new Locale("ja","JP"));
       locales.put("KRW", new Locale("ko","KR"));
       locales.put("LTL", new Locale("lt","LT"));
       locales.put("LUF", new Locale("fr","LU"));
       locales.put("LVL", new Locale("lv","LV"));
       locales.put("MKD", new Locale("mk","MK"));
       locales.put("MXN", new Locale("es","MX"));
       locales.put("NIO", new Locale("es","NI"));
       locales.put("NIS", new Locale("iw","IL"));
       locales.put("NLG", new Locale("nl","NL"));
       locales.put("NOK", new Locale("no","NO"));
       locales.put("NZD", new Locale("en","NZ"));
       locales.put("PAB", new Locale("es","PA"));
       locales.put("PEN", new Locale("es","PE"));
       locales.put("PLZ", new Locale("pl","PL"));
       locales.put("PTE", new Locale("pt","PT"));
       locales.put("PYG", new Locale("es","PY"));
       locales.put("ROL", new Locale("ro","RO"));
       locales.put("RUR", new Locale("ru","RU"));
       locales.put("SEK", new Locale("sv","SE"));
       locales.put("SIT", new Locale("sl","SI"));
       locales.put("SKK", new Locale("sk","SK"));
       locales.put("SVC", new Locale("es","SV"));
       locales.put("THB", new Locale("th","TH"));
       locales.put("TRL", new Locale("tr","TR"));
       locales.put("TWD", new Locale("zh","TW"));
       locales.put("UAH", new Locale("uk","UA"));
       locales.put("USD", new Locale("en","US"));
       locales.put("UYU", new Locale("es","UY"));
       locales.put("VEB", new Locale("es","VE"));
       locales.put("YUN", new Locale("sh","YU"));
       locales.put("ZAR", new Locale("en","ZA"));
     }
     return locales;
   }
   /**
   * Converts this GsCurrency object to a double and returns it.
   * The double value can be used as
   * input to a sort routine to sort multiple decimal data objects.
   * <P>
   * @return a double representing the value of the currency
   */
   public double getNumeric() throws NumberFormatException {
     double value;
     try
     {
       if (null == numeric)
         numeric = Double.valueOf(getDigits());

       value = numeric.doubleValue() / Math.pow(10.0, getPrecision());
     }
     catch (NumberFormatException e)
     {
       throw e;
     }
     if (DEBIT == getDebitCredit())
       value = -value;

     return value;
   }
   /**
   * getRegistry returns the Hashtable used to register user method overides
   * for GsCurrency.
   * <P>
   * @return a Hashtable used to register user method overides
   */
   public static Hashtable getRegistry() {
     return registry;
   }
   /**
   * setDebitCredit sets the sign indicator for the amount.
   * <P>
   * @param debitCredit -- the logical sign of the value, 0 = debit, negative, 1 = credit, positive.
   */
   public void setDebitCredit(int debitCredit) {
     super.setDebitCredit(debitCredit);
     text = null;
     numeric = null;
   }
   /**
   * setDigits sets the digits String. It must be digits only without punctuation.
   * <P>
   * @param digits -- a String of pure digits
   */
   public void setDigits(String digits) {
     super.setDigits(digits);
     text = null;
     numeric = null;
   }
   /**
   * setFormat assigns the GsCurrencyFormat object currently associated with the
   * supplied GsCurrency.
   * <P>
   * @param format -- a GsCurrencyFormat object created with the desired Locale
   * @param currency -- A GsCurrency object to set a format with
   */
   public static void setFormat(GsCurrencyFormat format, GsCurrency currency) {

        currency.format = format;
        return;
   }
   /**
   * private method to set the number for this GsCurrency object
   * @param number -- an input Number to to derive settings from
   */
   private void setNumber(Number number) {

     String tempString = Double.toString(number.doubleValue());
     GsCurrencyFormat tempFormat = getFormat();
     int decimalPlace=0;

     //If tempString value is negative, the first character will be MinusSign
     int i;
     setDebitCredit((tempString.startsWith(String.valueOf(tempFormat.getDecimalFormatSymbols().getMinusSign()))) ? DEBIT : CREDIT);

     if ((i = tempString.indexOf(tempFormat.getDecimalFormatSymbols().getDecimalSeparator())) == -1) //Set the precision based on decimal point
       setPrecision(NO_DECIMAL);

     else if ((1==(tempString.length() - i - 1))&&tempString.endsWith(String.valueOf(tempFormat.getDecimalFormatSymbols().getZeroDigit())))
     {
       //If value like ##.0, don't count as precision.
       setPrecision(NO_DECIMAL);
       tempString = tempString.substring(0,tempString.length() - 1);

     }
     else
     { //Check to ensure that tempString is not exponetial, i.e. 1.234567890E7
       StringTokenizer token = new StringTokenizer(tempString, "E");
       if (token.countTokens() == 2)
       {
         //String is exponetial.  Need to flatten
         tempString = token.nextToken();
         decimalPlace = Integer.parseInt(token.nextToken());
         setPrecision(tempString.length() - i - 1 - decimalPlace);
       }
       else
         setPrecision(tempString.length() - i - 1);

       //Make sure there are enough digits to satisfy the exponetial digits.  Add enough zeros to digits to
       //account for the exponents.  If precision is negative add the zeros after the digits, if precision is
       //positive add the zeros before the digits.
       if (Math.abs(getPrecision()) > tempString.length() - 1)
       {
         StringBuffer buff = null;
         int zerosToAdd = Math.abs(getPrecision());
         if (getPrecision() < 0)
            buff = new StringBuffer(tempString);

         else
            buff = new StringBuffer();

         for (i = 0; i < zerosToAdd; i++)
         {
            buff.append(tempFormat.getDecimalFormatSymbols().getZeroDigit());
         }
         if (getPrecision() > 0)
            buff.append(tempString);

         else
            setPrecision(0);

         tempString = buff.toString();

       }
     }

     int minimumFraction = getFormat().getMinimumFractionDigits();
     int maximumFraction = getFormat().getMaximumFractionDigits();
     int scale = 0;

     BigDecimal bigDecimal = new BigDecimal(tempString);
     bigDecimal = bigDecimal.movePointRight(decimalPlace);
     decimalPlace=0;

     if ( getPrecision() <= minimumFraction ) {
          scale = minimumFraction;
          setPrecision(minimumFraction);
     }
     else if ( getPrecision() > minimumFraction && getPrecision() < maximumFraction )
          scale = getPrecision();

     else if ( getPrecision() >= maximumFraction ) {
          scale = maximumFraction; 
          setPrecision(maximumFraction);
     }

     bigDecimal = bigDecimal.setScale(scale,BigDecimal.ROUND_DOWN);
     tempString = bigDecimal.toString();

     String tempDigits = "";
     StringBuffer delimeters = new StringBuffer();
     delimeters.append(tempFormat.getDecimalFormatSymbols().getMinusSign());
     delimeters.append(tempFormat.getDecimalFormatSymbols().getGroupingSeparator());
     delimeters.append(tempFormat.getDecimalFormatSymbols().getDecimalSeparator());

     StringTokenizer tokens = new StringTokenizer(tempString, delimeters.toString());
     while (tokens.hasMoreTokens())
     {
       tempDigits = tempDigits.concat(tokens.nextToken());
     }

     setDigits(tempDigits);

   }
   /**
   * setPrecision sets the precision indicator for the GsCurrency object.
   * <P>
   * @param precision -- number of decimal places to the right of the decimal point
   */
   public void setPrecision(int precision) {
     super.setPrecision(precision);
     text = null;
     numeric = null;
   }
   /**
   * toSelfDefiningFieldDef returns a GsGmSelfDefiningFieldDef object representing
   * this GsCurrency.
   * <P>
   * @return a GsGmSelfDefiningFieldDef object
   */
   public GsGmSelfDefiningFieldDef toSelfDefiningFieldDef() {
     GsGmSelfDefiningFieldDef definition = new GsGmSelfDefiningFieldDef();
     definition.setDataType(SD_TYPE_CURRENCY);
     definition.setLength(TOTAL_FIELDS_LENGTH);
     StringBuffer buffer = new StringBuffer(TOTAL_FIELDS_LENGTH);
     buffer.append(IfsApi.left(digits,15,' '));
     buffer.append(IfsApi.right(Integer.toString(precision),12,'0'));
     buffer.append(IfsApi.right(Integer.toString(debitCredit),12,'0'));
     buffer.append(IfsApi.right(isoCode,3,' '));
     definition.setSdChar(buffer.toString());
     return definition;
   }
   /**
   * toString returns a String formatting the GsCurrency with the current format.
   * <P>
   * @return a String of the formatted currency
   */
   public String toString() {
     String result = super.toString();
     if (result == null) {
	 text = getFormat().format(getDigits(), 
                                   getPrecision(),
                                   getDebitCredit(),
                                   new StringBuffer(MAX_LENGTH),
                                   new FieldPosition(FRONT)).toString();
	 result = text;
     }
     return result;
   }
   /**
   * toString returns a String formatting the GsCurrency with the specified format.
   * <P>
   * @return a String of the formatted currency
   */
   public String toString(GsCurrencyFormat format) {
     String superResult;
     return(null != (superResult=super.toString(format))) ? //if method has been registered, invoke registered method
     superResult : //registered method return value
     format.format(getDigits() //not registered, format according to provided format object.
               ,getPrecision()
               ,getDebitCredit()
               ,new StringBuffer(MAX_LENGTH)
               ,new FieldPosition(FRONT)).toString();
   }
   /**
   * toString returns a String formatting the GsCurrency with the current format
   * and appending the specified prefix to the String.
   * <P>
   * @param prefix -- a String prefix
   * @return a String of the formatted currency
   */
   public String toString(String prefix) {
     if (hasRegisteredMethod("toStringWithString"))
       return super.toString(prefix);

     StringBuffer results = new StringBuffer(200);
     results.append(super.toString(prefix));
     results.append(prefix);
     results.append("| digits = ");
     results.append(getDigits());
     results.append(prefix);
     results.append("| precision = ");
     results.append(Integer.toString(getPrecision()));
     results.append(prefix);
     results.append("| debitCredit = ");
     results.append(Integer.toString(getDebitCredit()));
     results.append(prefix);
     results.append("| text = ");
     results.append(toString());
     results.append(prefix);
     results.append("| numeric = ");
     results.append(Double.toString(getNumeric()));
     return results.toString();
   }
   /**
   * Default validation of the currency digits.  Default validation consists of:
   *
   * Making sure the digit field is not null, empty, or contains any spaces.
   * Checking that the digit field is less than or equal to IfsSystem.MAX_CURRENCY_VALUE
   * and that it is greater than or equal to IfsSystem.MIN_CURRENCY_VALUE.
   * <P>
   * @return "OK" if the IfsObject is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate() {
     String tempDigits;
     ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",
                                          getLocale());
     String str = super.validate();
     if (null!=str)
       return str;

     return(null==(tempDigits = getDigits())) ? //get Currency Digits
     bundle.getString("GsCurrency.defaultValidation.noDigits") : //no digits
     (tempDigits.length()==0) ? //have digits, see if empty
     bundle.getString("GsCurrency.defaultValidation.emptyDigits") :  //empty digits
     (getNumeric() < Double.valueOf(IfsSystem.MIN_CURRENCY_VALUE).doubleValue()) ? //less than min?
     bundle.getString("GsCurrency.defaultValidation.minimumValue") : //digits less than min
     (getNumeric() > Double.valueOf(IfsSystem.MAX_CURRENCY_VALUE).doubleValue()) ? //greater than max?
     bundle.getString("GsCurrency.defaultValidation.maximumValue") : //digits greater than max
     "OK"; //digits are valid

   }
}
