package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GsCurrencyFormat.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.
//    03/29/2000  ftc   d6545   Added NLS support, Javadocs
//    03/01/2002  pdl   d7465   Replace GsDecimalFormatSymbols with DecimalFormatSymbols
//
// ***************************************************************************

import java.awt.Color;
import java.awt.Font;
import java.lang.Math;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

/**
* A GsCurrencyFormat object can be associated with a GsCurrency object to  
* define the patterns and symbols used during parsing and formatting.
* A GsCurrencyFormat object has a DecimalFormatSymbols Object to define symbols. 
* <P>
* In general, formatting is based on the convention that a particular locale L  
* wants to see foriegn currencies formated in locale L's format with the exception
* of the use of the correct currency symbol.
* <P>
* Certain multiple-currency situations require the use of the correct method signatures. 
* NLS capabilities and restrictions will be noted in the affected methods documentation. 
*/
public class GsCurrencyFormat extends GsDecimalFormat
{
	public static boolean debugOn = false;
   private static final int CURRENCYSTYLE = 1;

   protected GsCurrencyFormat(){}   
   /**  
   * This GsCurrencyFormat constructor takes a currency pattern. Reference Java
   * documentation for the syntax of DecimalFormat patterns. Although the supplied
   * pattern will be applied, this method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the method:
   *            GsCurrencyFormat(String pattern, Locale locale)
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   */
   public GsCurrencyFormat(String pattern) {
	  this(pattern, IfsSystem.SERVER_LOCALE);
	  return;
   }   
   /**
   * This GsCurrencyFormat constructor takes a currency pattern and a
   * DecimalFormatSymbols object which can be made locale specific. Reference Java
   * documentation for the syntax of DecimalFormat patterns. 
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   * @param symbols -- a DecimalFormatSymbols object
   *
   */
   public GsCurrencyFormat(String pattern, DecimalFormatSymbols symbols) {
	  // Always applyPattern after the symbols are set
	  setDecimalFormatSymbols(symbols);                        //d7465
	  applyPattern(pattern);
	  setInputNegativeChars(IfsSystem.DEBIT_SIGN);
	  setRemoveTrailingFractionZeros(IfsSystem.REMOVE_TRAILING_FRACTION_ZEROES);
	  setFormatStyle(IfsSystem.CURRENCY_STYLE);
	  setPositiveFont(IfsSystem.CURRENCY_CREDIT_FONT);
	  setNegativeFont(IfsSystem.CURRENCY_DEBIT_FONT);
	  setPositiveColor(IfsSystem.CURRENCY_CREDIT_COLOR);
	  setNegativeColor(IfsSystem.CURRENCY_DEBIT_COLOR);

	  return;
   }   
   /**  
   * This GsCurrencyFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class. 
   * This method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the similar method that supports locale.
   * <P>
   * @param isoCode -- the three character ISO currency code                   
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   */
   public GsCurrencyFormat(
						  String isoCode,
						  String positivePrefix,
						  String positiveSuffix,
						  String negativePrefix,
						  String negativeSuffix,
						  String inputNegativeChars,
						  char groupingSeparator,
						  int groupingSize,
						  char decimalSeparator,
						  boolean decimalSeparatorAlwaysShown,
						  int minimumIntegerDigits,
						  int minimumFractionDigits,
						  int maximumFractionDigits,
						  boolean removeTrailingFractionZeros)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols();  //d7465
	  symbols.setInternationalCurrencySymbol(isoCode);
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(IfsSystem.CURRENCY_STYLE);
	  setPositiveFont(IfsSystem.CURRENCY_CREDIT_FONT);
	  setNegativeFont(IfsSystem.CURRENCY_DEBIT_FONT);
	  setPositiveColor(IfsSystem.CURRENCY_CREDIT_COLOR);
	  setNegativeColor(IfsSystem.CURRENCY_DEBIT_COLOR);
          setDecimalFormatSymbols(symbols);                      //d7465
	  return;
   }   
   /**  
   * This GsCurrencyFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class plus some other formatting options. 
   * This method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the similar method that supports locale.
   * <P>
   * @param isoCode -- the three character ISO currency code                   
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param formatStyle --  text = 0, html = 1
   * @param positiveFont -- a Font object for positive values 
   * @param negativeFont -- a Font object for negative values
   * @param positiveColor -- a Color object for positive values
   * @param negativeColor -- a Color object for negative values
   */
   public GsCurrencyFormat(
						  String isoCode,
						  String positivePrefix,
						  String positiveSuffix,
						  String negativePrefix,
						  String negativeSuffix,
						  String inputNegativeChars,
						  char groupingSeparator,
						  int groupingSize,
						  char decimalSeparator,
						  boolean decimalSeparatorAlwaysShown,
						  int minimumIntegerDigits,
						  int minimumFractionDigits,
						  int maximumFractionDigits,
						  boolean removeTrailingFractionZeros,
						  int formatStyle,
						  Font positiveFont,
						  Font negativeFont,
						  Color positiveColor,
						  Color negativeColor)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols();  //d7465
	  symbols.setInternationalCurrencySymbol(isoCode);
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(formatStyle);
	  setPositiveFont(positiveFont);
	  setNegativeFont(negativeFont);
	  setPositiveColor(positiveColor);
	  setNegativeColor(negativeColor);
          setDecimalFormatSymbols(symbols);                      //d7465

	  return;
   }   
   /**  
   * This GsCurrencyFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class plus some other formatting options. 
   * This method also accepts a Locale to accomodate NLS formatting.
   * <P>
   * @param isoCode -- the three character ISO currency code                   
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param formatStyle --  text = 0, html = 1
   * @param positiveFont -- a Font object for positive values 
   * @param negativeFont -- a Font object for negative values
   * @param positiveColor -- a Color object for positive values
   * @param negativeColor -- a Color object for negative values
   * @param locale -- a Locale object for formatting
   */
   public GsCurrencyFormat(
						  String isoCode,
						  String positivePrefix,
						  String positiveSuffix,
						  String negativePrefix,
						  String negativeSuffix,
						  String inputNegativeChars,
						  char groupingSeparator,
						  int groupingSize,
						  char decimalSeparator,
						  boolean decimalSeparatorAlwaysShown,
						  int minimumIntegerDigits,
						  int minimumFractionDigits,
						  int maximumFractionDigits,
						  boolean removeTrailingFractionZeros,
						  int formatStyle,
						  Font positiveFont,
						  Font negativeFont,
						  Color positiveColor,
						  Color negativeColor,
						  Locale locale)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);  //d7465
	  symbols.setInternationalCurrencySymbol(isoCode);
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(formatStyle);
	  setPositiveFont(positiveFont);
	  setNegativeFont(negativeFont);
	  setPositiveColor(positiveColor);
	  setNegativeColor(negativeColor);
          setDecimalFormatSymbols(symbols);                      //d7465

	  return;
   }   
   /**  
   * This GsCurrencyFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class. 
   * This method also accepts a Locale to accomodate NLS formatting.
   * <P>
   * @param isoCode -- the three character ISO currency code                   
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param locale -- a Locale object for formatting
   */
   public GsCurrencyFormat(
						  String isoCode,
						  String positivePrefix,
						  String positiveSuffix,
						  String negativePrefix,
						  String negativeSuffix,
						  String inputNegativeChars,
						  char groupingSeparator,
						  int groupingSize,
						  char decimalSeparator,
						  boolean decimalSeparatorAlwaysShown,
						  int minimumIntegerDigits,
						  int minimumFractionDigits,
						  int maximumFractionDigits,
						  boolean removeTrailingFractionZeros,
						  Locale locale)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale); //d7465
	  symbols.setInternationalCurrencySymbol(isoCode);
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(IfsSystem.CURRENCY_STYLE);
	  setPositiveFont(IfsSystem.CURRENCY_CREDIT_FONT);
	  setNegativeFont(IfsSystem.CURRENCY_DEBIT_FONT);
	  setPositiveColor(IfsSystem.CURRENCY_CREDIT_COLOR);
	  setNegativeColor(IfsSystem.CURRENCY_DEBIT_COLOR);
          setDecimalFormatSymbols(symbols);                      //d7465

	  return;
   }   
   /**
   * This GsCurrencyFormat constructor takes a currency pattern and a
   * Locale object used to create the DecimalFormatSymbols. Reference Java
   * documentation for the syntax of DecimalFormat patterns. 
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   * @param locale -- a Locale object for formatting
   */
   public GsCurrencyFormat(String pattern, Locale locale) {

	  // Always applyPattern after the symbols are set
	  setDecimalFormatSymbols(new DecimalFormatSymbols(locale));  //d7465
	  applyPattern(pattern);
	  setInputNegativeChars(IfsSystem.DEBIT_SIGN);
	  setRemoveTrailingFractionZeros(IfsSystem.REMOVE_TRAILING_FRACTION_ZEROES);
	  setFormatStyle(IfsSystem.CURRENCY_STYLE);
	  setPositiveFont(IfsSystem.CURRENCY_CREDIT_FONT);
	  setNegativeFont(IfsSystem.CURRENCY_DEBIT_FONT);
	  setPositiveColor(IfsSystem.CURRENCY_CREDIT_COLOR);
	  setNegativeColor(IfsSystem.CURRENCY_DEBIT_COLOR);

	  return;
   }      
   /**
   * This GsCurrencyFormat constructor takes a Locale object to get 
   * a pattern and create the DecimalFormatSymbols. 
   * <P>
   * @param localeInstance -- a Locale object for formatting
   */
   public GsCurrencyFormat(Locale localeInstance) {
      
	  this(ResourceBundle.getBundle("java.text.resources.LocaleElements",localeInstance).
		   getStringArray("NumberPatterns")[CURRENCYSTYLE],localeInstance);
	  return;
   }   
   /**
   * a protected method for html formatting
   */
   protected StringBuffer toHTML(StringBuffer buffer, int debitCredit) {
	  StringBuffer htmlBuffer = new StringBuffer(50);

	  if (debitCredit==GsCurrency.DEBIT)
	  {
		 htmlBuffer = addStyle(htmlBuffer,getNegativeFont(), getNegativeColor());
		 htmlBuffer.append(buffer.toString());
		 htmlBuffer = addEndStyle(htmlBuffer,getNegativeFont(),getNegativeColor());
	  }
	  else
	  {
		 htmlBuffer = addStyle(htmlBuffer,getPositiveFont(), getPositiveColor());
		 htmlBuffer.append(buffer.toString());
		 htmlBuffer = addEndStyle(htmlBuffer,getPositiveFont(),getPositiveColor());
	  }
	  return htmlBuffer;
   }   
}
