package com.ibm.ifs.gold;

//+------------------------------------------------------------------------+
//|                                                                        |
//|  FILE NAME:   GsDate.java                                              |
//|                                                                        |
//|  COPYRIGHT:                                                            |
//|                                                                        |
//|   Licensed Materials - Property of IBM                                 |
//|   5648-A06                                                             |
//|   (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved               |
//|    US Government Users Restricted Rights - Use, duplication            |
//|    or disclosure restricted by GSA ADP Schedule Contract               |
//|    with IBM                                                            |
//|                                                                        |
//|  CHANGES:                                                              |
//|    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.        |
//|    04/11/2000  rhs   d6590   NLS changes and Javadoc additions.        |
//|    05/25/2000  rhs   d6837   Added constructor which takes an IfsSession object.
//|
//+------------------------------------------------------------------------+
import java.util.Hashtable;
import java.util.Calendar;
import java.text.ParsePosition;
import java.util.Locale;
import java.util.Date;
import java.text.FieldPosition;
import java.util.ResourceBundle;

/**
* GsDate class represents the date in a Gold format (USA, ISO, EUR, JIS). The class
* also provides methods such as toString and toString(format) to display
* dates in locale-sensitive manner.
*
* <pre>
* Gold Pattern     Gold Format     Example
* ------------     -----------     -------
* MM/dd/yyyy       "USA"           04/20/1999
* dd.MM.yyyy       "EUR"           20.04.1999
* yyyy-MM-dd       "ISO","JIS"     1999-04-20
*
* </pre>
* @see          java.util.Calendar
* @see          java.text.DateFormat
* @see          java.text.DateFormatSymbols
* @see          java.text.SimpleDateFormat
* @see          GsDateFormat
*/
public class GsDate extends GsDateS implements IComparator
{
   //--------------------------------------------------------------------------
   // Attributes
   //--------------------------------------------------------------------------
   public static boolean debugOn = false;
   private static Hashtable registry = new Hashtable();
   private GsDateFormat localeFormatter;
   private GsDateFormat goldFormatter;

   /**
   *
   */
   public static int TOTAL_FIELDS_LENGTH = 13;

   /**
   * The yyyyMMdd pattern which is suitable for sorting of dates.
   */
   public static final String CANONICAL_PATTERN = "yyyyMMdd";

   /**
   * Default constructor
   */
   public GsDate()
   {
   }

   /**
   * Constructor with an associated IfsSession object.
   */
   public GsDate(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * GMJO/GMOB string constructor
   */
   public GsDate(IfsTxn txn)
   {
      super(txn);
      setIfsSession(txn.getIfsSession());
   }

   /**
   * GMJO/GMOB string constructor.
   * @param alias -- This parameter is not currently used,
   * it is there for future consideration.  Auto-gen requires the parameter.
   */
   public GsDate(IfsTxn txn, String alias)
   {
      this(txn);
   }

   /*
   * Gold Data constructor
   */
   public GsDate(String goldDate, String goldDatePattern, IfsSession session)
   {
      setIfsSession(session);
      String pattern = toGoldPattern(IfsSystem.GOLD_DATE_FORMAT);
      if (pattern != null)
      {
         setGoldFormatter(new GsDateFormat(pattern));
         try
         {
            GsDateFormat formatter = new GsDateFormat(goldDatePattern);
            Date parseDate = formatter.parse(goldDate);
            if (parseDate != null)
            {
               date = getGoldFormatter().format(parseDate);
               format = IfsSystem.GOLD_DATE_FORMAT;
            }
         }
         catch (Exception e)
         {

         }
      }
   }

   public GsDate(String goldDate, String goldDatePattern, String goldDateFormat, IfsSession session) // Gold Data constructor
   {
      setIfsSession(session);
      String pattern = toGoldPattern(goldDateFormat);
      if (pattern != null)
      {
         setGoldFormatter(new GsDateFormat(pattern));
         try
         {
            GsDateFormat formatter = new GsDateFormat(goldDatePattern);
            Date parseDate = formatter.parse(goldDate);
            if (parseDate != null)
            {
               date = getGoldFormatter().format(parseDate);
               format = goldDateFormat;
            }
         }
         catch (Exception e)
         {

         }
      }
   }

   /**
   * Compares this Gold date object with another Gold date object for equality.
   * @param d -- date to be compared with.
   * @return
   *    Returns true when they are equal, false otherwise.
   */
   public boolean equals(GsDate d)
   {
      if (date != null)
      {
         if (IfsSystem.IS_SYNCHRONIZE_DATE_REQUIRED) {
            synchronized(getDate()) {
               return date.equals(d.getDate());
            }
         }
         else
            return date.equals(d.getDate());
      }
      return false;
   }

   /**
   * Converts a date to GsDate object using GsDateFormat argument.
   * @param dateToConvert -- the date to be converted.
   * @param dateFormatter -- the GsDateFormat to convert the date.
   * @return
   *    The GsDate object which was created from the date.
   */
   private static GsDate fromDate(Date dateToConvert, GsDateFormat dateFormatter)
   {
      GsDate gsDate = new GsDate();
      String pattern = dateFormatter.toPattern();
      String format = fromGoldPattern(pattern);

      if (format == null) // check for gold pattern
      {
         format = IfsSystem.GOLD_DATE_FORMAT;
         pattern = toGoldPattern(format);
         dateFormatter = new GsDateFormat(pattern);
      }
      gsDate.setGoldFormatter(dateFormatter);
      gsDate.setDate(dateFormatter.format(dateToConvert));
      gsDate.setFormat(format);

      return gsDate;
   }

   /**
   * Converts a gold date pattern to gold date format string.
   */
   public static String fromGoldPattern(String goldPattern)
   {
      String goldFormat = null;
      if (goldPattern.equals("yyyy-MM-dd"))
      {
         goldFormat = "ISO";
      }
      else if (goldPattern.equals("dd.MM.yyyy"))
      {
         goldFormat = "EUR";
      }
      else if (goldPattern.equals("MM/dd/yyyy"))
      {
         goldFormat = "USA";
      }
      return goldFormat;
   }

   /**
   * Converts text string to GsDate object. It calls the registered fromString
   * method to parse the string. If no fromString method is registered,
   * it does default parsing by using the GoldFormatter attribute to parse the text.
   */
   public static GsDate fromString(String text) throws Exception
   {
      GsDate d = new GsDate();
      if (d.hasRegisteredMethod("fromString"))
      {
         try
         {
            return(GsDate)d.invokeMethod("fromString",new Object[]{text});
         }
         catch (Exception e)
         {
            if (e.getMessage().equals("Method not registered."))
            {
               return null;
            }
            else
            {
               throw e;
            }
         }
      }
      else
      {
         GsDateFormat formatter = d.getGoldFormatter();
         formatter.setLenient(false);
         Date parseDate = formatter.parse(text,new ParsePosition(0));
         if (parseDate != null)
         {
            d = fromDate(parseDate,formatter);
         }
         return d;
      }
   }

   /**
   * Converts text string to GsDate object. It calls the registered fromStringWithFormat
   * method to parse the string. If no fromStringWithFormat method is registered,
   * it uses the myFormat argument to parse the text.
   */
   public static GsDate fromString(String text, GsDateFormat myFormat) throws Exception
   {
      GsDate d = new GsDate();
      d.setLocaleFormatter(myFormat);
      if (d.hasRegisteredMethod("fromStringWithFormat"))
      {
         try
         {
            return(GsDate)d.invokeMethod("fromStringWithFormat",new Object[]{text,myFormat});
         }
         catch (Exception e)
         {
            if (e.getMessage().equals("Method not registered."))
            {
               return null;
            }
            else
            {
               throw e;
            }
         }
      }
      else
      {
         Date parseDate = myFormat.parse(text,new ParsePosition(0));
         if (parseDate != null)
         {
            d = fromDate(parseDate,myFormat);
         }
      }
      return d;
   }

   /**
   * Returns the day of the month. It is a number in the range 1-31, inclusive.
   */
   public String getDay()
   {
      String day = null;
      if (date != null)
      {
         String pattern = getGoldFormatter().toPattern();
         int firstIndex, lastIndex;

         if (pattern.indexOf('-') > -1)
         {
            lastIndex = date.lastIndexOf('-');        // gold format is "ISO" or "JIS"
            day = date.substring(lastIndex+1);           // dd of yyyy-MM-dd
         }
         else if (pattern.indexOf('/') > -1)
         {
            firstIndex = date.indexOf('/');           // gold format is "USA"
            lastIndex = date.lastIndexOf('/');
            day = date.substring(firstIndex+1,lastIndex);// dd of MM/dd/yyyy
         }
         else if (pattern.indexOf('.') > -1)
         {
            firstIndex = date.indexOf('.');           // gold format is "EUR"
            day = date.substring(0,firstIndex);       // dd of dd.MM.yyyy
         }
      }
      return day;
   }

   /**
   * Returns the goldFormatter attribute.
   * When the goldFormatter attribute is null, the format attribute is used to set the
   * goldFormatter attribute and it is then returned.
   * If the format attribute is also null, the IfsSystem.GOLD_DATE_FORMAT is used to set the
   * goldFormatter attribute and it is then returned.
   */
   public GsDateFormat getGoldFormatter()
   {
      if (goldFormatter == null)
      {
         if (format != null)
         {
            setGoldFormatter(new GsDateFormat(toGoldPattern(format)));
         }
         else
         {
            setGoldFormatter(new GsDateFormat(toGoldPattern(IfsSystem.GOLD_DATE_FORMAT)));
         }
      }
      return goldFormatter;
   }

   /**
   * Returns localeFormatter attribute.
   * When localeFormatter attribute is null and a session exists, the session
   * locale is used to create the localeFormatter.
   * Otherwise, the server locale is used to create the localeFormatter.
   */
   public GsDateFormat getLocaleFormatter()
   {
      if (localeFormatter == null)
      {
         GsDateFormat tempFormat;
         if (getIfsSession() != null)
         {
            tempFormat = new GsDateFormat(getIfsSession().getLocale() );
         }
         else
         {
            tempFormat = new GsDateFormat();
         }
         return tempFormat;
      }
      return localeFormatter;
   }

   /**
   * Returns the month of the year. It is a number in the range 1-12, inclusive.
   */
   public String getMonth()
   {
      String month = null;
      if (date != null)
      {
         String pattern = getGoldFormatter().toPattern();
         int firstIndex, lastIndex;

         if (pattern.indexOf('-') > -1)
         {
            firstIndex = date.indexOf('-');           // gold format is "ISO" or "JIS"
            lastIndex = date.lastIndexOf('-');
            month=date.substring(firstIndex+1,lastIndex);// MM of yyyy-MM-dd
         }
         else if (pattern.indexOf('.') > -1)
         {
            firstIndex = date.indexOf('.');           // gold format is "EUR"
            lastIndex = date.lastIndexOf('.');
            month=date.substring(firstIndex+1,lastIndex);// MM of dd.MM.yyyy
         }
         else if (pattern.indexOf('/') > -1)
         {
            firstIndex = date.indexOf('/');           // gold format is "USA"
            month=date.substring(0,firstIndex);       // MM of MM/dd/yyyy
         }
      }
      return month;
   }

   /**
   * Returns registry attribute that contains registered methods and its names mapping.
   */
   public static Hashtable getRegistry()
   {
      return registry;
   }

   /**
   * Returns the year. It is a number in the format of YYYY.
   */
   public String getYear()
   {
      String year = null;
      if (date != null)
      {
         String pattern = getGoldFormatter().toPattern();
         int index;

         if (pattern.indexOf('-') > -1) // "ISO" or "JIS"?
         {
            index = date.indexOf('-');
            year = date.substring(0,index);// yyyy of yyyy-MM-dd
         }
         else if (pattern.indexOf('/') >-1) //"USA"?
         {
            index = date.lastIndexOf('/');
            year = date.substring(index+1); // yyyy of MM/dd/yyyy
         }
         else if (pattern.indexOf('.') > -1) // "EUR"?
         {
            index = date.lastIndexOf('.');
            year = date.substring(index+1); // yyyy of dd.MM.yyyy
         }
      }
      return year;
   }

   /**
   * Determines whether this date comes after another Gold date.
   */
   public boolean isAfter(GsDate when)
   {
      return Integer.parseInt(toCanonical()) > Integer.parseInt(when.toCanonical());
   }

   /**
   * Determines whether this date comes before another Gold date.
   */
   public boolean isBefore(GsDate when)
   {
      return Integer.parseInt(toCanonical()) < Integer.parseInt(when.toCanonical());
   }

   /**
   * Compares if this date is the same as another date.
   */
   public boolean isEqual(GsDate when)
   {
      return Integer.parseInt(toCanonical()) == Integer.parseInt(when.toCanonical());
   }

   /**
   * Determines whether this date is todays date.
   */
   public boolean isToday()
   {
      GsDateFormat formatter = new GsDateFormat(CANONICAL_PATTERN);
      Date today = new Date();

      if (toCanonical().equals(formatter.format(today)))
      {
         return true;
      }
      return false;
   }
   public void setGoldFormatter(GsDateFormat formatter)
   {
      goldFormatter = formatter;
   }
   public void setLocaleFormatter(GsDateFormat formatter)
   {
      localeFormatter = formatter;
      return;
   }

   /**
   * Returns a string in the canonical pattern (yyyyMMdd).
   * This method is used mainly for comparision purpose.
   */
   public String toCanonical()
   {
      if (date != null)
      {
         Date myDate = null;
         try
         {
            if (IfsSystem.IS_SYNCHRONIZE_DATE_REQUIRED) {
               synchronized(getDate()) {
                  myDate = getGoldFormatter().parse(getDate());
               }
            }
            else
               myDate = getGoldFormatter().parse(getDate());
         }
         catch (Exception e)
         {
            return null;
         }
         return new GsDateFormat(CANONICAL_PATTERN).format(myDate); //canonical form
      }
      return null;
   }

   /**
   * Converts a gold date format string to gold date pattern.
   */
   public static String toGoldPattern(String goldFormat)
   {
      String pattern = null;
      if (goldFormat.equals("USA"))
      {
         pattern = "MM/dd/yyyy";
      }
      else if (goldFormat.equals("EUR"))
      {
         pattern = "dd.MM.yyyy";
      }
      else if (goldFormat.equals("ISO")||goldFormat.equals("JIS"))
      {
         pattern = "yyyy-MM-dd";
      }
      return pattern;
   }

   /**
   * Converts the GsDate object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, the GsDate object is Locale formatted.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      String str = super.toString();
      if (str == null)
      {
         return(toString(getLocaleFormatter()));
      }
      else
      {
         return str;
      }
   }

   /**
   * Converts the GsDate object to a localized string representation using
   * the pattern specified by <EM>myFormat</EM> parameter.
   * When a method is registered to <EM>toString</EM> for this class, it is used.
   * Otherwise, the default formatting is done with <EM>myFormat</EM> parameter.
   *
   * @param myFormat -- Pattern used to format this object.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(GsDateFormat myFormat)
   {
      String str = super.toString(myFormat);
      if (str != null)
      {
         return str;
      }
      else
      {
         Calendar cal = Calendar.getInstance();
    String localeDate = null;
    try
    {
           cal.clear();
           cal.set(Calendar.DATE, Integer.parseInt(getDay(),10));
           cal.set(Calendar.MONTH, Integer.parseInt(getMonth(),10)-1);
           cal.set(Calendar.YEAR, Integer.parseInt(getYear(),10));
      localeDate = myFormat.format(cal.getTime(), new StringBuffer(), new FieldPosition(0)).toString();
    }
    catch(Exception e)
    {
      localeDate = "";
    }
    return localeDate;
      }
   }

   /**
   * Converts the GsDate object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   * The default conversion prints date and format attributes
   * along with the super class attributes.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
      {
         return super.toString(prefix);
      }
      else
      {

         StringBuffer results = new StringBuffer("");
         results.append(super.toString(prefix));
         results.append(prefix);
         results.append("| date = ");
         results.append(getDate());
         results.append(prefix);
         results.append("| format = ");
         results.append(getFormat());

         return results.toString();
      }
   }

   /**
   * Validates the GsDate object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GsDate is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate()
   {
      String str = super.validate();
      if (str == null) // check if validate method is registered
      {
         boolean isValid = false;
         ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
         try
         {
            isValid = IfsApi.isValidDate(Integer.parseInt(getMonth()), Integer.parseInt(getDay()),Integer.parseInt(getYear()));
            if (!isValid)
            {
               return bundle.getString("GsDate.validation.date");
            }
         }
         catch (Exception e)
         {
            return bundle.getString("GsDate.validation.date");
         }
         return "OK";
      }
      return str;
   }

   /**
   * Compares 2 GsDate objects.
   *
   * @param a -- First GsDate object.
   * @param b -- Second GsDate object.
   * @return
   *    Returns -2 when "a" or "b" are not GsDate objects.
   *    Returns -1 when "a" is before "b".
   *    Returns 0 when "a" equals "b".
   *    Returns 1 when "a" is after "b".
   */
   public int compare(Object a, Object b)
   {
      if ((a instanceof GsDate) && (b instanceof GsDate))
      {
         if (((GsDate)a).isAfter((GsDate)b)) return 1;
         else if (((GsDate)a).isBefore((GsDate)b)) return -1;
         else return 0;
      }
      return -2;
   }

   public static GsGmSelfDefiningFieldDef asSelfDefiningFieldDef() {
      GsGmSelfDefiningFieldDef definition = new GsGmSelfDefiningFieldDef();
      definition.setDataType(SD_TYPE_CHAR);
      definition.setLength(TOTAL_FIELDS_LENGTH);
      return definition;
   }
   public GsGmSelfDefiningFieldDef toSelfDefiningFieldDef() {
      GsGmSelfDefiningFieldDef definition = new GsGmSelfDefiningFieldDef();
      definition.setDataType(SD_TYPE_CHAR);
      definition.setLength(TOTAL_FIELDS_LENGTH);
      StringBuffer buffer = new StringBuffer(TOTAL_FIELDS_LENGTH);
      buffer.append(IfsApi.right(date,10,' '));
      buffer.append(IfsApi.right(format,3,' '));
      definition.setSdChar(buffer.toString());
      return definition;
   }
   public static GsDate fromSelfDefiningFieldDef(GsGmSelfDefiningFieldDef definition) {
      GsDate newDate = new GsDate();
      newDate.setDate(IfsApi.chop(definition.getSdChar().substring(0,10),' '));
      newDate.setFormat(IfsApi.chop(definition.getSdChar().substring(10,13),' '));
      return newDate;
   }
}
