package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GsDecimal.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.
//    03/29/2000  ftc   d6545   Added NLS support, Javadocs
//    05/25/2000  rhs   d6837   Added constructor which takes an IfsSession object.
//    11/06/2001  bjb   d7364   Fixed toString() so it will compile with a
//                              JDK1.3 compiler.
//    02/05/2002  pdl   d7430   Fix calls to deprecated APIs.
//
// ***************************************************************************
import java.lang.Math;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import javax.servlet.http.HttpSession;

/**
* GsDecimal class extends GsDecimalS.  It provides function for the
* construction, formatting and comparing of GsDecimal instances.
*
* There are two types of NLS support offered through GsDecimal:
* <UL type = round>
*  <LI>1. If a user wants a GsDecimal object to be formatted according to the
*     current Locale (which may be changed during a session), then constructors
*     should be used that do not pass in a GsDecimalFormat object. Also, direct
*     setting of the object's format via the setFormat method must be avoided.
*
*   <LI>2. If a user wants a GsDecimal object to be formatted a certain way regardless
*     of the current Locale, then constructors can be used that pass in the GsDecimalFormat
*     object or the format can be set via the setFormat method.
* </UL>
*/

public class GsDecimal extends GsDecimalS implements IComparator
{
   public static boolean debugOn = false;
   public static final int DEBIT = 0;
   public static final int CREDIT = 1;
   public static final int NO_DECIMAL = 0;
   public static final int EQUAL = 0;
   public static final int LESS = -1;
   public static final int GREATER = 1;
   public static final int FORMAT_TEXT = 0;
   public static final int FORMAT_HTML = 1;
   public static final String DECIMAL_FORMAT = "__DecimalFormat";
   public static final int MAX_LENGTH = 15;
   private String  text;
   private Double numeric;
   public GsDecimalFormat format;
   private static Hashtable registry = new Hashtable();

   /**
   * GsDecimal Default constructor. No GsDecimalFormat is associated with the
   * constructed object
   */
   public GsDecimal()
   {
     super();
   }

   /**
   * Constructor with an associated IfsSession object.
   */
   public GsDecimal(IfsSession sess)
   {
      super();
      setIfsSession(sess);
   }

   /**
   * GsDecimal copy constructor to create a GsDecimal from another
   * GsDecimal.  If a GsDecimalFormat is associated, it is copied as well.
   * <P>
   * @param decimal -- an existing GsDecimal to copy
   */
   public GsDecimal(GsDecimal decimal)
   {
     String temp;
     setDigits((null==(temp = decimal.getDigits())) ? null : new String(temp));
     setPrecision(decimal.getPrecision());
     setDebitCredit(decimal.getDebitCredit());
     setIfsSession(decimal.getIfsSession());
     if (decimal.format != null)
     {
       //specific format being used, call format copy constructor
       format = (GsDecimalFormat) decimal.format.clone();
     }
     return;
   }
   /**
   * GsDecimal GMJO/GMOB string constructor to create a GsDecimal from
   * a transactions data. No GsDecimalFormat is associated with the
   * constructed object.
   * <P>
   * @param transaction -- a transaction with the data
   * @param alias -- unused
   */
   public GsDecimal(IfsTxn transaction, String alias)
   {
     super(transaction);
     setIfsSession(transaction.getIfsSession());
     return;
   }
   /**
   * GsDecimal Gold data constructor. No GsDecimalFormat is associated with the
   * constructed object.
   * <P>
   * @param digits -- raw digits of the decimal
   * @param precision -- number of digits to right of decimal point
   * @param debitCredit -- 0 = debit, 1 = credit
   * @param session -- the IfsSession
   */
   public GsDecimal(String digits, int precision, int debitCredit, IfsSession session)
   {
     setDigits(digits);
     setPrecision(precision);
     setDebitCredit(debitCredit);
     setIfsSession(session);
     return;
   }
   /**
   * String constructor.
   * The String, number, must be compliant with the current locale's number pattern.
   * It is allowable, however, to have no or varying amounts of punctuation in the number.
   * This means that the current locales number format will be used for this GsDecimal.
   * The table below shows the toString() of some GsDecimal objects made
   * with this constructor:
   *
   * <pre>
   *     current locale         number         toString()
   *    ----------------------------------------------------
   *     English (US)        1,234,567.89     1,234,567.89
   *     English (US)           123456789     1,234,567.89
   *     French (France)     1 234 567,89     1 234 567,89
   *     French (France)        123456789     1 234 567,89
   * </pre>
   * <P>
   * @param number -- a String representing the number, punctuation is optional
   * @param session -- an IFS session object
   */
   public GsDecimal(String number, IfsSession session) {
     try
     {
       GsDecimal decimal = GsDecimal.fromString(number);
       setDigits(new String(decimal.getDigits()));
       setPrecision(decimal.getPrecision());
       setDebitCredit(decimal.getDebitCredit());
       setIfsSession(session);
       if (decimal.format != null)
       {
         //specific format being used, clone it
         format = (GsDecimalFormat) decimal.format.clone();
       }
     }
     catch (Exception e)
     {
     }
     return;
   }
   /**
   * compare compares two GsDecimal objects.
   * <P>
   * @param currency1 -- a GsDecimal object
   * @param currency2 -- another GsDecimal object
   * @return 0 if equal, -1 if the d1 < d2, 1 if d1 > d2
   */
   public int compare(Object decimal1, Object decimal2) {
     double myValue, otherValue;
     if ((decimal1 instanceof GsDecimal) && (decimal2 instanceof GsDecimal))
     {
       return((myValue=((GsDecimal)decimal1).getNumeric())==(otherValue=((GsDecimal)decimal2).getNumeric())) ?
       EQUAL :
       (myValue<otherValue) ? LESS : GREATER;
     }
     return EQUAL;
   }
   /**
   * Creates a GsDecimal object from a Number object.
   */
   private void setNumber(Number number) {
     String tempString = Double.toString(number.doubleValue());
     GsDecimalFormat tempFormat = getFormat();

     //If tempString value is negative, the first character will be "-"
     int i;
     setDebitCredit((tempString.startsWith(String.valueOf(tempFormat.getDecimalFormatSymbols().getMinusSign()))) ? DEBIT : CREDIT);
     if ((i = tempString.indexOf(tempFormat.getDecimalFormatSymbols().getDecimalSeparator())) == -1) //Set the precision based on decimal point
       setPrecision(NO_DECIMAL);

     else if ((1==(tempString.length() - i - 1))&&tempString.endsWith(String.valueOf(tempFormat.getDecimalFormatSymbols().getZeroDigit())))
     {
       //If value like ##.0, don't count as precision.
       setPrecision(NO_DECIMAL);
       tempString = tempString.substring(0,tempString.length() - 1);

     }
     else
     { //Check to ensure that tempString is not exponetial, i.e. 1.234567890E7
       StringTokenizer token = new StringTokenizer(tempString, "E");
       if (token.countTokens() == 2)
       {
         //String is exponetial.  Need to flatten
         tempString = token.nextToken();
         setPrecision(tempString.length() - i - 1 - Integer.parseInt(token.nextToken()));
       }
       else
         setPrecision(tempString.length() - i - 1);

       //Make sure there are enough digits to satisfy the exponetial digits.  Add enough zeros to digits to
       //account for the exponents.  If precision is negative add the zeros after the digits, if precision is
       //positive add the zeros before the digits.
       if (Math.abs(getPrecision()) > tempString.length() - 1)
       {
         StringBuffer buff = null;
         int zerosToAdd = Math.abs(getPrecision());
         if (getPrecision() < 0)
            buff = new StringBuffer(tempString);

         else
            buff = new StringBuffer();

         for (i = 0; i < zerosToAdd; i++)
         {
            buff.append(String.valueOf(tempFormat.getDecimalFormatSymbols().getZeroDigit()));
         }
         if (getPrecision() > 0)
            buff.append(tempString);

         else
            setPrecision(0);

         tempString = buff.toString();
       }
     }
     String tempDigits = "";
     StringBuffer delimeters = new StringBuffer();
     delimeters.append(tempFormat.getDecimalFormatSymbols().getMinusSign());
     delimeters.append(tempFormat.getDecimalFormatSymbols().getGroupingSeparator());
     delimeters.append(tempFormat.getDecimalFormatSymbols().getDecimalSeparator());

     StringTokenizer tokens = new StringTokenizer(tempString, delimeters.toString());
     while (tokens.hasMoreTokens())
     {
       tempDigits = tempDigits.concat(tokens.nextToken());
     }
     setDigits(tempDigits);
   }
   /**
   * This fromString method constructs a GsDecimal object from the String number using the
   * default server locale.
   * The String, number, must be compliant with the default server locale's number pattern.
   * The table below shows the toString() of some GsDecimal objects made with this method:
   *
   * <pre>
   *     default server locale         amount        toString()
   *    -----------------------------------------------------
   *     English (US)               1,234,567.89     1,234,567.89
   *     English (US)                  123456789     1,234,567.89
   *     French (France)            1 234 567,89     1 234 567,89
   *     French (France)               123456789     1 234 567,89
   * </pre>
   * <P>
   * @param number -- a String representing the number, punctuation is optional
   * @return a GsDecimal object
   */
   public static GsDecimal fromString(String number) throws Exception {
     GsDecimal decimal = new GsDecimal();
     //the user is responsible for setting IfsSession before execution
     try
     {
       if (decimal.hasRegisteredMethod("fromString"))
       {
         return(GsDecimal) decimal.invokeMethod("fromString", new Object[] { number});

       }
       else
       {
         ParsePosition position = new ParsePosition(0);
         Number newNumber = decimal.getFormat().parse(number, position);
         if (null!=newNumber)
            decimal.setNumber(newNumber);
         else
            decimal = null;

       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }
     return decimal;
   }
   /**
   * This fromString method constructs a GsDecimal object from the number using the
   * locale that is associated with the supplied GsDecimalFormat.
   * The String, number, must be compliant with the pattern associated with the supplied
   * GsDecimalFormat parameter.
   * The table below shows the toString() of some GsDecimal objects made with this method:
   *
   * <pre>
   *     default server locale      format locale       amount        toString()
   *    -----------------------------------------------------------------------
   *     English (US)              English (US)       1,234,567.89     1,234,567.89
   *     English (US)              French (France)    1 234 567,89     1 234 567,89
   *     French (France)           English (US)       1,234,567.89     1,234,567.89
   *     French (France)           French (France)    1 234 567,89     1 234 567,89
   * </pre>
   * <P>
   * @param amount -- a String representing the number, punctuation is optional
   * @param format -- a GsDecimalFormat object created with the desired Locale
   * @return a GsCurrency object
   */
   public static GsDecimal fromString(String number, GsDecimalFormat format) throws Exception {
     GsDecimal decimal = new GsDecimal();
     decimal.format = format;
     //the user is responsible for setting IfsSession before execution
     try
     {
       if (decimal.hasRegisteredMethod("fromStringWithFormat"))
       {
         return(GsDecimal) decimal.invokeMethod("fromStringWithFormat", new Object[] { number, format});

       }
       else
       {
         ParsePosition position = new ParsePosition(0);
         Number newNumber = format.parse(number, position);
         if (null!=newNumber)
         {
            decimal.setNumber(newNumber);
         }
         else
            decimal = null;
       }
     }
     catch (Exception e)
     {
       if (e.getMessage().equals("Method not registered."))
         return null;
       else
         throw e;
     }

     return decimal;
   }
   /**
   * getFormat returns the GsDecimalFormat object currently associated with this
   * GsDecimal.  If one is not yet set, one is calculated based on the current Locale.
   * <P>
   * @return a GsDecimalFormat object
   */
   public GsDecimalFormat getFormat()
   {
     if (format == null)
     {
        GsDecimalFormat tempFormat;
        Locale tempLocale = getLocale();

        if (getIfsSession() != null)
        {
           // build cache key
           String cacheKey = DECIMAL_FORMAT;
           cacheKey = cacheKey + "_" + tempLocale.toString();

           // check for format object in cache
           tempFormat = (GsDecimalFormat)getHttpSession().getAttribute(cacheKey);
           if (tempFormat == null)
           {
              tempFormat = new GsDecimalFormat(tempLocale);
              getHttpSession().setAttribute(cacheKey, tempFormat);
           }
        }
        else  // IfsSession is null
        {
           tempFormat = new GsDecimalFormat(tempLocale);
        }
        return tempFormat;
     }
     return format;
   }
   /**
   * getNumeric converts this GsDecimal object to a double and returns it.
   * The double value can be used as
   * input to a sort routine to sort multiple decimal data objects.
   * <P>
   * @return a double representing the number
   */
   public double getNumeric() throws NumberFormatException {
     double value;
     try
     {
       if (null == numeric)
         numeric = Double.valueOf(getDigits());

       value = numeric.doubleValue() / Math.pow(10.0, getPrecision());
     }
     catch (NumberFormatException e)
     {
       throw e;
     }
     if (DEBIT == getDebitCredit())
       value = -value;

     return value;
   }
   /**
   * getRegistry returns the Hashtable used to register user method overides
   * for GsDecimal.
   * <P>
   * @return a Hashtable used to register user method overides
   */
   public static Hashtable getRegistry() {
     return registry;
   }
   /**
   * setDebitCredit sets the sign indicator for the number.
   * <P>
   * @param debitCredit -- the logical sign of the value, 0 = debit, negative, 1 = credit, positive.
   */
   public void setDebitCredit(int debitCredit) {
     super.setDebitCredit(debitCredit);
     text = null;
     numeric = null;
   }
   /**
   * setDigits sets the digits String. It must be digits only, without punctuation.
   * <P>
   * @param digits -- a String of pure digits
   */
   public void setDigits(String digits) {
     super.setDigits(digits);
     text = null;
     numeric = null;
   }
   /**
   * setFormat assigns the GsDecimalFormat object currently associated with the
   * supplied GsDecimal.
   * <P>
   * @param format -- a GsDecimalFormat object created with the desired Locale
   * @param decimal -- A GsDecimal object to set a format with
   */
   public static void setFormat(GsDecimalFormat format, GsDecimal decimal) {
     decimal.format = format;
     return;
   }
   /**
   * setPrecision sets the precision indicator for the GsDecimal object.
   * <P>
   * @param precision -- number of decimal places to the right of the decimal point
   */
   public void setPrecision(int precision) {
     super.setPrecision(precision);
     text = null;
     numeric = null;
   }
   /**
   * toString returns a String formatting the GsDecimal with the current format.
   * <P>
   * @return a String of the formatted currency
   */
   public String toString() {
     String result = super.toString();
     if (result == null) {
	 text = getFormat().format(getDigits(),
                                   getPrecision(),
                                   getDebitCredit(),
                                   new StringBuffer(MAX_LENGTH),
                                   new FieldPosition(0)).toString();
	 result = text;
     }

     return result;
   }
   /**
   * toString returns a String formatting the GsDecimal with the specified format.
   * <P>
   * @return a String of the formatted currency
   */
   public String toString(GsDecimalFormat format) {
     String superResult;

     return(null != (superResult=super.toString(format))) ? //if registered invoke registered method
     superResult : // registered return value
     format.format(getDigits() //not registered, format with provided format object.
               ,getPrecision()
               ,getDebitCredit()
               ,new StringBuffer(MAX_LENGTH)
               ,new FieldPosition(0)).toString();
   }
   /**
   * toString returns a String formatting the GsDecimal with the current format
   * and appending the specified prefix to the String.
   * <P>
   * @param prefix -- a String prefix
   * @return a String of the formatted decimal
   */
   public String toString(String prefix) {
     if (hasRegisteredMethod("toStringWithString"))
       return super.toString(prefix);

     StringBuffer results = new StringBuffer(200);
     results.append(super.toString(prefix));
     results.append(prefix);
     results.append("| digits = ");
     results.append(getDigits());
     results.append(prefix);
     results.append("| precision = ");
     results.append(Integer.toString(getPrecision()));
     results.append(prefix);
     results.append("| debitCredit = ");
     results.append(Integer.toString(getDebitCredit()));
     results.append(prefix);
     results.append("| text = ");
     results.append(toString());
     results.append(prefix);
     results.append("| numeric = ");
     results.append(Double.toString(getNumeric()));
     return results.toString();
   }
   /**
   * Default validation of the currency digits.  Default validation consists of:
   *
   * Making sure the digit field is not null, empty, or contains any spaces.
   * Checking that the digit field is less than or equal to IfsSystem.MAX_DECIMAL_VALUE
   * and that it is greater than or equal to IfsSystem.MIN_DECIMAL_VALUE.
   * <P>
   * @return "OK" if the IfsObject is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate() {
     String tempDigits = null, tempRslts = null;
     ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",
                                          getLocale());
     if (null!=(tempRslts = super.validate()))
       return tempRslts;

     return(null==(tempDigits = getDigits())) ? //get digits
     bundle.getString("GsDecimal.defaultValidation.noDigits") : //no digits available
     (tempDigits.length()==0) ? //have digits, are they empty?
     bundle.getString("GsDecimal.defaultValidation.emptyDigits") :  //digits empty
     (getNumeric() < Double.valueOf(IfsSystem.MIN_DECIMAL_VALUE).doubleValue()) ? //less than min?
     bundle.getString("GsDecimal.defaultValidation.minimumValue") : //nope, less than minimum requirement
     (getNumeric() > Double.valueOf(IfsSystem.MAX_DECIMAL_VALUE).doubleValue()) ? //greater than max?
     bundle.getString("GsDecimal.defaultValidation.maximumValue") : //nope, greater than maximum reqmnt
     "OK"; //digits are good
   }
}
