package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   GsDecimalformat.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.
//    03/29/2000  ftc   d6545   Added NLS support, Javadocs
//    03/01/2002  pdl   d7465   Change GsDecimalFormatSymbols to DecimalFormatSymbols
//
// ***************************************************************************

import java.awt.Color;
import java.awt.Font;
import java.lang.Math;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.FieldPosition;
import java.text.ParsePosition;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

/**
* A GsDecimalFormat object can be associated with a GsDecimal object to  
* define the patterns and symbols used during parsing and formatting.
* A GsDecimalFormat object has a DecimalFormatSymbols Object to define symbols. 
*/
public class GsDecimalFormat extends DecimalFormat
{
	public static boolean debugOn = false;
   public String inputNegativeChars;
   public boolean removeTrailingFractionZeros;
   private int formatStyle;
   private Font positiveFont;
   private Font negativeFont;
   private Color positiveColor;
   private Color negativeColor;
   private static final int NUMBERSTYLE = 0;

   /**
   * not to be called publicly
   */
   protected GsDecimalFormat() {
	  return;
   }   
   /**  
   * This GsDecimalFormat constructor takes a number pattern. Reference Java
   * documentation for the syntax of DecimalFormat patterns. Although the supplied
   * pattern will be applied, this method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the method:
   *            GsDecimalFormat(String pattern, Locale locale)
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   */
   public GsDecimalFormat(String pattern) {
	  this(pattern, IfsSystem.SERVER_LOCALE);
	  return;
   }   
   /**
   * This GsDecimalFormat constructor takes a number pattern and a
   * DecimalFormatSymbols object which can be made locale specific. Reference Java
   * documentation for the syntax of DecimalFormat patterns. 
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   * @param symbols -- a DecimalFormatSymbols object
   *
   */
   public GsDecimalFormat(String pattern, DecimalFormatSymbols symbols) {
	  // Always applyPattern after the symbols are set
	  setDecimalFormatSymbols(symbols);                     //d7465
	  applyPattern(pattern);
	  setInputNegativeChars(IfsSystem.DEBIT_SIGN);
	  setRemoveTrailingFractionZeros(IfsSystem.REMOVE_TRAILING_FRACTION_ZEROES);
	  setFormatStyle(IfsSystem.DECIMAL_STYLE);
	  setPositiveFont(IfsSystem.DECIMAL_POSITIVE_FONT);
	  setNegativeFont(IfsSystem.DECIMAL_NEGATIVE_FONT);
	  setPositiveColor(IfsSystem.DECIMAL_POSITIVE_COLOR);
	  setNegativeColor(IfsSystem.DECIMAL_NEGATIVE_COLOR);
	  return;

   }   
   /**  
   * This GsDecimalFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class. 
   * This method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the similar method that supports locale.
   * <P>
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   */
   public GsDecimalFormat(
						 String positivePrefix,
						 String positiveSuffix,
						 String negativePrefix,
						 String negativeSuffix,
						 String inputNegativeChars,
						 char groupingSeparator,
						 int groupingSize,
						 char decimalSeparator,
						 boolean decimalSeparatorAlwaysShown,
						 int minimumIntegerDigits,
						 int minimumFractionDigits,
						 int maximumFractionDigits,
						 boolean removeTrailingFractionZeros)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols();  //d7465
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(IfsSystem.DECIMAL_STYLE);
	  setPositiveFont(IfsSystem.DECIMAL_POSITIVE_FONT);
	  setNegativeFont(IfsSystem.DECIMAL_NEGATIVE_FONT);
	  setPositiveColor(IfsSystem.DECIMAL_POSITIVE_COLOR);
	  setNegativeColor(IfsSystem.DECIMAL_NEGATIVE_COLOR);
	  setDecimalFormatSymbols(symbols);                           //d7465

	  return;
   }   
   /**  
   * This GsDecimalFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class plus some other formatting options. 
   * This method has to use the default server locale which 
   * can differ from the session's current locale. If a locale is desired that may
   * differ from the server default, use the similar method that supports locale.
   * <P>
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param formatStyle --  text = 0, html = 1
   * @param positiveFont -- a Font object for positive values 
   * @param negativeFont -- a Font object for negative values
   * @param positiveColor -- a Color object for positive values
   * @param negativeColor -- a Color object for negative values
   */
   public GsDecimalFormat(
						 String positivePrefix,
						 String positiveSuffix,
						 String negativePrefix,
						 String negativeSuffix,
						 String inputNegativeChars,
						 char groupingSeparator,
						 int groupingSize,
						 char decimalSeparator,
						 boolean decimalSeparatorAlwaysShown,
						 int minimumIntegerDigits,
						 int minimumFractionDigits,
						 int maximumFractionDigits,
						 boolean removeTrailingFractionZeros,
						 int formatStyle,
						 Font positiveFont,
						 Font negativeFont,
						 Color positiveColor,
						 Color negativeColor)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols(); //d7465
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(formatStyle);
	  setPositiveFont(positiveFont);
	  setNegativeFont(negativeFont);
	  setPositiveColor(positiveColor);
	  setNegativeColor(negativeColor);
	  setDecimalFormatSymbols(symbols);       //d7465

	  return;
   }   
   /**  
   * This GsDecimalFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class plus some other formatting options. 
   * This method also accepts a Locale to accomodate NLS formatting.
   * <P>
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param formatStyle --  text = 0, html = 1
   * @param positiveFont -- a Font object for positive values 
   * @param negativeFont -- a Font object for negative values
   * @param positiveColor -- a Color object for positive values
   * @param negativeColor -- a Color object for negative values
   * @param locale -- a Locale object for formatting
   */
   public GsDecimalFormat(
						 String positivePrefix,
						 String positiveSuffix,
						 String negativePrefix,
						 String negativeSuffix,
						 String inputNegativeChars,
						 char groupingSeparator,
						 int groupingSize,
						 char decimalSeparator,
						 boolean decimalSeparatorAlwaysShown,
						 int minimumIntegerDigits,
						 int minimumFractionDigits,
						 int maximumFractionDigits,
						 boolean removeTrailingFractionZeros,
						 int formatStyle,
						 Font positiveFont,
						 Font negativeFont,
						 Color positiveColor,
						 Color negativeColor,
						 Locale locale)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);   //d7465
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(formatStyle);
	  setPositiveFont(positiveFont);
	  setNegativeFont(negativeFont);
	  setPositiveColor(positiveColor);
	  setNegativeColor(negativeColor);
	  setDecimalFormatSymbols(symbols);       //d7465

	  return;
   }   
   /**  
   * This GsDecimalFormat constructor takes many of the parameters that
   * can be set in the format and the symbols class. 
   * This method also accepts a Locale to accomodate NLS formatting.
   * <P>
   * @param positivePrefix -- String prefix to be used for positive currencies           
   * @param positiveSuffix -- String suffix to be used for positive currencies           
   * @param negativePrefix -- String prefix to be used for negative currencies           
   * @param negativeSuffix -- String suffix to be used for negative currencies           
   * @param inputNegativeChars -- String of chars that indicate negative       
   * @param groupingSeparator -- char used for group seperation          
   * @param groupingSize --  the size of the group (such as 3, thousands)                
   * @param decimalSeparator --  char used for decimal seperation          
   * @param decimalSeparatorAlwaysShown -- boolean to show decimal always
   * @param minimumIntegerDigits --  minimum number of digits to left of decimal       
   * @param minimumFractionDigits -- minimum number of fraction digits shown       
   * @param maximumFractionDigits -- maximum number of fraction digits shown      
   * @param removeTrailingFractionZeros -- boolean to remove trailing fraction zeros
   * @param locale -- a Locale object for formatting
   */
   public GsDecimalFormat(
						 String positivePrefix,
						 String positiveSuffix,
						 String negativePrefix,
						 String negativeSuffix,
						 String inputNegativeChars,
						 char groupingSeparator,
						 int groupingSize,
						 char decimalSeparator,
						 boolean decimalSeparatorAlwaysShown,
						 int minimumIntegerDigits,
						 int minimumFractionDigits,
						 int maximumFractionDigits,
						 boolean removeTrailingFractionZeros,
						 Locale locale)
   {
	  DecimalFormatSymbols symbols = new DecimalFormatSymbols(locale);  //d7465
	  setPositivePrefix(positivePrefix);
	  setPositiveSuffix(positiveSuffix);
	  setNegativePrefix(negativePrefix);
	  setNegativeSuffix(negativeSuffix);
	  setInputNegativeChars(inputNegativeChars);
	  symbols.setGroupingSeparator(groupingSeparator);
	  setGroupingSize(groupingSize);
	  symbols.setDecimalSeparator(decimalSeparator);
	  setDecimalSeparatorAlwaysShown(decimalSeparatorAlwaysShown);
	  setMinimumIntegerDigits(minimumIntegerDigits);
	  setMinimumFractionDigits(minimumFractionDigits);
	  setMaximumFractionDigits(maximumFractionDigits);
	  setRemoveTrailingFractionZeros(removeTrailingFractionZeros);
	  setFormatStyle(IfsSystem.DECIMAL_STYLE);
	  setPositiveFont(IfsSystem.DECIMAL_POSITIVE_FONT);
	  setNegativeFont(IfsSystem.DECIMAL_NEGATIVE_FONT);
	  setPositiveColor(IfsSystem.DECIMAL_POSITIVE_COLOR);
	  setNegativeColor(IfsSystem.DECIMAL_NEGATIVE_COLOR);
	  setDecimalFormatSymbols(symbols);       //d7465

	  return;
   }   
   /**
   * This GsDecimalFormat constructor takes a number pattern and a
   * Locale object used to create the DecimalFormatSymbols. Reference Java
   * documentation for the syntax of DecimalFormat patterns. 
   * <P>
   * @param pattern -- a String pattern using the DecimalFormat pattern syntax
   * @param locale -- a Locale object for formatting
   */
   public GsDecimalFormat(String pattern, Locale locale) {
	  // Always applyPattern after the symbols are set
	  setDecimalFormatSymbols(new DecimalFormatSymbols(locale));    //d7465
	  applyPattern(pattern);
	  setInputNegativeChars(IfsSystem.DEBIT_SIGN);
	  setRemoveTrailingFractionZeros(IfsSystem.REMOVE_TRAILING_FRACTION_ZEROES);
	  setFormatStyle(IfsSystem.DECIMAL_STYLE);
	  setPositiveFont(IfsSystem.DECIMAL_POSITIVE_FONT);
	  setNegativeFont(IfsSystem.DECIMAL_NEGATIVE_FONT);
	  setPositiveColor(IfsSystem.DECIMAL_POSITIVE_COLOR);
	  setNegativeColor(IfsSystem.DECIMAL_NEGATIVE_COLOR);
	  return;

   }      
   /**
   * This GsDecimalFormat constructor takes a Locale object to get 
   * a pattern and create the DecimalFormatSymbols. 
   * <P>
   * @param locale -- a Locale object for formatting
   */
   public GsDecimalFormat(Locale locale) {
	  this(ResourceBundle.getBundle("java.text.resources.LocaleElements").
		   getStringArray("NumberPatterns")[NUMBERSTYLE],locale);
	  return;
   }   
   /**
   * protected method to add End Style
   */
   protected StringBuffer addEndStyle(StringBuffer buffer, Font font, Color color) {
	  if (null!=font)
	  {
		 if (font.isItalic())
			buffer.append("</I>");
		 if (font.isBold())
			buffer.append("</B>");
	  }
	  if ((null!=font) || (null!=color))
		 buffer.append("</FONT>");

	  return buffer;
   }   
   /**
   * protected method to add End Style
   */
   protected StringBuffer addStyle(StringBuffer buffer, Font font, Color color) {
	  if ((null!=font) || (null!=color))
		 buffer.append("<FONT ");

	  if (null!=font)
		 buffer.append("SIZE=" + Integer.toString(font.getSize()));

	  if (null!=color)
	  {
		 String red = Integer.toHexString(color.getRed());
		 String green = Integer.toHexString(color.getGreen());
		 String blue = Integer.toHexString(color.getBlue());

		 buffer.append("COLOR=#"+
					   ((red.length()==1)? "0" + red : red) +
					   ((green.length()==1)? "0" + green : green) +
					   ((blue.length()==1)? "0" + blue : blue));
	  }
	  if ((null!=font) || (null!=color))
		 buffer.append(">");

	  if (null!=font)
	  {
		 if (font.isBold())
			buffer.append("<B>");
		 if (font.isItalic())
			buffer.append("<I>");
	  }
	  return buffer;
   }   
   /**
   * format returns a StringBuffer of the formatted decimal value 
   * according to the current format and symbol settings.
   * <P>
   * @param digits -- a String of digits without punctuation
   * @param precision -- number of decimal places to the right of the decimal point
   * @param debitCredit -- the logical sign of the value, 0 = debit, negative, 1 = credit, positive.
   * @param bufferToAppend -- a StringBuffer for the formatted value
   * @param position -- a FieldPosition to start appending
   * @return --  a StringBuffer of the formatted decimal
   */
   public StringBuffer format(String digits
							  , int precision
							  , int debitCredit
							  , StringBuffer bufferToAppend
							  , FieldPosition position) {
	  //This method will take digits, precision, and debitCredit and turn them into a Double object.
	  //The super class format method will then be called.  The method will then determine if trailing
	  //fraction zeros need to be removed.  Finally, this method will return the StringBuffer according
	  //to formatStyle.
          StringBuffer number = new StringBuffer(25);
          if (GsDecimal.DEBIT==debitCredit)
             number.append("-");

          if(precision <= digits.length())
             number.append(digits.substring(0,(digits.length() - precision)));
          if (precision!=0)
          {
             number.append(".");
             if(precision <= digits.length())
                    number.append(digits.substring((digits.length() - precision),digits.length()));
             else {
                    char zeroDigit = getDecimalFormatSymbols().getZeroDigit();
                    for(int i =0; i < precision - digits.length(); i++) {
                            number.append(zeroDigit);
                    }

                    number.append(digits);
             }
          }
	  StringBuffer buffer = super.format(Double.valueOf(number.toString()), bufferToAppend, position);

	  //If required, chop off fraction zeros
	  if (getRemoveTrailingFractionZeros())
	  {
		 buffer = removeTrailingFractionZeros(buffer);
	  }
	  return(getFormatStyle()==GsDecimal.FORMAT_HTML)? toHTML(buffer,debitCredit) : buffer;
   }   
   /**
   * getFormatStyle returns a int indicating the text/html option on 
   * formatting.
   * <P>
   * @return an int for style: text = 0, html = 1
   */
   public int getFormatStyle() {
	  return formatStyle;
   }   
   /**
   * getInputNegativeChars returns the String indicating the chars that
   * are to indicate a negative value
   * <P>
   * @return a String of the input negative chars
   */
   public String getInputNegativeChars() {
	  return inputNegativeChars;
   }   
   /**
   * getNegativeColor returns the Color object that is used for formatting
   * negative values.
   * <P>
   * @return the Color object used for formatting negative values
   */
   public Color getNegativeColor() {
	  return negativeColor;
   }   
   /**
   * getNegativeFont returns the Font object that is used for formatting.
   * negative values.
   * <P>
   * @return the Font object used for formatting negative values
   */
   public Font getNegativeFont() {
	  return negativeFont;
   }   
   /**
   * getPositiveColor returns the Color object that is used for formatting
   * positive values.
   * <P>
   * @return the Color object used for formatting positive values
   */
   public Color getPositiveColor() {
	  return positiveColor;
   }   
   /**
   * getPositiveFont returns the Font object that is used for formatting
   * positive values.
   * <P>
   * @return the Font object used for formatting positive values
   */
   public Font getPositiveFont() {
	  return positiveFont;
   }   
   /**
   * getRemoveTrailingFractionZeros returns the boolean that indicates
   * whether to remove trailing fraction zeros.
   * <P>
   * @return the Boolean RemoveTrailingFractionZeros
   */
   public boolean getRemoveTrailingFractionZeros() {
	  return removeTrailingFractionZeros;
   }   
   /**
   * parse parses a Number out of the supplied String.
   * <P>
   * @param decimalValue -- a String decimal value
   * @param position -- the position of the parse
   * @return the Number object for the decimal
   */
   public Number parse(String decimalValue, ParsePosition position) {
	  Number number = null;
	  String invalidString;
	  int i;
	  boolean done = false, negative = false;
	  //Super class parse method will handle parsing except for inputNegativeChars.  If super.parse()
	  //returns a null value see if character at current position is an inputNegativeChar.  If so adjust
	  //parse position, and resume.  If not return null.  Also, DecimalFormat object expects to see a monetary
	  //symbol with the string.  If this is missing, it will fail to parse the data.  Therefore, if there is
	  //data in decimalValue, and number is null, check to see if the first character of the string is either a
	  //positive or negative prefix.  If decimalValue does not contain either, insert a positive prefix value from
	  //the format object.
	  while (!done)
	  {
		 number = super.parse(decimalValue, position);
		 //Super class documentation lies.  JavaDocs says that if parsing fails a null object is returned.
		 //However, when parsing fails a 0 value is returned instead of null.  Bad documentation.
		 if (null==number || (0==position.getIndex() && 0 < decimalValue.length()))
		 {
			invalidString = decimalValue.substring(position.getIndex(),position.getIndex() + 1);
			//IfsSystem.debug("Input Negative Characters are " + inputNegativeChars);
			//IfsSystem.debug("Positive Prefix is " + getPositivePrefix());
			//IfsSystem.debug("Negative Prefix is " + getNegativePrefix());
			for (i=0,done=true; i<inputNegativeChars.length(); i++)
			   if (inputNegativeChars.startsWith(invalidString,i))
			   {
				  position.setIndex(position.getIndex() + 1);
				  negative = true;
				  done = false;
				  break;
			   }
			//IfsSystem.debug("Decimal Value is " + decimalValue);
			//IfsSystem.debug("Current Position is " + position.getIndex());
			//IfsSystem.debug("Decimal Value at current position is " + decimalValue.substring(position.getIndex()));
			if (!(null!=getPositivePrefix() && decimalValue.substring(position.getIndex()).startsWith(getPositivePrefix())))
			{
			   decimalValue = decimalValue.substring(0,position.getIndex()) + getPositivePrefix() + decimalValue.substring(position.getIndex());
			   //IfsSystem.debug("New Decimal Value is " + decimalValue);
			   done = false;

			}

		 }
		 else
			done = true;
	  }
	  //If number is 0 and the parsePosition value does not equal the length of the decimalValue then parsing
	  //failed and we should have gotten a null value instead.  Make it null before returning.
	  if (null!=number && 0==number.intValue() && position.getIndex()!=decimalValue.length())
		 number = null;

	  return(negative && null!=number) ? (Number) new Double(-(number.doubleValue())) : number;
   }   
   /**
   * protected method for removing trailing fraction zeros
   */
   protected StringBuffer removeTrailingFractionZeros(StringBuffer buffer) {
	  //Ensure there is a decimal separator
	  char[] decimalArray = {getDecimalFormatSymbols().getDecimalSeparator()}; //d7465
	  String decimal = new String(decimalArray);
	  StringTokenizer tokens = new StringTokenizer(buffer.toString(),decimal);
	  //If we got tokens, only interested in the last token
	  if (tokens.countTokens()!=0)
	  {
		 String token = tokens.nextToken();
		 while (tokens.hasMoreTokens())
		 {
			token = tokens.nextToken();
		 }
		 //At this point we are guaranteed that we have a number that contains a decimal and
		 //token contains the digits after the decimal point.  See if it ends in the zeroDigit.
		 int zeroCount = 0;
		 while (token.charAt(token.length()-1-zeroCount)==getDecimalFormatSymbols().getZeroDigit())  //d7465
		 {
			zeroCount++;
		 }
		 buffer.setLength(buffer.length()-zeroCount);
	  }
	  return buffer;
   }   
   /**
   * setFormatStyle sets the Format Style
   * <P>
   * @param formatStyle -- an int: text = 0, html = 1
   */
   public void setFormatStyle(int style) {
	  formatStyle = style;
	  return;
   }   
   /**
   * setInputNegativeChars sets the String indicating the chars that
   * are to indicate a negative value
   * <P>
   * @param negativeCharacters -- a String of the input negative chars
   */
   public void setInputNegativeChars(String negativeCharacters) {
	  inputNegativeChars = negativeCharacters;
	  return;
   }   
   /**
   * setNegativeColor sets the Color object that is used for formatting
   * negative values.
   * <P>
   * @param color --  the Color object used for formatting negative values
   */
   public void setNegativeColor(Color color) {
	  negativeColor = color;
	  return;
   }   
   /**
   * setNegativeFont sets the Font object that is used for formatting.
   * negative values.
   * <P>
   * @param font -- the Font object used for formatting negative values
   */
   public void setNegativeFont(Font font) {
	  negativeFont = font;
	  return;
   }   
   /**
   * setPositiveColor sets the Color object that is used for formatting
   * positive values.
   * <P>
   * @param color -- the Color object used for formatting positive values
   */
   public void setPositiveColor(Color color) {
	  positiveColor = color;
	  return;
   }   
   /**
   * setPositiveFont sets the Font object that is used for formatting
   * positive values.
   * <P>
   * @param font -- the Font object used for formatting positive values
   */
   public void setPositiveFont(Font font) {
	  positiveFont = font;
	  return;
   }   
   /**
   * setRemoveTrailingFractionZeros sets the boolean that indicates
   * whether to remove trailing fraction zeros.
   * <P>
   * @param flag -- the Boolean RemoveTrailingFractionZeros
   */
   public void setRemoveTrailingFractionZeros(boolean flag) {
	  removeTrailingFractionZeros = flag;
	  return;
   }   
   /**
   * protected method to output in html style
   */
   protected StringBuffer toHTML(StringBuffer buffer, int debitCredit) {
	  StringBuffer htmlBuffer = new StringBuffer(50);

	  if (debitCredit==GsDecimal.DEBIT)
	  {
		 htmlBuffer = addStyle(htmlBuffer,getNegativeFont(),getNegativeColor());
		 htmlBuffer.append(buffer.toString());
		 htmlBuffer = addEndStyle(htmlBuffer,getNegativeFont(),getNegativeColor());
	  }
	  else
	  {
		 htmlBuffer = addStyle(htmlBuffer,getPositiveFont(),getPositiveColor());
		 htmlBuffer.append(buffer.toString());
		 htmlBuffer = addEndStyle(htmlBuffer,getPositiveFont(),getPositiveColor());
	  }
	  return htmlBuffer;
   }   
}
