package com.ibm.ifs.gold;

//+------------------------------------------------------------------------+
//|                                                                        |
//|  FILE NAME:   GsTime.java                                              |
//|                                                                        |
//|  COPYRIGHT:                                                            |
//|                                                                        |
//|   Licensed Materials - Property of IBM                                 |
//|   5648-A06                                                             |
//|   (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved               |
//|    US Government Users Restricted Rights - Use, duplication            |
//|    or disclosure restricted by GSA ADP Schedule Contract               |
//|    with IBM                                                            |
//|                                                                        |
//|  CHANGES:                                                              |
//|    02/16/2000  rhs   d6341   Made JavaDoc correction/additions.        |
//|    04/11/2000  rhs   d6590   NLS changes and Javadoc additions.        |
//|    05/25/2000  rhs   d6837   Added constructor which takes an IfsSession object.
//|
//+------------------------------------------------------------------------+
import java.text.ParsePosition;
import java.util.Locale;
import java.util.Hashtable;
import java.util.Calendar;
import java.util.Date;
import java.text.FieldPosition;
import java.util.ResourceBundle;

/**
* GsTime class represents time in a Gold format (USA, ISO, EUR, JIS). The class
* also provides methods such as toString and toString(format) to display
* times in locale-sensitive manner.
* <pre>
* Gold Pattern     Gold Format    Example
* ------------     -----------    -------
* hh:mm a          "USA"          08:00 AM
* hh.mm.ss         "EUR","ISO"    08.28.21
* hh:mm:ss         "JIS"          08:28:21
* </pre>
* @see  java.util.Calendar
* @see  java.text.DateFormat
* @see  java.text.DateFormatSymbols
* @see  java.text.SimpleDateFormat
* @see  GsTimeFormat
*/
public class GsTime extends GsTimeS implements IComparator
{
   //--------------------------------------------------------------------------
   // Attributes
   //--------------------------------------------------------------------------
   public static boolean debugOn = false;
   private static Hashtable registry = new Hashtable();
   private GsTimeFormat localeFormatter;
   private GsTimeFormat goldFormatter;

   /**
   * Default constructor
   */
   public GsTime() // null constructor
   {
   }

   /**
   * Constructor with an associated IfsSession object.
   */
   public GsTime(IfsSession sess)
   {
      setIfsSession(sess);
   }

   /**
   * GMJO/GMOB string constructor
   */
   public GsTime(IfsTxn txn)
   {
      super(txn);
      setIfsSession(txn.getIfsSession());
   }

   /**
   * GMJO/GMOB string constructor.
   * @param alias -- This parameter is not currently used, it is there for future
   * consideration.  Auto-gen requires the parameter.
   */
   public GsTime(IfsTxn txn, String alias)
   {
      this(txn);
   }

   public GsTime(String goldTime, String goldTimePattern, IfsSession session) // explicit gold data constructor
   {
      setIfsSession(session);
      String pattern = toGoldPattern(IfsSystem.GOLD_TIME_FORMAT);
      if (pattern != null)
      {
         setGoldFormatter(new GsTimeFormat(pattern));
         try
         {
            GsTimeFormat formatter = new GsTimeFormat(goldTimePattern);
            Date parseTime = formatter.parse(goldTime);
            if (parseTime != null)
            {
               time = getGoldFormatter().format(parseTime);
               format = IfsSystem.GOLD_TIME_FORMAT;
            }
         }
         catch (Exception e)
         {
         }
      }
   }

   public GsTime(String goldTime, String goldTimePattern, String goldTimeFormat, IfsSession session) // Gold Data constructor
   {
      setIfsSession(session);
      String pattern = toGoldPattern(goldTimeFormat);
      if (pattern != null)
      {
         setGoldFormatter(new GsTimeFormat(pattern));
         try
         {
            GsTimeFormat formatter = new GsTimeFormat(goldTimePattern);
            Date parseTime = formatter.parse(goldTime);
            if (parseTime != null)
            {
               time = getGoldFormatter().format(parseTime);
               format = goldTimeFormat;
            }
         }
         catch (Exception e)
         {

         }
      }
   }

   /**
   * Compares this Gold time object with another Gold time object for equality.
   * @param d -- The time which is compared with this GsTime object.
   * @return
   *    Returns true when they are equal, false otherwise.
   */
   public boolean equals(GsTime d)
   {
      if (time != null)
      {
         return time.equals(d.getTime());
      }
      return false;
   }

   /**
   * Converts a Date object using the GsTimeFormat argument and returns
   * a GsTime object.
   * @param timeToConvert -- the Date object to be converted to a GsTime object.
   * @param timeFormatter -- the format used to convert the date.
   * @return
   *    The GsTime object which was created from the date.
   */
   public static GsTime fromDate(Date timeToConvert, GsTimeFormat timeFormatter)
   {
      GsTime gsTime = new GsTime();
      String pattern = timeFormatter.toPattern();
      String format = fromGoldPattern(pattern);

      if (format == null) // check for gold pattern
      {
         format = IfsSystem.GOLD_TIME_FORMAT;
         pattern = toGoldPattern(format);
         timeFormatter = new GsTimeFormat(pattern);
      }
      gsTime.setGoldFormatter(timeFormatter);
      gsTime.setTime(timeFormatter.format(timeToConvert));
      gsTime.setFormat(format);

      return gsTime;
   }

   /**
   * Converts a gold time pattern to gold time format string.
   */
   public static String fromGoldPattern(String goldPattern)
   {
      String goldFormat = null;
      if (goldPattern.equals("hh.mm.ss"))
      {
         goldFormat = "ISO";
      }
      else if (goldPattern.equals("hh:mm:ss"))
      {
         goldFormat = "JIS";
      }
      else if (goldPattern.equals("hh:mm a"))
      {
         goldFormat ="USA";
      }
      return goldFormat;
   }

   /**
   * Converts text string to GsTime object. It calls the registered fromString
   * method to parse the string. If the fromString method is not registered,
   * the GoldFormatter attribute is used to do the default parsing of the string.
   */
   public static GsTime fromString(String text) throws Exception
   {
      GsTime t = new GsTime();
      if (t.hasRegisteredMethod("fromString"))
      {
         try
         {
            return(GsTime)t.invokeMethod("fromString",new Object[]{text});
         }
         catch (Exception e)
         {
            if (e.getMessage().equals("Method not registered."))
            {
               return null;
            }
            else
            {
               throw e;
            }
         }
      }
      else
      {
         GsTimeFormat formatter = t.getGoldFormatter();
         formatter.setLenient(false);
         Date parseTime = formatter.parse(text,new ParsePosition(0));
         if (parseTime != null)
         {
            t = fromDate(parseTime, formatter);
         }
         return t;
      }
   }

   /**
   * Converts text string to GsTime object. It calls the registered fromStringWithFormat
   * method to parse the string. If the fromStringWithFormat method is not registered,
   * it uses myFormat argument to parse the text.
   */
   public static GsTime fromString(String text, GsTimeFormat myFormat) throws Exception
   {
      GsTime d = new GsTime();
      d.setLocaleFormatter(myFormat);
      if (d.hasRegisteredMethod("fromStringWithFormat"))
      {
         try
         {
            return(GsTime)d.invokeMethod("fromStringWithFormat",new Object[]{text,myFormat});
         }
         catch (Exception e)
         {
            if (e.getMessage().equals("Method not registered."))
            {
               return null;
            }
            else
            {
               throw e;
            }
         }
      }
      else
      {
         Date parseTime = myFormat.parse(text,new ParsePosition(0));
         if (parseTime != null)
         {
            d = fromDate(parseTime,myFormat);
         }
      }
      return d;
   }

   /**
   * Returns the goldFormatter attribute.
   * When the goldFormatter attribute is null, the format attribute is used to set the
   * goldFormatter attribute and it is then returned.
   * If the format attribute is also null, the IfsSystem.GOLD_TIME_FORMAT is used to set
   * the goldFormatter attribute and it is then returned.
   */
   public GsTimeFormat getGoldFormatter()
   {
      if (goldFormatter == null)
      {
         if (format != null)
         {
            setGoldFormatter(new GsTimeFormat(toGoldPattern(format)));
         }
         else
         {
            setGoldFormatter(new GsTimeFormat(toGoldPattern(IfsSystem.GOLD_TIME_FORMAT)));
         }
      }
      return goldFormatter;
   }

   public int getHours()
   {
      int hour = 0;
      if (time != null)
      {
         hour = Integer.parseInt(time.substring(0,2),10); // first two chars of gold pattern time
         if (Character.isSpaceChar(time.charAt(5)))
         {
            if (hour == 12) hour = 0;
            if (String.valueOf(time.charAt(6)).toUpperCase().equals("P")) hour += 12;
         }
         if (hour < 0 || hour > 23) hour = 0;
      }
      return hour;
   }

   /**
   * Returns localeFormatter attribute.
   * When localeFormatter attribute is null and a session exists, the session
   * locale is used to create the localeFormatter.
   * Otherwise, the server locale is used to create the localeFormatter.
   */
   public GsTimeFormat getLocaleFormatter()
   {
      if (localeFormatter == null)
      {
         GsTimeFormat tempFormat;
         if (getIfsSession() != null)
         {
            tempFormat = new GsTimeFormat(getIfsSession().getLocale() );
         }
         else
         {
            tempFormat = new GsTimeFormat();
         }
         return tempFormat;
      }
      return localeFormatter;
   }

   public int getMinutes()
   {
      int minute = 0;
      if (time != null)
      {
         minute = Integer.parseInt(time.substring(3,5),10);
         if (minute < 0 || minute > 59) minute = 0;
      }
      return minute;
   }

   /**
   * Returns registry attribute that contains registered methods and its names mapping.
   */
   public static Hashtable getRegistry()
   {
      return registry;
   }
   public int getSeconds()
   {
      int second = 0;
      if (time != null)
      {
         if (Character.isSpaceChar(time.charAt(5)))
         {
            second = 0;
         }
         else
         {
            second = Integer.parseInt(time.substring(6,8),10);
         }
         if (second < 0 || second > 59) second = 0;
      }
      return second;
   }
   public void setGoldFormatter(GsTimeFormat formatter)
   {
      goldFormatter = formatter;
   }
   public void setLocaleFormatter(GsTimeFormat formatter)
   {
      localeFormatter = formatter;
      return;
   }

   /**
   * Converts a gold time format string to gold time pattern.
   */
   public static String toGoldPattern(String goldFormat)
   {
      String pattern = null;
      if (goldFormat.equals("USA"))
      {
         pattern = "hh:mm a";
      }
      else if (goldFormat.equals("ISO")||goldFormat.equals("EUR"))
      {
         pattern = "hh.mm.ss";
      }
      else if (goldFormat.equals("JIS"))
      {
         pattern = "hh:mm:ss";
      }
      return pattern;
   }

   /**
   * Converts the GsTime object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, the GsTime object is Locale formatted.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      String str = super.toString();
      if (str == null)
      {
         return(toString(getLocaleFormatter()));
      }
      else
      {
         return str;
      }
   }

   /**
   * Converts the GsTime object to a localized string representation using
   * the pattern specified by <EM>myFormat</EM> parameter.
   * When a method is registered to <EM>toString</EM> for this class, it is used.
   * Otherwise, the default formatting is done with <EM>myFormat</EM> parameter.
   *
   * @param myFormat -- Pattern used to format this object.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(GsTimeFormat myFormat)
   {
      String str = super.toString(myFormat);
      if (str != null)
      {
         return str;
      }
      else
      {
         Calendar cal = Calendar.getInstance();
         cal.clear();
         cal.set(Calendar.SECOND, getSeconds());
         cal.set(Calendar.MINUTE, getMinutes());
         cal.set(Calendar.HOUR, getHours());
         return myFormat.format(cal.getTime(), new StringBuffer(), new FieldPosition(0)).toString();
      }
   }

   /**
   * Converts the GsTime object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   * The default conversion prints time and format attributes
   * along with the super class attributes.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
      {
         return super.toString(prefix);
      }
      else
      {

         StringBuffer results = new StringBuffer("");
         results.append(super.toString(prefix));
         results.append(prefix);
         results.append("| time = ");
         results.append(getTime());
         results.append(prefix);
         results.append("| format = ");
         results.append(getFormat());

         return results.toString();
      }
   }

   /**
   * Validates the GsTime object.
   * When a method is registered to <EM>validate</EM> for this class,
   * it is used.  Otherwise, a default validation is done.
   *
   * @return
   *    Returns "OK" if the GsTime is valid.  Otherwise, it returns a message
   *    string indicating the error.
   */
   public String validate()
   {
      String str = super.validate();
      if (str == null) // check if validate method is registered
      {
         int hour = getHours();
         int min = getMinutes();
         int sec = getSeconds();
         ResourceBundle bundle = ResourceBundle.getBundle("com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());

         if (hour < 0 || hour > 23)
            return bundle.getString("GsTime.validation.hour");
         else if (min < 0 || min > 59)
            return bundle.getString("GsTime.validation.minute");
         else if (sec <0 || sec >59)
            return bundle.getString("GsTime.validation.second");
         else
            return "OK";
      }
      return str;
   }

   /**
   * Compares 2 GsTime objects.
   *
   * @param a -- First GsTime object.
   * @param b -- Second GsTime object.
   * @return
   *    Returns -2 when "a" or "b" are not GsTime objects.
   *    Returns -1 when "a" less than "b".
   *    Returns 0 when "a" equals "b".
   *    Returns 1 when "a" greater than "b".
   */
   public int compare(Object a, Object b)
   {
      int rc = -2;
      if ((a instanceof GsTime) && (b instanceof GsTime))
      {
         GsTime ta = (GsTime)a;
         int ha = ta.getHours();
         int ma = ta.getMinutes();
         int sa = ta.getSeconds();

         GsTime tb = (GsTime)b;
         int hb = tb.getHours();
         int mb = tb.getMinutes();
         int sb = tb.getSeconds();

         if (ha < hb)
         {
            rc = -1;
         }
         else if (ha > hb)
         {
            rc = 1;
         }
         else if (ha == hb)
         {
            if (ma < mb)
            {
               rc = -1;
            }
            else if (ma > mb)
            {
               rc = 1;
            }
            else if (ma == mb)
            {
               if (sa < sb)
               {
                  rc = -1;
               }
               else if (sa > sb)
               {
                  rc = 1;
               }
               else if (sa == sb)
               {
                  rc = 0;
               }
            }
         }
      }
      return rc;
   }
}

