package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsApi.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/15/2000  rhs   d6423   Made JavaDoc additions.
//    03/30/2000  rhs   d6545   MAX_FIXED_STRING_LENGTH constant for NLS support.
//    04/06/2000  clr   d6539   MAX_FIXED_STRING_LENGTH must be 'final' - it can't
//				change at runtime and it must be identified as constant
//				to be included in generated C header files.
//    02/02/2002  tlt  T1000102 Broke up gmji/gmjo toString calls for debug, so determine address
//                              display length based on complete byte array length.
//
// ***************************************************************************

/**
* IfsApi provides utility functions for IFS objects:
* for example, left and right justification, trimming of trailing blanks, etc..
*/
public class IfsApi
{
   /**
   * @Deprecated
   * The digits 0 thru 9.
   * Should use the isDigit class method in java.lang.Character.
   */
   public static final String DIGITS = "0123456789";

   /**
   * When the length parameter for getNextCharFromGMJO
   * method is greater than this value, it is assumed that the next field from GMJO
   * is a variable length field instead of a fixed length field.
   */
   public static final int MAX_FIXED_STRING_LENGTH = 32;

   //-----------------------------------------------------------------------------
   // Function: chop (str)
   //           chop (str, pad)
   //
   // Removes leading characters from str.  If pad is
   // not specified, blanks are removed.
   //
   // Examples:
   //    x = Chop ("  ANC");       // x now contains "ANC"
   //    y = Chop ("00123", "0");  // y now contains "123"
   //-----------------------------------------------------------------------------

   /**
   * Removes leading blank characters from a String object.
   *
   * @param str -- String object which requires removal of leading blanks.
   * @return
   *    Returns a String object with blanks removed from left side.
   */
   public static String chop (String str)
   {
      return(IfsApi.chop(str, ' '));
   }

   /**
   * Removes leading "pad" characters from a String object.
   *
   * @param str -- String object which requires removal of leading "pad" characters.
   * @param pad -- The padding character which is removed from "str" String object.
   * @return
   *    Returns a String object with "pad" characters removed from left side.
   */
   public static String chop (String str, char pad)
   {
      int i;
      for (i=0; i<str.length(); i++)
      {
         if (str.charAt(i) != pad)
            break;
      }
      return(str.substring(i,str.length()));
   }

   /**
   * @Deprecated
   * Should use the isDigit class method in java.lang.Character.
   *
   * @param c -- The character which is checked for being a digit.
   * @return
   *    Returns true when 'c' is a valid digit and false otherwise.
   */
   public static final boolean isDigit(char c)
   {
      if (DIGITS.indexOf(c) >= 0)
         return true;
      else
         return false;
   }

   /**
   * Indicates whether the specified input parameters represent a valid date.
   *
   * @param m -- The month.
   * @param d -- The day.
   * @param y -- The year in xxxx form.
   * @return
   *    Returns true when input is a valid date and false otherwise.
   */
   public static final boolean isValidDate (int m,int d, int y)
   {
      int n;
      boolean leap;

      leap = ((y%4 == 0) && (y%100 != 0)) || (y%400 == 0);

      // check for valid Gregorian month; no adjustment for pre-Gregorian times

      if (m<1 || m>12) return false;

      // check for valid day based on month and year
      if (m==2)
      {
         n = (leap ? 29 : 28);
      }
      else
      {
         n = (m==4 || m==6 || m==9 || m==11) ? 30 : 31;
      }

      if (d<1 || d>n) return false;

      return true;
   }

   /**
   * Converts a double to a String object,
   * left justifies it, and makes it a specified length.
   * When double string length is less than "width", then blank characters are
   * padded on right side of double string to make total length equal to "width".
   * When double string length is greater than "width",
   * then double string is truncated on the right.
   *
   * @param i -- The double which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(double i, int width)
   {
      return(IfsApi.left(Double.toString(i), width, ' '));
   }

   /**
   * Converts a double to a String object,
   * left justifies it, and makes it a specified length.
   * When double string length is less than "width", then "pad" characters are
   * padded on right side of double string to make total length equal to "width".
   * When double string length is greater than "width",
   * then double string is truncated on the right.
   *
   * @param i -- The double which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character used as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(double i, int width, char pad)
   {
      return(IfsApi.left(Double.toString(i), width, pad));
   }

   /**
   * Converts a double to a String object,
   * left justifies it, and makes it a specified length.
   * When double string length is less than "width", then "pad" characters are
   * padded on right side of double string to make total length equal to "width".
   * When double string length is greater than "width",
   * then double string is truncated on the right.
   *
   * @param i -- The double which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(double i, int width, String pad)
   {
      return(IfsApi.left(Double.toString(i), width, pad.charAt(0)));
   }

   /**
   * Converts an integer to a String object,
   * left justifies it, and makes it a specified length.
   * When integer string length is less than "width", then blank characters are
   * padded on right side of integer string to make total length equal to "width".
   * When integer string length is greater than "width",
   * then integer string is truncated on the right.
   *
   * @param i -- The integer which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(int i, int width)
   {
      return(IfsApi.left(Integer.toString(i), width, ' '));
   }

   /**
   * Converts an integer to a String object,
   * left justifies it, and makes it a specified length.
   * When integer string length is less than "width", then "pad" characters are
   * padded on right side of integer string to make total length equal to "width".
   * When integer string length is greater than "width",
   * then integer string is truncated on the right.
   *
   * @param i -- The integer which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character used as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(int i, int width, char pad)
   {
      return(IfsApi.left(Integer.toString(i), width, pad));
   }

   /**
   * Converts an integer to a String object,
   * left justifies it, and makes it a specified length.
   * When integer string length is less than "width", then "pad" characters are
   * padded on right side of integer string to make total length equal to "width".
   * When integer string length is greater than "width",
   * then integer string is truncated on the right.
   *
   * @param i -- The integer which is left justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(int i, int width, String pad)
   {
      return(IfsApi.left(Integer.toString(i), width, pad.charAt(0)));
   }

   //------------------------------------------------------------------------------
   // Function: Left (string, width [,pad])
   //
   // Left justifies a string.  If string.length > width, pad characters
   // will be added to the right side of the string to pad out the width.
   // If pad is not specified, the pad character defaults to " ".
   // If str.length < width, str is truncated on the right.
   //
   // Example:
   //    x = Left ("123",5,'0');  // x now contains "12300"
   //------------------------------------------------------------------------------

   /**
   * Left justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then blank characters are
   * padded on right side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the right.
   *
   * @param s -- The String object which is left justified.
   * @param width -- The length of the resulting String object.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(String s, int width)
   {
      return(IfsApi.left(s,width,' '));
   }

   /**
   * Left justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then "pad" characters are
   * padded on right side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the right.
   *
   * @param s -- The String object which is left justified.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character used as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(String s, int width, char pad)
   {
      String r;

      if (s == null)     // allow null input strings
         r = "";
      else
         r = s;

      // if string too big or just right, trim off the back
      if (r.length()>=width)
      {
         return(r.substring(0, width));
      }

      // string is shorter than width, need to pad on front
      while (r.length() < width)
      {
         r = r + pad;
      }
      return(r);
   }

   /**
   * Left justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then "pad" characters are
   * padded on right side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the right.
   *
   * @param s -- The String object which is left justified.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A left justified String object of "width" length.
   */
   public static String left(String s, int width, String pad)
   {
      return(IfsApi.left(s,width,pad.charAt(0)));
   }

   /**
   * Converts a byte array to a hexadecimal String object and outputs it to the
   * standard output.
   * Nothing is outputted when "b" is null or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param b -- The byte array which is converted to a hex String object.
   */
   public static void log(byte[] b)
   {
      if (IfsSystem.SHOW_DEBUG_CODE) printByteBuffer(b);
   }

   /**
   * Outputs a String object to the standard output.
   * Nothing is outputted when "s" is null or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param s -- The String object which is to be sent to standard output.
   */
   public static void log(String s)
   {
      if (IfsSystem.SHOW_DEBUG_CODE) System.out.println(s);
   }

   /**
   * Converts a byte array to a hexadecimal String object and outputs it to the
   * standard output.
   * Nothing is outputted when "b" is null or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param flag -- When true, "b" is sent to the standard output.
   *    When false, nothing is outputted.
   * @param b -- The byte array which is converted to a hex String object.
   */
   public static void log(boolean flag, byte[] b)
   {
      if (flag) printByteBuffer(b);
   }

   /**
   * Outputs a String object to the standard output.
   * Nothing is outputted when "s" is null.
   *
   * @param flag -- When true, "s" is sent to the standard output.
   *    When false, nothing is outputted.
   * @param s -- The String object which is to be sent to standard output.
   */
   public static void log(boolean flag, String s)
   {
      if (flag) System.out.println(s);
   }

   /**
   * Converts a byte array to a hexadecimal String object and outputs it to the
   * standard output.
   * Nothing is outputted when "buf" is null or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param buf -- The byte array which is converted to a hex String.
   */
   public static void printByteBuffer(byte[] buf)
   {
      printByteBuffer(buf, 0, buf.length);
   }

   /**
   * Converts a byte array to a hexadecimal String object and outputs it to the
   * standard output.
   * Nothing is outputted when input parameters result in a zero length byte array
   * or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param buf -- The byte array which is converted to a hex String.
   * @param numbytes -- Starting at offset 0, the number of bytes in "buf" byte
   *    array to convert.
   */
   public static void printByteBuffer(byte[] buf, int numbytes)
   {
      printByteBuffer(buf, 0, numbytes);
   }

   /**
   * Converts a byte array to a hexadecimal String object and outputs it to the
   * standard output.
   * Nothing is outputted when input parameters result in a zero length byte array
   * or IfsSystem.SHOW_DEBUG_CODE is false.
   *
   * @param ba -- The byte array which is converted to a hex String.
   * @param offset -- The conversion starting offset in "ba" byte array.
   * @param numbytes -- The number of bytes in "ba" byte array to convert.
   */
   public static void printByteBuffer(byte[] ba, int offset, int numbytes)
   {
      String[] hex = {"0","1","2","3","4","5","6","7",
         "8","9","A","B","C","D","E","F"};
      int endOffset, al;

      if (ba == null) return;            // do nothing if no byte array passed
      if (offset < 0) return;            // do nothing if negative offset
      if (numbytes < 1) return;          // do nothing if length 0 or negative
      if (offset > ba.length) return;    // do nothing if starting offset is larger than end of string

      endOffset = offset + numbytes;     // get ending offset

      if (endOffset > ba.length)         // if number of bytes to print from starting offset is larger
                                         // than size of string,
         endOffset = ba.length;          // then stop at end of string


      if (endOffset < 0x00010000)        // does endOffset fit in 4 hex digits
         al = 4;                         // yes, display 4 digits
      else
         al = 6;                         // no, display 6 digits

      // start at offset into array, looking at lines 16 bytes at a time

      for (int i = offset; i < endOffset; i+=16)
      {
         String s = "00000000" + Integer.toHexString(i); // get offset
         s = s.substring(s.length()-al, s.length());     // adjust size
         s = s+"  ";                                     // add 2 blanks

         int n = Math.min(16, endOffset-i); // get lesser of 16 or number of bytes still to process

         // process the hex output
         for (int j = 0; j < n; j++)
         {
            int ihi = (ba[i+j]& 0x000000F0)/16;
            int ilo = ba[i+j] & 0x0000000F;
            s = s + hex[ihi] +hex[ilo];
            if (j%4 == 4-1)               // add blank separator every 4 bytes
               s = s + " ";
         }

         s = s.toUpperCase();             // so far only hex has been output, convert
                                          // to upper case before adding character output

         s = s + "                                                                 ";
         s = s.substring(0, al + 40);
         s = s + "[";

         // process the character output
         for (int j = 0; j < n; j++)
         {
            if (ba[i+j] < 0x20 || ba[i+j] > 0x7e)
               s = s + ".";
            else
               s = s + new String(ba, i+j, 1);
         }

         s = s + "]";
         log(s);
      }
   }

   /**
   * Converts a double to a String object,
   * right justifies it, and returns a String of "width" length.
   * When double string length is less than "width", then blank characters are
   * padded on left side to make total length equal to "width".
   * When double string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The double which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(double i, int width)
   {
      return(IfsApi.right(Double.toString(i), width, ' '));
   }

   /**
   * Converts a double to a String object,
   * right justifies it, and returns a String of "width" length.
   * When double string length is less than "width", then "pad" characters are
   * padded on left side to make total length equal to "width".
   * When double string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The double which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character for padding.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(double i, int width, char pad)
   {
      return(IfsApi.right(Double.toString(i), width, pad));
   }

   /**
   * Converts a double to a String object,
   * right justifies it, and returns a String of "width" length.
   * When double string length is less than "width", then "pad" characters are
   * padded on left side to make total length equal to "width".
   * When double string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The double which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(double i, int width, String pad)
   {
      return(IfsApi.right(Double.toString(i), width, pad.charAt(0)));
   }

   /**
   * Converts an integer to a String object,
   * right justifies it, and returns a String of "width" length.
   * When integer string length is less than "width", then blank characters are
   * padded on left side to make total length equal to "width".
   * When integer string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The integer which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(int i, int width)
   {
      return(IfsApi.right(Integer.toString(i), width, ' '));
   }

   /**
   * Converts an integer to a String object,
   * right justifies it, and returns a String of "width" length.
   * When integer string length is less than "width", then "pad" characters are
   * padded on left side to make total length equal to "width".
   * When integer string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The integer which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character for padding.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(int i, int width, char pad)
   {
      return(IfsApi.right(Integer.toString(i), width, pad));
   }

   /**
   * Converts an integer to a String object,
   * right justifies it, and returns a String of "width" length.
   * When integer string length is less than "width", then "pad" characters are
   * padded on left side to make total length equal to "width".
   * When integer string length is greater than "width",
   * then it is truncated on the left.
   *
   * @param i -- The integer which is right justified in a String object.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(int i, int width, String pad)
   {
      return(IfsApi.right(Integer.toString(i), width, pad.charAt(0)));
   }

   //------------------------------------------------------------------------------
   //
   // Function: Right (string, width [,pad])
   //
   // Right justifies a string.  If string.length > width, pad characters
   // will be added to the left side of the string to pad out the width.
   // If pad is not specified, the pad character defaults to " ".
   // If string.length < width, string is truncated on the left.
   //
   // Example:
   //    x = Right ("123",5,'0');  // x now contains "00123"
   //------------------------------------------------------------------------------

   /**
   * Right justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then blank characters are
   * padded on left side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the left.
   *
   * @param s -- The String object which is right justified.
   * @param width -- The length of the resulting String object.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(String s, int width)
   {
      return(IfsApi.right(s,width,' '));
   }

   /**
   * Right justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then "pad" characters are
   * padded on left side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the left.
   *
   * @param s -- The String object which is right justified.
   * @param width -- The length of the resulting String object.
   * @param pad -- The character for padding.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(String s, int width, char pad)
   {
      String r;

      if (s == null)   // allow null input strings
         r = "";
      else
         r = s;

      // if string too big or just right, trim off the front
      if (r.length()>=width)
      {
         return(r.substring(r.length()-width));
      }
      // string is shorter than width, need to pad on front

      while (r.length() < width)
      {
         r = pad + r;
      }
      return(r);
   }

   /**
   * Right justifies a String object and makes it a specified length.
   * When length of "s" is less than "width", then "pad" characters are
   * padded on left side of "s" to make total length equal to "width".
   * When length of "s" is greater than "width",
   * then "s" is truncated on the left.
   *
   * @param s -- The String object which is right justified.
   * @param width -- The length of the resulting String object.
   * @param pad -- The first character in this String object is used
   *    as the padding character.
   * @return
   *    A right justified String object of "width" length.
   */
   public static String right(String s, int width, String pad)
   {
      return(IfsApi.right(s,width,pad.charAt(0)));
   }

   /**
   * Converts all the bytes in a byte array to a hexadecimal String object.
   *
   * @param buf -- The byte array which is converted to a hex String.
   * @return
   *    A String object of hexadecimal characters.
   *    A null is returned when input parameters result in a zero length byte array.
   */
   public static String toString(byte[] buf)
   {
      return toString(buf, 0, buf.length);
   }

   /**
   * Starting at index 0, converts a byte array to a hexadecimal String object.
   *
   * @param buf -- The byte array which is converted to a hex String.
   * @param numbytes -- Starting from index 0, the number of bytes in "buf" byte array
   *    to convert.
   * @return
   *    A String object of hexadecimal characters.
   *    A null is returned when input parameters result in a zero length byte array.
   */
   public static String toString(byte[] buf, int numbytes)
   {
      return toString(buf, 0, numbytes);
   }

   /**
   * Converts a byte array to a hexadecimal String object.
   *
   * @param ba -- The byte array which is converted to a hex String.
   * @param offset -- The conversion starting offset in "ba" byte array.
   * @param numbytes -- The number of bytes in "ba" byte array to convert.
   * @return
   *    A String object of hexadecimal characters.
   *    A null is returned when input parameters result in a zero length byte array.
   */
   public static String toString(byte[] ba, int offset, int numbytes)
   {
      String[] hex = {"0","1","2","3","4","5","6","7",
         "8","9","A","B","C","D","E","F"};
      int endOffset, al;

      if (ba == null) return null;            // do nothing if no byte array passed
      if (offset < 0) return null;            // do nothing if negative offset
      if (numbytes < 1) return null ;          // do nothing if length 0 or negative
      if (offset > ba.length) return null;    // do nothing if starting offset is larger than end of string

      endOffset = offset + numbytes;     // get ending offset

      if (endOffset > ba.length)         // if number of bytes to print from starting offset is larger
                                         // than size of string,
         endOffset = ba.length;          // then stop at end of string


      // if (endOffset < 0x00010000)        // T1000102 
      if (ba.length < 0x00010000)        // does buffer length fit in 4 hex digits
         al = 4;                         // yes, display 4 digits
      else
         al = 6;                         // no, display 6 digits

      // start at offset into array, looking at lines 16 bytes at a time

      String s, str = "";

      for (int i = offset; i < endOffset; i+=16)
      {
         s = "00000000" + Integer.toHexString(i); // get offset
         s = s.substring(s.length()-al, s.length());     // adjust size
         s = s+"  ";                                     // add 2 blanks

         int n = Math.min(16, endOffset-i); // get lesser of 16 or number of bytes still to process

         // process the hex output
         for (int j = 0; j < n; j++)
         {
            int ihi = (ba[i+j]& 0x000000F0)/16;
            int ilo = ba[i+j] & 0x0000000F;
            s = s + hex[ihi] +hex[ilo];
            if (j%4 == 4-1)               // add blank separator every 4 bytes
               s = s + " ";
         }

         s = s.toUpperCase();             // so far only hex has been output, convert
         // to upper case before adding character output

         s = s + "                                                                 ";
         s = s.substring(0, al + 40);
         s = s + "[";

         // process the character output
         for (int j = 0; j < n; j++)
         {
            if (ba[i+j] < 0x20 || ba[i+j] > 0x7e)
               s = s + ".";
            else
               s = s + new String(ba, i+j, 1);
         }

         s = s + "]\n";
         str += s;
      }
      return str;
   }

   //-----------------------------------------------------------------------------
   // Examples:
   //    x = trim ("ANC  ");       // x now contains "ANC"
   //    y = trim ("ANCXX", "X");  // y now contains "ANC"
   //-----------------------------------------------------------------------------

   /**
   * Removes trailing blank characters from a String object.
   *
   * @param str -- The String object from which trailing blanks are removed.
   * @return
   *    The input String with trailing blanks removed.
   */
   public static String trim (String str)
   {
      return(IfsApi.trim(str, ' '));
   }

   /**
   * Removes trailing characters from a String object.
   *
   * @param str -- The String object from which trailing "pad" characters are removed.
   * @param pad -- The padding character which is to be removed.
   * @return
   *    The input String with trailing "pad" characters removed.
   */
   public static String trim(String str, char pad)
   {
      int i;
      for (i=str.length()-1; i>=0; --i)
      {
         if (str.charAt(i) != pad)
            break;
      }
      return(str.substring(0,i+1));
   }
}

