package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsException.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/15/2000  rhs   d6423   Made JavaDoc additions.
//
// ***************************************************************************

import java.io.PrintStream;
import java.io.PrintWriter;

/**
* IfsException class provides exception processing for exceptions which are
* generated by and unique to IFS objects.
*/
public class IfsException extends Exception
{
   private Throwable e;
   private String    type;
   private String    value;

   /**
   * Default constructor.
   */
   public IfsException()
   {
   }

   /**
   * Constructor with exception type and exception message.
   *
   * @param type -- Indicates the type of Java exception which occurred.
   * @param msg -- A description of the exception.
   */
   public IfsException(String type, String msg)
   {
      super(msg);
      setType(type);
      e = null;
   }

   /**
   * Constructor with exception type, exception value, and exception message.
   *
   * @param type -- Indicates the type of Java exception which occurred.
   * @param value -- Provides additional data about the exception.
   * @param msg -- A description of the exception.
   */
   public IfsException(String type, String value, String msg)
   {
      super(msg);
      setType(type);
      setValue(value);
      e = null;
   }

   /**
   * Constructor with throwable exception and exception message.
   *
   * @param e -- The throwable exception.
   * @param msg -- A description of the exception.
   */
   public IfsException(Throwable e, String msg)
   {
      super(msg);
      if (e instanceof IfsException)
         setType(((IfsException)e).getType());
      else
         setType(e.getClass().getName());
      this.e = e;
   }

   /**
   * Constructor with throwable exception, exception value, and exception message.
   *
   * @param e -- The throwable exception.
   * @param value -- Provides additional data about the exception.
   * @param msg -- A description of the exception.
   */
   public IfsException(Throwable e, String value, String msg)
   {
      super(msg);
      if (e instanceof IfsException)
         setType(((IfsException)e).getType());
      else
         setType(e.getClass().getName());
      setValue(value);
      this.e = e;
   }

   /**
   * Returns the exception type.
   * The type attribute is used to provide the type of Java exception that occurred:
   * for example, NullPointerException, NoSuchFieldError, etc.
   */
   public String getType()
   {
      return type;
   }

   /**
   * Returns the exception value.
   * The value attribute is used to provide additional data about the exception:
   * for example, what field couldn't be set, what method was not found, etc.
   */
   public String getValue()
   {
      return value;
   }

   /**
   * Outputs the exception stack trace to the specified PrintStream object.
   */
   public void printStackTrace(PrintStream s)
   {
      s.println(this);
      s.println("\tStack Trace:");
      s.print("\t");
      if (e != null) e.printStackTrace(s);
      s.println(" ");
   }

   /**
   * Outputs the exception stack trace to the specified PrintWriter object.
   */
   public void printStackTrace(PrintWriter s)
   {
      s.println(this);
      s.println("\tStack Trace:");
      s.print("\t");
      if (e != null) e.printStackTrace(s);
      s.println(" ");
   }

   /**
   * Sets the exception type.
   * The type attribute is used to provide the type of Java exception that occurred:
   * for example, NullPointerException, NoSuchFieldError, etc.
   */
   public void setType(String type)
   {
      this.type = type;
   }

   /**
   * Sets the exception value.
   * The value attribute is used to provide additional data about the exception:
   * for example, what field couldn't be set, what method was not found, etc.
   */
   public void setValue(String value)
   {
      this.value = value;
   }

   /**
   * Converts the IfsException object to a String object and returns it.
   */
   public String toString()
   {
      return(new java.util.Date()).toString() + " " + "Throwable Type: " + getType() +
      ";\n\tValue: "    + getValue() +
      ";\n\tMessage: "  + getMessage();
   }
}
