/* -------------------------------------------------------------------------- *
 *                                                                            *
 * File Name: IfsJDBC.java                                                    *
 *                                                                            *
 * Copyright:                                                                 *
 *                                                                            *
 *   Licensed Materials - Property of IBM                                     *
 *   5648-A06                                                                 *
 *   (C) Copyright IBM Corp. 2000 All Rights Reserved                         *
 *   US Government Users Restricted Rights - Use, duplication                 *
 *   or disclosure restricted by GSA ADP Schedule Contract                    *
 *   with IBM                                                                 *
 *                                                                            *
 * Change History:                                                            *
 *                                                                            *
 * Date                         Name                Reason For Change         *
 * ------                       ------              -------------------       *
 *                                                                            *
 * --                           --                  Initial Creation          *
 *                                                                            *
 * 01/19/2001                   A.Asokan            Fixed bugs and validation *
 *                                                  so that exceptions are    *
 *                                                  thrown for error messages * 
 *                                                                            *
 * 06/20/2001                   A.Asokan            Now uses WebSphere        *
 *                                                  connection manager.       *
 *                                                                            *
 * 06/25/2001                   M.Curtiss           Code cleanup.             *
 *                                                                            *
 * 11/05/2001		        B.Brockway          Updated for WAS 4         *
 *                                                  connection pooling.       * 
 *                                                  Code cleanup.             *
 * 02/13/2002			B.Brockway          Loading DataSource and    *
 *                                                  InitialContext from       *
 *                                                  IfsSystem property. d7435 *
 *                                                                            *
 * -------------------------------------------------------------------------- */

package com.ibm.ifs.gold;

import java.sql.*;
import java.util.*;
import javax.naming.*;
import java.math.BigDecimal;

/**
 * The IfsJDBC class abstracts the common properties and behaviors of the
 * IFS database transaction objects defined in DCRs 598 and 599.
 * <BR>
 * The common features include validation, data conversion, logging,
 * database queries/updates, error handling, etc..
 * <BR>
 * Every transaction class that performs database operations
 * should inherit from IfsJDBC. This class extends the
 * com.ibm.ifs.gold.IfsTxn class.
*/
public class IfsJDBC extends IfsTxn {

/* --------------------------- Class Variables ------------------------------ */

  /* ------------------------------------------------------------------------ *
   * Completion codes                                                         *
   * ------------------------------------------------------------------------ */

  /** Completion code - Row added successfully. */
  public static final String IFSCC_SUCCESS = "0";

  /** Completion code - Row operation was not successful. */
  public static final String IFSCC_FAILURE = "1"; 

  /**
   * Completion code - DB2 is not available or unexpected SQL return code 
   * received.
  */
  public static final String IFSCC_SYSTEM  = "2"; 

  /* ------------------------------------------------------------------------ *
   * Reason codes                                                             *
   * ------------------------------------------------------------------------ */

  /** Success */
  public static final String IFSRC_SUCCESS = "0";

  /**
   * Data failed validation, sense data indicates which variable failed.
  */
  public static final String IFSRC_INVALID_PARM_VALUE = "1"; 

  /**
   * DB2 is not available or unexpected SQL return code. Sense data includes
   * formatted sqlca data or additional text explaining the error.
  */
  public static final String IFSRC_SYSTEM_DB2 = "2"; 

  /** The row was not found. */
  public static final String IFSRC_DB_NORECORD = "3"; 

  /* ------------------------------------------------------------------------ *
   * Status codes, codes 1 through 11 are defined in IfsTxn                   *
   * ------------------------------------------------------------------------ */

  /**  */
  public static final int SQL_EXCEPTION = 12;

  /**  */
  public static final int INVALID_SQL_STRING = 13;

  /**  */
  public static final int INVALID_COLUMN_NAME = 14;

  /**  */
  public static final int CONNECTION_ERROR = 15;

  /**  */
  public static final int AUTHORIZATION_ERROR = 16;

  /**  */
  public static final int DATA_CORRUPTION_ERROR = 17;

  /**  */
  public static final int SPECIAL_REROUT_REQUIRED = 18;

  /**  */
  public static final int NO_ROWS_CHANGED = 19;

  /**  */
  public static final int NO_ROWS_RETURNED = 20;

  /**  */
  public static final int MISSING_STRING_DELIMETER = 21;

  /** Default DataSource name. */
  private static String defaultDataSourceName_ =
    IfsSystem.getIfsSystemProperty(IfsSystem.JDBC_DATASOURCE_PROP_NAME); 

  /**  
   * The javax.naming.InitialContext class name used to retrieve the 
   * javax.sql.DataSource Object containing our JDBC connections. 
  */
  private static String initialContext_ =
    IfsSystem.getIfsSystemProperty(IfsSystem.JDBC_INITIAL_CONTEXT_PROP_NAME);

/* -------------------------- Instance Variables ---------------------------- */

  /** Indicates multiple SQLExceptions are chained together. */
  private boolean moreExceptions = false;

  /**
   * Indicates an SQLWarning has been returned and stored. The getWarning
   * method can be used to obtain this object
  */
  private boolean areWarnings = false;

  /** Flag for enabling or disabling parameter validation. */
  protected boolean validateActionRequired = 
    IfsSystem.IS_VALIDATE_ACTION_REQUIRED;

  /** Flag for enabling or disabling request ready action. */
  protected boolean requestReadyActionRequired = 
    IfsSystem.IS_REQUEST_READY_ACTION_REQUIRED;

  /** Flag for enabling or disabling response ready action. */
  protected boolean responseReadyActionRequired = 
    IfsSystem.IS_RESPONSE_READY_ACTION_REQUIRED;

  /** Set to false if an update fails. */
  private boolean updateSuccessful = true;

  /** Set to false if a query fails. */
  private boolean querySuccessful = true;

  /** Set if a read method is called within a subclass. */
  protected boolean readTxn = false;

  /** Set if a create method is called within a subclass. */
  protected boolean createTxn = false;

  /** Set if an update method is called within a subclass. */
  protected boolean updateTxn = false;

  /** Set if a delete method is called  within a subclass. */
  protected boolean deleteTxn = false;

  /** Set if a list method is called  within a subclass. */
  protected boolean listTxn = false;

  /** SQL string set in sublasses to issue updates/queries. */
  protected String sqlString = null;

  /** Used to store/access registered methods. */
  private static Hashtable registry = new Hashtable();

  /** Debug flag. */
  public static boolean debugOn = 
    Boolean.valueOf(IfsSystem.getIfsSystemProperty(IfsSystem.DEBUG_PROP_NAME)).booleanValue();

  /** Contains query results. */
  private  ResultSet resSet = null;
  
  /** Used to connect to the database. */
  protected Connection connection = null;

  /** Provides access to the executeQuery and executeUpdate methods. */
  protected Statement statement = null;

  /** Provides access to the executeQuery and executeUpdate methods. */
  protected PreparedStatement preparedStatement = null;

  /**  The JDBC DataSource name. This is defined in WebSphere 4. */
  private String dataSourceName = defaultDataSourceName_;

  /**
   * SQLWarnings can be returned when a connection is made, or within a 
   * ResultSet.
  */
  private SQLWarning warnings = null;

/* ---------------------------- Constructors -------------------------------- */
  
  /**
   * Default constructor method.
   * <P>
   * Use if session cache and locale specific information are not required.
  */
  public IfsJDBC() { 
    super();
  }

  /**
   * Constructor method that takes an IfsSession object
   * <P>
   * @param ifsSession  An instance of IfsSession.
   * <P>
   * Use if session cache and/or locale specific information is required.
   * Example: currency formats are to be based on the locale stored in the 
   * current user session. This constructor must also be used if using the 
   * "handleErrors" servlet method defined in IfsHttpServlet.
  */
  public IfsJDBC(IfsSession ifsSession) {
    super(ifsSession);
  }

/* -------------------------- Instance Methods ----------------------------- */

  /**
   * SQLWarnings can be returned when a connection is made or within a 
   * ResultSet.
  */
  public SQLWarning getWarning() {
    return warnings;
  }

  /**
   * This method is used to determine if there are more than one exception 
   * chained together when an SQLException is thrown.
   * <P>
   * @return moreExceptions
   *    Returns true when there are more SQLExceptions in the chain
  */
  public boolean areMoreExceptions() {
    return moreExceptions;
  }

  /**
   * This method is used to determine if an SQLWarning object has been 
   * retrieved. If so, the getWarning() method can be be used to retrieve it. 
   * Once the object has been obtained, the getNetxWarning() method of the 
   * SQLWarning class can be used to retrieve the next warning in the chain.
   * <P>
   * @return  boolean true when there are more SQLWarning in the chain.
  */
  public boolean areWarnings() {
    return areWarnings;
  }

  /**
   * This method can be used to retrieve the next SQLWarning object in a chain 
   * of warnings if one exists. This method calls the getNetxWarning() method of 
   * the SQLWarning class can be used to retrieve the next warning in the chain
   * @return SQLWarning The next SQLWarning if there are more in the chain,
   *         otherwise, null is returned.
  */
  public SQLWarning getNextWarning() {
    return 
      (warnings != null && areWarnings()) ? warnings.getNextWarning() : null;
  }

  /**
   * Calls the close() method of java.sql.ResultSet to release database and JDBC 
   * resources associated with the ResultSet instance variable. This variable 
   * will be set when a query is performed.
   * <P>
   * @exception SQLException
  */
  public void closeResultSet() throws SQLException, IfsException {
    try {
      if (resSet != null)
        resSet.close();
    }
    catch (SQLException e) {
      debug("in closeResultSet() method: an exception was thrown by " +
            "resSet.close()" + e.toString());
      throw e;
    }
  }

  /**
   * This method calls the close() method of java.sql.Connection to close an 
   * active database connection. It also calls the close() method of 
   * java.sql.Statement class. When a Statement is closed, its current ResultSet, 
   * if one exists, is also closed.
  */
  public void close() throws SQLException {
    try {
      debug("in close method: calling connection.close()");
      if (connection != null) {
        connection.close();
	  connection = null;
        statement = null;
        preparedStatement = null;
        resSet = null;
      }
    }
    catch (SQLException e) {
      debug("in close() method: an exception was thrown by  connection.close() " 
            + e.toString());
      throw e;
    }
  }

  /**
   * Returns the name of the DataSource, as defined in the EJS container,
   * that is being used to obtain pooled connections.
   *
   * @return The data source name.
  */
  public String getDataSourceName() {
    return dataSourceName;
  }

  /**
   * Set the DataSource name used to obtain pooled JDBC connections.
   * <CODE>establishConnection</CODE> is called immediately after the
   * DataSource name has been set. This results in all outstanding
   * Statements being closed, therefore, this method should only be
   * called when work with the current Statements and connection is
   * completed.
   *
   * @return void
   * @exception IfsException If <CODE>establishConnection</CODE>
   *            throws an instance.
   * @exception IllegalArgumentException If <CODE>newName</CODE>
   *            is null.
   * @see #establishConnection()
  */
  public void setDataSourceName(String newName) throws IfsException {
    if (newName == null) {
      throw new IllegalArgumentException(getClass().getName() + 
        ".setDataSourceName: must supply a non-null data source name.");
    }
    dataSourceName = newName;

    establishConnection();
  }

  /**
   * A protected method called to establish an initial connection to the 
   * database. The connection is obtained from a pool created by WebSphere.
   * A handle to the pool is retrieved from the naming service. The name of the
   * DataSource is specified in the jdbc_data_source property in the
   * IfsSystem.properties file. If the property is not set, or there is a 
   * problem retrieving the property an SQLException will be thrown. Existing
   * Connections and their related Statements and ResultSets are closed before
   * retrieving the new Connection.
   * <P>
   * The setErrorStatus method is called within establishConnection to proccess 
   * SQLExceptions. An SQLWarning object may also be set if a problem is 
   * encountered during the connection  process. The getSqlWarning method can 
   * be used to obtain this object.
  */
  protected boolean establishConnection() throws IfsException {

    // Check that the DataSource and InitialContext names were retrieved
    // from the properties file.
    if (dataSourceName == null || initialContext_ == null) {
      String message =
        "Problem retrieving the DataSource name and/or InitialContext name " +
        "from the IfsSystem properties file\n"       +
        "DataSource properties name: " + 
          IfsSystem.JDBC_DATASOURCE_PROP_NAME + "\n" +
        "DataSource value: "               + dataSourceName           + "\n" +
        "InitialContext properties name: " + 
          IfsSystem.JDBC_INITIAL_CONTEXT_PROP_NAME + "\n" +
        "InitialContext value: "           + initialContext_          + "\n";
      debug(message);
      throw new IfsException("java.sql.SQLException", message);
    }

    // Close the existing connection if it hasnt been done yet
    try {
      close();
    }
    catch (SQLException e) {
      debug(getClass().getName() + ".establishConnection():\n"   +
            "\tcaught the following Exception while attempting " +
            "to close current connection\n"                      +
		"\t" + e.getClass().getName() + ": " + e.getMessage());
    }
    
    boolean connectionFlag = true;
    Boolean bool = null;
    SQLWarning warn = null;

    try {
      // WebSphere 4 connection pooling code
      Properties parms = new Properties();
      parms.setProperty(Context.INITIAL_CONTEXT_FACTORY, initialContext_);
      Context ctx = new InitialContext(parms);
      javax.sql.DataSource ds =(javax.sql.DataSource)ctx.lookup(dataSourceName);
      connection = ds.getConnection();
    }
    catch (Throwable e) {
      connectionFlag = false;
      throw new IfsException(e, "Error in establishConnection " + e.toString());
    }

    try {
      warnings = warn = connection.getWarnings ();
    }
    catch (SQLException e) {
      setErrorStatus(e);
      debug("<Connection>.getWarnings() has thrown an exception in " +
            "establishConnection()" + e.toString());
      connectionFlag = false;
      throw new IfsException(e, "connection.getWarnings has thrown an " +
        "exception in establishConnection " + e.toString());
    }

    if (warn != null) {
      debug("\n *** Warning ***\n");
      while (warn != null) {
        debug(
          "SQLState: " + warn.getSQLState()  + "\n" +
          "Message:  " + warn.getMessage()   + "\n" +
          "Vendor:   " + warn.getErrorCode() + "\n\n");
        warn = warn.getNextWarning ();
      }
    }

    if (connection == null) {
      debug("in establishConnection(), before connection.prepareStatement: " +
            "connection object is null");
    }

    try {
      if (createTxn || updateTxn) {
        preparedStatement =  connection.prepareStatement(sqlString);
      }
      else if (listTxn || readTxn || deleteTxn) {
        statement =  connection.createStatement();
      }
      else {
        debug(" In establishConnection. Invalid txn type ");
      }
    }
    catch (Exception ex) {
      debug("<Connection>.createStatement() has thrown an exception in " + 
            "establishConnection" + ex.toString());
      throw new IfsException(ex, "connection.createStatement has thrown " +
        "an exception in establishConnection" + ex.toString());
    }

    debug("exiting establishConnection()");

    return connectionFlag;
  }

  /**
   * This method is used to perform various database operations.
   * If desired, it can call registered/default methods such as validate,
   * requestReadyAction, and responseReadyAction. It also calls
   * the invokesDatabase method to issue database queries/updates.
   * <P>
   * Finally, the setErrorStatus method is called to handle any errors that
   * may occur. The execute method is called within all subclass methods that 
   * perform database operations (create, read, list, delete, update)
  */
  public void execute() throws IfsException {
    debug("<BR>Entry to execute.");

    String msg = null;

    try {
      //try to establish a connection, if one does not exist
      if (!(createTxn || updateTxn)) {
        if (!establishConnection()) {
          // setErrorStatus is called within establishConnection
          return;
        }
      }

      // Perform validate action if required.
      if (isValidateActionRequired()) {
        debug("I'm getting into the isValidationActionRequired block.");
        msg = validate();
        if (msg != null && !msg.equals("OK")) {
          status = VALIDATION_ERROR;
          message = msg;
          debug("I'm getting into the isValidationActionRequired block: " +
                "message = " + message);
          throw new IfsException("Validation Error",msg);
        }
      }

      // Perform requestReadyAction action if required.
      if (isRequestReadyActionRequired()) {
        msg = requestReadyAction();
        if (msg != null && !msg.equals("OK")) {
          status = REQUEST_READY_ACTION_ERROR;
          message = msg;
          debug("Getting into request ready action required block in " +
                "IfsJDBC. Message = " + message);
          //return;
        }
      }

      // Initialize/reset "status"
      status = OK;

      // invoke the db2 database
      debug("<BR>Calling invokeDataBase().");
      invokeDataBase();

      // we now  examine for errors
      debug("<BR>Calling setErrorStatus().");
      setErrorStatus();
      debug("<BR>After setErrorStatus().");

      // Perform responseReadyAction action if required.
      if (isResponseReadyActionRequired()) {
        msg = responseReadyAction();
        if (msg != null && !msg.equals("OK")) {
          if (status == OK) {
            status = RESPONSE_READY_ACTION_ERROR;
          }
          message = message + "  responseReadyActionErrorMassage = " + msg;
          debug("Getting into request ready action required block in " +
                "IfsJDBC. Message = " + message);
          //return;
        }
      }

      debug("Exit from execute.");
    }
    catch (SQLException e){
      String exceptionMsg = null;
      debug("In execute SQLException catch: Exception = ." +e.toString());
      setErrorStatus(e);
      //status = REDO_ERROR;
      if (message != null) {
        exceptionMsg = message;
      }
      else {
        exceptionMsg = "execute has thrown an exception " + e.toString();
      }
      debug(exceptionMsg);
      throw new IfsException(e, exceptionMsg);
    }
    catch (Throwable e) {
      String exceptionMsg = null;
      status = THROWABLE_ERROR;
      if (message != null) {
        exceptionMsg = message;
      }
      else {
        exceptionMsg = "execute has thrown an exception " + e.toString();
      }
      debug(exceptionMsg);
      throw new IfsException(e, exceptionMsg);
    }
  }

  /**
   * This method overrides the finalize method of the java.lang.Object class.
   * It explicitly closes the database connection and calls super.finalize when 
   * garbage collection occurs.
   * <P>
   * @exception Throwable
  */
  protected void finalize() throws Throwable {
    // explicitly closes the connection.It will call super.finalize
    close();  //close the connection and the statement
    Boolean temp = new Boolean(isClosed());
    debug("in finalize: calling close: isClosed = " + temp.toString());
    super.finalize();
  }

  /**
   * This method returns the completion code set by setErrorStatus
   * <P>
   * Valid values include <BR>
   * IFSCC_SUCCESS<BR>
   * IFSCC_FAILURE <BR>
   * IFSCC_SYSTEM <BR>
   * <P>
   * @return  compCode
  */
  public String getCompCode() {
    return compCode;
  }

  /**
   * This method returns the instance of the java.sql.Connection stored in this 
   * class.
   * <P>
   * @return  Connection An object of type java.sql.Connection
  */
  public Connection getConnection() {
    return connection;
  }

  /**
   * This method returns the registry that stores registered methods.
   * A registered method can be used to override a default implementation.
   * Common registry methods include:
   * <UL>
   * <LI>The validator method</LI>
   * <LI>The requestReadyAction method</LI>
   * <LI>The responseReadyAction method</LI>
   * </UL>
   * These methods are invoked within the execute method.
   * <P>
   * @return registry - A hashtable used to access registered methods..
  */
  public static Hashtable getRegistry() {
    return registry;
  }

  /**
   * This method returns the java.sql.ResultSet instance that is stored within 
   * this class. This object is populated with data obtained from a database 
   * query. It is set within subclass methods that perform such queries.
   * <P>
   * @return resSet - The java.sql.ResultSet instance that is set by a database
   *                  query.
  */
  protected ResultSet getResultSet() {
    return resSet;
  }

  /**
   * This method returns the sqlString property that used to issue a database 
   * query/update. The sqlString must be set in subclasses of IfsJDBC before the 
   * execute() method is called.
   * <P>
   * @return  sqlString -- This is the actual sql command that is sent to the 
   *          database.
  */
  public String getSqlString() {
    return sqlString;
  }

  /**
   * This method calls the executeQuery or executeQuery of the Statement class
   * to perform various database operations. It is called within the execute 
   * method of this class. Therefore, it is called anytime a subclass method 
   * issues a transaction (e.g. create(), read(), delete() list())
   * <P>
   * @exception -- throws an SQLException
  */
  protected void invokeDataBase() throws SQLException {
    int rowsAffected = 0;
    if (sqlString == null) {
      status = REDO_ERROR; //INVALID_SQL_STRING;
      ResourceBundle bundle = ResourceBundle.getBundle(
        "com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
      message = bundle.getString(
        "IfsJDBC.invokedatabase.validation.null.sql.string");
      return;
    }

    debug("In invokeDataBase() - SQLString value is: " + sqlString);
    if (readTxn) {
      debug("<BR>DEBUG: We are calling statement.executeQuery for readTxn");
      readTxn = false;
      resSet = statement.executeQuery(sqlString);
      if (resSet == null) {
        querySuccessful = false;
        compCode =   IFSCC_SUCCESS;
        reasonCode =  IFSRC_DB_NORECORD;
      }
      debug("<BR>DEBUG: After calling executeQuery, querySuccessful = " + 
            querySuccessful);
    }
    else if (createTxn) {
      debug("<BR>DEBUG: We are calling preparedStatement.executeUpdate for " +
            "createTxn");
      createTxn = false;
      rowsAffected = preparedStatement.executeUpdate();
      if (rowsAffected == 0) {
        updateSuccessful = false;
      }
      debug("<BR>DEBUG: After calling preparedStatement.executeUpdate for "  +
            "createTxn: rowsAffected = " + rowsAffected);
    }
    else if (updateTxn) {
      debug("<BR>DEBUG: We are calling preparedStatement.executeUpdate for " +
            "updateTxn");
      updateTxn = false;
      rowsAffected = preparedStatement.executeUpdate();
      if (rowsAffected == 0) {
        updateSuccessful = false;
      }
      debug("<BR>DEBUG: After calling preparedStatement.executeUpdate for "  +
            "updateTxn: rowsAffected = " + rowsAffected);
    }
    else if (deleteTxn) {
      debug("<BR>DEBUG: We are calling statement.executeUpdate for deleteTxn");
      deleteTxn = false;
      rowsAffected = statement.executeUpdate(sqlString);
      if (rowsAffected == 0) {
        updateSuccessful = false;
      }
      debug("<BR>DEBUG: After calling executeUpdate, updateSuccessful = " + 
            updateSuccessful);
    }
    else if (listTxn) {
      debug("<BR>DEBUG: We are calling statement.executeQuery for listTxn");
      listTxn = false;
      resSet = statement.executeQuery(sqlString);
      if (resSet == null) {
        querySuccessful = false;
        compCode =   IFSCC_SUCCESS;
        debug("<BR>DEBUG:in invokeDataBase in listTxn: resSet = null");
        reasonCode =  IFSRC_DB_NORECORD;
      }
      debug("<BR>DEBUG: After calling executeQuery, querySuccessful = " + 
            querySuccessful);
    }
    else {
      debug("<BR>DEBUG: **********No Txn flag was set********");
      status = NO_EXECUTION;
    }
  }

  /**
   *
  */
  protected void resetTxnTypes() {
    listTxn   = false;
    deleteTxn = false;
    readTxn   = false;
    updateTxn = false;
    createTxn = false;
  }

  /**
   * This method calls the isClosed() method of the java.sql.Connection class to 
   * check whether a database connection is still active.
   * <P>
   * @return  connectionFlag -- This is set to true if the connection is 
   *          inactive. Otherwise it is set to false.
  */
  public boolean isClosed() throws IfsException {
    boolean connectionFlag = false;

    if (connection == null) {
      connectionFlag = true;
      debug("in isClosed: connection == null");
    }
    else {
      try {
        if (connection.isClosed()) {
          connectionFlag = true;
          debug("in isClosed: calling connection.isClosed()");
        }
      }
      catch (Exception e) {
        //put debug stmnt here
        String exceptionMsg = "isClosed has thrown an exception " + 
          e.toString();
        debug(exceptionMsg);
        throw new IfsException(e, exceptionMsg);
      }
    }
    return connectionFlag;
  }

  /**
   * This method returns the boolean value requestReadyActionRequired. If this 
   * is set to true, then a requestReadyAction method  - defualt or registered - 
   * will be called within the execute method. If it is false, then the method 
   * is not invoked by the execute method.
   * <P>
   * @return  requestReadyActionRequired -- A protected boolean that is set to 
   *          ensure that the requestReadyAction method is called.
  */
  public boolean isRequestReadyActionRequired() {
    return requestReadyActionRequired;
  }

  /**
   * This method returns the boolean value responseReadyActionRequired. If this 
   * is set to true, then a responseReadyAction method  - default or registered 
   * - will be called within the execute method. If it is false, then the method 
   * will not be invoked by the execute method.
   * <P>
   * @return  responseReadyActionRequired -- A protected boolean that is set to 
   *          ensure that the responseReadyAction method is called.
  */
  public boolean isResponseReadyActionRequired() {
    return responseReadyActionRequired;
  }

  /**
   * This method returns the boolean value validateActionRequired. If this is 
   * set to true, then a validator method - default or registered - will be 
   * called within the execute method. If it is false, then the method will not 
   * be invoked by the execute method.
   * <P>
   * @return  validateActionRequired -- A protected boolean that is set to 
   *          ensure that the validator() method is called.
  */
  public boolean isValidateActionRequired() {
    return validateActionRequired;
  }

  /**
   * This method returns the java.sql.Statement object stored within this class. 
   * This instance is used to issue queries/updates to the DB2 database using 
   * the executeQuery and executeUpdate methods.
   * <P>
   * @return  statement -- The java.sql.Statement object stored within this 
   *          class
  */
  public Statement getStatement() {
    return statement;
  }

  /**
  * This method is used to set the java.sql.Statement object stored within this class. This instance is used to
  * issue queries/updates to the DB2 database using the executeQuery and executeUpdate methods.
  * <P>
  * @param  stment -- The java.sql.Statement object stored within this class
  */
  public void  setStatement(Statement stment)
  {
    statement = stment;
  }

  /**
   * This method returns the java.sql.PreparedStatement object stored within 
   * this class. This instance is used to issue queries/updates to the DB2 
   * database using the executeQuery and executeUpdate methods.
   * <P>
   * @return  preparedStatement -- The java.sql.PreparedStatement object stored 
   *          within this class
  */
  public PreparedStatement getPreparedStatement() {
    return preparedStatement;
  }

  /**
   * This method is used to set the java.sql.PreparedStatement object stored 
   * within this class. This instance is used to issue queries/updates to the 
   * DB2 database using the executeQuery and executeUpdate methods.
   * <P>
   * @param  stment -- The java.sql.PreparedStatement object stored within 
   *         this class
  */
  public void  setPreparedStatement(PreparedStatement stment) {
    preparedStatement = stment;
  }

  //this data should be migrated to 3 property files

  /**
   * This method sets the status, completion code, and reason code properties 
   * based on the SQLState returned within an SQLException. SQL states are 
   * standardized error codes. This method maps them to IFS specific error 
   * codes.  These codes can then be displayed on the errorPage passed to the
   * handleErrors servlet function.
   * @param  SQLState -- The sql state obtained from an SQLException
  */
  protected void processSQLState(String SQLState) {
    compCode   = IFSCC_SUCCESS;
    reasonCode = IFSRC_SUCCESS;
    status     = OK;

    debug("In processSQLState");
    if (SQLState == null) {
    }
    else if (SQLState.equals("00000")) {
      // Successful
      compCode   = IFSCC_SUCCESS;
      reasonCode = IFSRC_SUCCESS;
      status     = OK;
    }
    else if (SQLState.equals("01515")) { 
      // Overflow occurrred
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01547")) {
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01534")) { 
      // The string representation of a date,
      // time or timestamp value is not in the acceptable
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01519")) {   
      //data was either not valid or too large
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01004")) { 
      // value that is longer than
      // the maximum length
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01002")) { 
      // An error occurred during
      // DISCONNECT of a relational database
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01535")) { 
      // The result of an arithmetic operation is a date or timestamp
      // that is not within the valid range
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01536")) { 
      // invalid database name
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01539")) { 
      // invalid connection
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01542")) { 
      // The operation cannot be performed without the required authority
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01544")) {
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01548")) { 
      // object operation cannot be performed
      // without the required authority.
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("01564")) { 
      // data was either not valid or too large
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("01593")) { 
      // A table was altered, resulting in the reduction of the 
      // length of a column
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("02000")) { 
      // No rows were updated, inserted, or deleted
      compCode   = IFSCC_FAILURE;
      reasonCode = IFSRC_DB_NORECORD;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("08001")) { 
      // A communication error occurred,
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("08002")) { 
      // An attempt was made to CONNECT to a relational database
      // when the connection is active.
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("08003")) { 
      // relational database inactive
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = FATAL_ERROR;
    }
    else if (SQLState.equals("24000")) { 
      // DB2Exception range
      debug("<BR>In processSQLState: SQLState = 24000");
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("22001")) { 
      // DB2Exception: sql parm exceeds required length range
      debug("<BR>In processSQLState: SQLState = 22001");
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = REDO_ERROR;
    }
    else if (SQLState.equals("42603")) { 
      // DB2Exception: delimeter error range
      debug("<BR>In processSQLState: SQLState = 42603");
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = MISSING_STRING_DELIMETER;
    }
    else if (SQLState.equals("22007")) { 
      // DB2Exception: delimeter error range
      debug("<BR>In processSQLState: SQLState = 22007");
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_INVALID_PARM_VALUE;
      status     = MISSING_STRING_DELIMETER;
    }
    else {
      debug("In processSQLState else case");
      compCode   = IFSCC_SYSTEM;
      reasonCode = IFSRC_SYSTEM_DB2;
      status     = REDO_ERROR;
    }
  }

  /** Customizable method to process bank specific logic before executing a 
   * database operation. It returns "OK" if the action is successful. Otherwise, 
   * it returns a message string indicating the error.  If the registered method 
   * is not found, it returns  null.
   * <P>
   * @return msg -- If a registered method exists, it's message will be 
   *         returned. Otherwise null is returned.
  */
  public String requestReadyAction() {
    String msg = "";
    try {
      // Invoke requestReadyAction method in the registries whichever is found 
      // first from the concrete transaction class up to the IfsJDBC class.
      msg = (String)invokeMethod("requestReadyAction", new Object[]{this}, "IfsJDBC");
    }
    catch (Exception e) {
      msg = null;
    }

    return msg;
  }

  /** 
   * Customizable method to process bank specific logic after the transaction is
   *  executed. It returns "OK" if the action is successful.  Otherwise, it 
   * returns a message string indicating the error.  If the registered method is 
   * not found, it returns  null.
   * <P>
   * @return msg -- If a registered method exists, it's message will be 
   *         returned. Otherwise null is returned.
  */
  public String responseReadyAction() {
    String msg = "";
    try {
      // Invoke responseReadyAction method in the registries whichever is found 
      // first from the concrete transaction class up to the IfsJDBC class.
      msg = (String)invokeMethod(
        "responseReadyAction", new Object[]{this}, "IfsJDBC");
    }
    catch (Exception e) {
      msg = null;
    }
    return msg;
  }

  /**
   * This method sets error handling variables, based on error data returned 
   * from an sql query. If a ResultSet object returned from a query contains an 
   * instance of the  SQLWarning class, the information contained in it will be 
   * extracted and made available to the servlet writer. The setErrorStatus 
   * method is called within the execute method.
  */
  private void  setErrorStatus() throws IfsException {
    // Initialize "results" data
    //status = IfsTxn.OK;
    errorCode = 0;
    compCode = IFSCC_SUCCESS;
    reasonCode = IFSRC_SUCCESS;
    errorFunctionName = null;
    errorModuleName = null;
    gmMessage = null;
    message = null;
    senseData = null;
    senseDataLength = 0;

    try {
      if (resSet != null && resSet.getWarnings() != null) {
        //report warnings
        SQLWarning warn = null;
        debug ("\n *** Warning ***\n");
        warn = resSet.getWarnings();
        warnings = warn;
        processSQLState(warn.getSQLState ());
        while (warn != null)
        {
          debug ("SQLState: " + warn.getSQLState ());
          debug ("Message:  " + warn.getMessage ());
          debug ("Vendor:   " + warn.getErrorCode ());
          debug ("");
          warn = warn.getNextWarning ();
        }
      }

      if (!querySuccessful) {
        ResourceBundle bundle = ResourceBundle.getBundle(
          "com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
        compCode =   IFSCC_SUCCESS;
        reasonCode =  IFSRC_DB_NORECORD;
        status = REDO_ERROR;
        message =bundle.getString("IfsJDBC.update.validation.no.rows.returned");
        querySuccessful = true;
        throw new IfsException("No Records Returned",message);
      }

      if (!updateSuccessful) {
        ResourceBundle bundle = ResourceBundle.getBundle(
          "com.ibm.ifs.resource.IfsSystemErrorResourceBundle",getLocale());
        compCode =   IFSCC_SUCCESS;
        reasonCode =  IFSRC_DB_NORECORD;
        status =  REDO_ERROR;
        //message =  bundle.getString("IfsJDBC.update.validation.no.rows.updated");
        message = "The SQL sent did not affect any record of the database.";
        updateSuccessful = true;
        throw new IfsException("No Records Affected",message);
      }
    }
    catch (SQLException e) {
      //set error codes
      String exceptionMsg = "setErrorStatus(e) has thrown an exception " + 
        e.toString();
      debug(exceptionMsg);
      throw new IfsException(e, exceptionMsg);
    }
  }

  /**
   * This method sets error handling variables, based on error data returned 
   * from an sql query or update. Since this method accepts an instance of 
   * SQLException, it is meant to be called within catch blocks. If a ResultSet 
   * object returned from a query contains an instance of the  SQLWarning class,
   * the information contained in it will be extracted and made available to the 
   * servlet writer. The setErrorStatus method is called within the execute 
   * method. All required error handling properties are set within this method - 
   * e.g. message, errorCode, errorFunctionName, etc.
   * <P>
   * @param  e -- An SQLException object.
  */
  private void  setErrorStatus(SQLException e) throws IfsException {
    String state = null;

    // Initialize "results" data

    //status = IfsTxn.OK;
    errorCode = 0;
    compCode = IFSCC_SUCCESS;
    reasonCode = IFSRC_SUCCESS;
    errorFunctionName = null;
    errorModuleName = null;
    gmMessage = null;
    message = null;
    senseData = null;
    senseDataLength = 0;

    debug(" Inside setErrorStatus(SQLException e): something has thrown an " +
          "exception " + e.toString());
    try {
      message = e.toString();
      if (resSet != null && resSet.getWarnings() != null) {  
        //listTxn or readTxn
        //report warnings
        SQLWarning warn = null;
        debug ("\n *** Warning ***\n");
        warn = resSet.getWarnings();
        warnings = warn;
        while (warn != null) {
          debug ("SQLState: " + warn.getSQLState ());
          debug ("Message:  " + warn.getMessage ());
          debug ("Vendor:   " + warn.getErrorCode ());
          debug ("");
          warn = warn.getNextWarning ();
        }
      }

      message = e.toString();
      errorCode = e.getErrorCode();
      state =  e.getSQLState();
      errorFunctionName = "execute";
      errorModuleName = "IfsJDBC";

      processSQLState(state); //set comp/reason codes based on the SQLState
      if (e.getNextException() != null) {
        moreExceptions = true;
      }
    }
    catch (SQLException ex) {
      //set error codes
      String exceptionMsg = "setErrorStatus(e) has thrown an exception " +
                            ex.toString();

      debug(exceptionMsg);
      throw new IfsException(ex, exceptionMsg);
    }
  }

  /**
   * This method is used to set the requestReadyActionRequired flag. If this 
   * flag is set to true, then the requestReadyAction() method will be invoked 
   * within the execute method.
   * @param  b -- A boolean value.
  */
  public void setRequestReadyActionRequired(boolean b) {
    requestReadyActionRequired = b;
  }

  /**
   * This method returns the requestReadyActionRequired flag. If this flag is set 
   * to true, then the requestReadyAction() method will be invoked within the 
   * execute method.
   * @return  requestReadyActionRequired -- A boolean value.
  */
  public boolean getRequestReadyActionRequired() {
    return requestReadyActionRequired;
  }

  /**
   * This method is used to set the responseReadyActionRequired flag. If this 
   * flag is set to true, then the responseReadyAction() method will be invoked 
   * within the execute method.
   * @param  b -- A boolean value.
  */
  public void setResponseReadyActionRequired(boolean b) {
    responseReadyActionRequired = b;
  }

  /**
   * This method returns the responseReadyActionRequired flag. If this flag is 
   * set to true, then the responseReadyAction() method will be invoked within 
   * the execute method.
   * @return  responseReadyActionRequired -- A boolean value.
  */
  public boolean getResponseReadyActionRequired() {
    return responseReadyActionRequired;
  }

  /**
   * This method is used to set the java.sql.ResultSet instance within this 
   * class.
   * @param  o. A java.sql.ResultSet object.
  */
  protected void setResultSet(ResultSet o) {
    resSet = o;
  }

  /**
   * This method is used to set the sqlString property within this class. It 
   * should be used within all subclasses of IfsJDBC before the execute method 
   * is called.
   * @param  o -- A valid sql command.
  */
  protected void setSqlString(String o) {
    sqlString = o;
  }

  /**
   * Determines whether the validate method is invoked within the execute 
   * method. If this flag is set to true, then the validator() method will be 
   * invoked within the execute method.
   * @return  new value of the <CODE>validateActionRequired</CODE> flag
  */
  public void setValidateActionRequired(boolean b) {
    validateActionRequired = b;
  }

  /**
   * This method returns the validateActionRequired flag. If this flag is set to 
   * true, then the validate() method will be invoked within the execute method.
   * @return  current value of the <CODE>validateActionRequired</CODE> flag
  */
  public boolean getValidateActionRequired() {
    return validateActionRequired;
  }

  /**
   *
  */
  protected void replace(byte[] byteArray, String oldString, String newString) {
    int startIndex = 0;
    int endIndex;
    boolean done = false;
    StringBuffer editedString = new StringBuffer();
    String originalString = new String(byteArray);
    endIndex = originalString.indexOf(oldString);
    if (endIndex == -1) {
      done = true;
    }

    while (!done) {
      editedString.append(originalString.substring(startIndex, endIndex));
      editedString.append(newString);
      startIndex = endIndex + newString.length();

      endIndex = originalString.indexOf(oldString, startIndex);
      if (endIndex == -1) {
        done = true;
        editedString.append(
          originalString.substring(startIndex, originalString.length()-1));
      }
    }
  }

  /**
   * This method returns a string of name/value pairs, separated by the newline 
   * character. These pairs represent the properties of this class.
   * <P>
   * @return  A string of name/value pairs that represent the properties of 
   *          this class
  */
  public String toString() {
    return hasRegisteredMethod("toString") ? super.toString() : toString("\n");
  }

  /**
   * This method returns a string of name/value pairs, separated by the given 
   * prefix. These pairs represent the properties of this class.
   * <P>
   * @param  prefix -- A string that seperates the pairs.
   * @return  A string of name/value pairs that represent the properties of 
   *          this class
  */
  public String toString(String prefix) {
    if (hasRegisteredMethod("toStringWithString")) {
      return super.toString(prefix);
    }
    else {
      StringBuffer buffer = new StringBuffer(256);
      buffer.append(super.toString(prefix));

      buffer.append(prefix);
      buffer.append("validateActionRequired = ");
      buffer.append(validateActionRequired);

      buffer.append(prefix);
      buffer.append("requestReadyActionRequired = ");
      buffer.append(requestReadyActionRequired);

      buffer.append(prefix);
      buffer.append("responseReadyActionRequired = ");
      buffer.append(responseReadyActionRequired);

      buffer.append(prefix);
      buffer.append("status = ");
      buffer.append(status);

      buffer.append(prefix);
      buffer.append("errorCode = ");
      buffer.append(errorCode);

      buffer.append(prefix);
      buffer.append("brc = ");
      buffer.append(brc);

      buffer.append(prefix);
      buffer.append("message = ");
      buffer.append(message);

      buffer.append(prefix);
      buffer.append("gmMessage = ");
      buffer.append(gmMessage);

      buffer.append(prefix);
      buffer.append("compCode = ");
      buffer.append(compCode);

      buffer.append(prefix);
      buffer.append("reasonCode = ");
      buffer.append(reasonCode);

      buffer.append(prefix);
      buffer.append("errorFunctionName = ");
      buffer.append(errorFunctionName);

      buffer.append(prefix);
      buffer.append("errorModuleName = ");
      buffer.append(errorModuleName);

      buffer.append(prefix);
      buffer.append("senseData = ");
      buffer.append(senseData);

      buffer.append(prefix);
      buffer.append("senseDataLength = ");
      buffer.append(senseDataLength);

      return buffer.toString();
    }
  }

  /**
   * This method returns the Busineess Response Code property (brc). At present, 
   * this property is not set by this class. However, it is needed by current 
   * the errorPage servlet.
   * <P>
   * @return brc -- The brc property of this class
  */
  public String getBrc() {
    return brc;
  }

} // End IfsJDBC class
