package com.ibm.ifs.gold;

//+---------------------------------------------------------------------------------
//|
//|  FILE NAME:   IfsNLS.java
//|
//|  COPYRIGHT:
//|
//|   Licensed Materials - Property of IBM
//|   5648-A06
//|   (C) Copyright IBM Corp. 2000 All Rights Reserved
//|    US Government Users Restricted Rights - Use, duplication
//|    or disclosure restricted by GSA ADP Schedule Contract
//|    with IBM
//|
//|
//|  d6556,d6557  larockc  25-Apr-2000 Modified getString() to return "" when passed
//|                  null key, and to try default code value "ZZZ"
//|                  when key not found.
//|  d6676  rhs  04/26/2000  In getString(), corrected check for empty string
//|                          and added IfsException message for invalid keys.
//|  d6707  rhs  05/04/2000  Added getDefaultCountryCode and populateDefaultCountryCode
//|                          methods.
//|  d6684  rhs  05/08/2000  Added use of URLEncoder.encode() to find keys with blanks
//|                          in the resource bundles.
//|  d6721  rhs  05/10/2000  Added use of IfsHttpServlet.toUnicodeString() to encode
//|                          property file strings from UTF-8 encoding to unicode.
//|  d6743  kris 05/15/2000  Added routines to locate appropriate resource bundle
//|
//|  dT1000070 mbc  10/19/2000  Added use of local toUnicodeString() to encode
//|                             property file strings from UTF-8 encoding to unicode.
//|
//+----------------------------------------------------------------------------------

import java.io.UnsupportedEncodingException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Vector;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

import com.ibm.ifs.gold.IfsSystem;

/**
 * IfsNLS is a NLS utility class to extract the text resource (property value) from the properties
 * file given the <code>code</code> (property name) and the <code>locale</code>.
 * The properties file contains a list of code-to-text mapping, each of which defines a complete
 * description for the corresponding code resource. The class provides some helper methods to extract
 * this textual resource.
 *
 */
public class IfsNLS
{
   /** Contains the available languages and their corresponding default country code. */
   private static Hashtable defaultLangCountryDict = null;

  /** Loads system classes (those in your classpath). */
   private static IfsSystemClassLoader ifsSystemClassLoader = new IfsSystemClassLoader();

   private static final Integer NOTFOUND = new Integer(-1);
   private static Hashtable cacheList = new Hashtable();

  /**
   * This method encodes a String object read from a UTF-8 resource bundle into
   * a valid unicode String object.  For each character in "inString", a byte
   * is created in a byte array and then the byte array is encoded into a
   * String object via the character set represented by IfsSystem.cwsGMJCharSet
   * (which should be UTF-8).
   *
   * @param inString -- The String object to encode.
   * @return
   *    The encoded String object.
   *    A null is returned when either one of the input parameters is null
   *    or when an UnsupportedEncodingException exception occurs.
   */
   public static String toUnicodeString(String inString)
   {
      String unicodeString = null;

      if (inString != null && IfsSystem.cwsGMJCharSet != null)
      {
         try
         {
            int    lenValues = inString.length();
            byte[] encBytes  = new byte[lenValues];
            for (int j = 0; j < lenValues; j++)
            {
              encBytes[j] = (byte)inString.charAt(j);
            }
            unicodeString = new String(encBytes, IfsSystem.cwsGMJCharSet);
         }
         catch (UnsupportedEncodingException e)
         {
            unicodeString = null;
         }
      }

      return unicodeString;
   }

   /**
    * This method returns the text associated with the code, <code>key</code>, that is defined in the resource bundle
    * <code>bundleName</code>. It uses the default locale, <code>IfsSystem.SERVER_LOCALE</code> to retrieve the correct bundle.
    * For example, if the locale identifier of the server locale is <code>en_US</code> then the
    * resource bundle returned is either <code>IfsXXXResources_en_US</code> or <code>IfsXXXResources_en</code> or
    * <code>IfsXXXResources</code>.  The <code>bundleName</code> is a complete and fully qualified name.
    * The method throws an <code>IfsException</code>, whose message contents include the resource bundle name and the locale information.
    * <P>
    * If the specified key does not exist in the resource bundle, a search is made for the default key, <EM>ZZZ</EM> .
    * If neither the specified key or the default key exist, an <EM>IfsException</EM> is thrown for the missing key.
    *
    * @param bundleName    The fully qualified name of the properties file or class name
    * @param key    The code of the textual resource to be extracted
    * @return       Textual resource string.
    * @exception IfsException When the resource bundle is missing or key does not exist.
    */
     public static String getString(String bundleName, String key) throws IfsException
     {
        return getString(bundleName,IfsSystem.SERVER_LOCALE,key);
     }

   /**
    * This method returns the text associated with the code, <code>key</code>, that is defined in the resource bundle
    * <code>bundleName</code>. It uses the <code>locale</code> object, to retrieve the correct bundle.
    * For example, if the locale identifier of the server locale is <code>en_US</code> then the
    * resource bundle returned is either <code>IfsXXXResources_en_US</code> or <code>IfsXXXResources_en</code> or
    * <code>IfsXXXResources</code>.  The <code>bundleName</code> is a complete and fully qualified name.
    * The method throws an <code>IfsException</code>, whose message contents include the resource bundle name and the locale information.
    * <P>
    * If the specified key does not exist in the resource bundle, a search is made for the default key, <EM>ZZZ</EM> .
    * If neither the specified key or the default key exist, an <EM>IfsException</EM> is thrown for the missing key.
    *
    * @param bundleName    The fully qualified name of the properties file or class name
    * @param key    The code of the textual resource to be extracted
    * @param locale  The Locale object to identify the correct resource bundle
    * @return       Textual resource string.
    * @exception IfsException When the resource bundle is missing or key does not exist.
    */
     public static String getString(String bundleName, Locale locale,  String key) throws IfsException
     {
        ResourceBundle bundle = null;
        String s = "";

        // Return empty string when key is null or empty.
        if ( (key == null) || (key.equals("")) )
           return(s);

        try { bundle = getBundle(bundleName,locale); }
        catch (MissingResourceException e)
        {
           // The resource bundle does not exist.
           StringBuffer buffer = new StringBuffer();
           buffer.append(" Resource Bundle does not exist -- Name: ");
           buffer.append(bundleName);
           buffer.append(", Language: ");
           buffer.append(locale.getLanguage());
           buffer.append(", Country: ");
           buffer.append(locale.getCountry());
           buffer.append(" .");
           throw new IfsException(e,buffer.toString());
        }

        // Bundle exists now try to get the text for the key.
        // Since the key may have blanks and is NLS sensitive, need to convert
        // to byte array using UTF8 encoding. Then convert back to String and
        // then convert to URL-encoded form before getting the resource string.
        try
        {
           String tempKey = null;
           try { tempKey = new String(key.getBytes("UTF8")); }
           catch (UnsupportedEncodingException eU) { tempKey = key; }

           // The ResourceBundle.getString method does not convert the UTF-8 character
           // codes to Java unicode character codes when the string is obtained
           // from the properties file.  Have to use toUnicodeString
           // method to convert the UTF-8 string retrieved from the properties file.
           //
           s = toUnicodeString(bundle.getString(URLEncoder.encode(tempKey)));
        }
        catch (MissingResourceException e)
        {
           // URL encoded key doesn't exist. Try unmodified key.
           try
           {
              s = toUnicodeString(bundle.getString(key));
           }
           catch (MissingResourceException ee)
           {
              // The key does not exist in the resource bundle.
              // Try to get the default key of ZZZ.  They ZZZ key is used by code
              // refresh with FISIM/MQWEB.
              try
              {
                 s = toUnicodeString(bundle.getString("ZZZ"));
              }
              catch (MissingResourceException eee)
              {
                 // Generate the exception data associated with the key parameter.
                 StringBuffer buffer = new StringBuffer();
                 buffer.append(" Key: ");
                 buffer.append(key);
                 buffer.append(", is not in resource bundle: ");
                 buffer.append(bundleName);
                 buffer.append(", Language: ");
                 buffer.append(locale.getLanguage());
                 buffer.append(", Country: ");
                 buffer.append(locale.getCountry());
                 buffer.append(" .");
                 throw new IfsException(e,buffer.toString());
              }
           }
        }
        return(s);
     }

   /**
    * This method returns the text (property value) associated with the code (property name), <code>key</code>,
    * that is defined in the resource bundle <code>shortBundleName</code> in the <code>com.ibm.ifs.resource package</code>.
    * The method computes to locate the fully qualified resource bundle by concatenating the <code>shortBundleName</code>
    * to the package name <code>com.ibm.ifs.resource</code>. It uses the default <code>Locale</code> object <code>IfsSystem.SERVER_LOCALE</code>
    * to retrieve the correct bundle. For example, if the locale identifier is <code>en_US</code> then the
    * resource bundle returned is either <code>IfsXXXResources_en_US</code> or <code>IfsXXXResources_en</code> or <code>IfsXXXResources</code>.
    * The method throws an <code>IfsException</code>, whose message contents include the resource bundle name and the locale information.
    * <P>
    * If the specified key does not exist in the resource bundle, a search is made for the default key, <EM>ZZZ</EM> .
    * If neither the specified key or the default key exist, an <EM>IfsException</EM> is thrown for the missing key.
    *
    * @param shortBundleName    The name of the properties file located in the com.ibm.ifs.resource package
    * @param key    The code of the textual resource to be extracted
    * @return    Textual resource string.
    * @exception IfsException When the resource bundle is missing or key does not exist.
    */
     public static String getText(String shortBundleName, String key) throws IfsException
     {
       return getText(shortBundleName,IfsSystem.SERVER_LOCALE,key);
     }

   /**
    * This method returns the text (property value) associated with the code (property name), <code>key</code>,
    * that is defined in the resource bundle <code>shortBundleName</code> in the <code>com.ibm.ifs.resource package</code>.
    * The method computes to locate the fully qualified resource bundle by concatenating the <code>shortBundleName</code>
    * to the package name <code>com.ibm.ifs.resource</code>. It uses the <code>locale</code> Locale object
    * to retrieve the correct bundle. For example, if the locale identifier is <code>en_US</code> then the
    * resource bundle returned is either <code>IfsXXXResources_en_US</code> or <code>IfsXXXResources_en</code> or <code>IfsXXXResources</code>.
    * The method throws an <code>IfsException</code>, whose message contents include the resource bundle name and the locale information.
    * <P>
    * If the specified key does not exist in the resource bundle, a search is made for the default key, <EM>ZZZ</EM> .
    * If neither the specified key or the default key exist, an <EM>IfsException</EM> is thrown for the missing key.
    *
    * @param shortBundleName    The name of the properties file located in the com.ibm.ifs.resource package
    * @param key    The corresponding code string of the textual resource to be extracted
    * @param locale  The locale object to identify the correct resource bundle
    * @return       Textual resource string.
    * @exception IfsException When the resource bundle is missing or key does not exist.
    */
     public static String getText(String shortBundleName, Locale locale,  String key) throws IfsException
     {
       return getString("com.ibm.ifs.resource."+shortBundleName,locale,key);
     }

   /**
   * Returns the default country code for the specified language.
   *
   * @param language -- Two character (lower case) language code.
   * @return
   *    Returns a two character (upper case) String of the country code.
   *    Returns an empty string when there is no default country code for
   *    the requested language or the requested language is invalid.
   */
   public static String getDefaultCountryCode(String language)
   {
      String temp = null;

      if (defaultLangCountryDict == null)
      {
         // First time, need to populate the hash table.
         defaultLangCountryDict = new Hashtable();
         populateDefaultLangCountryDict(defaultLangCountryDict);
      }

      try { temp = (String) defaultLangCountryDict.get(language); }
      catch ( Exception e ) { temp = ""; }

      return( (temp != null) ? temp : "" );
   }

   /**
   * Populates the specified hash table with the default country for the available languages.
   *
   * @param langTable -- The hash table to be initialized with the language and
   *    country code pairs.
   */
   private static void populateDefaultLangCountryDict(Hashtable langTable)
   {
      langTable.put("ar", "EG");  // Arabic (Egypt)
      langTable.put("be", "BY");  // Byelorussian (Belarus)
      langTable.put("bg", "BG");  // Bulgarian (Bulgaria)
      langTable.put("ca", "ES");  // Catalan (Spain)
      langTable.put("cs", "CZ");  // Czech (Czech Republic)
      langTable.put("da", "DK");  // Danish (Denmark)
      langTable.put("de", "DE");  // German (Germany)
      langTable.put("el", "GR");  // Greek (Greece)
      langTable.put("en", "US");  // English (United States)
      langTable.put("es", "ES");  // Spanish (Spain)
      langTable.put("et", "EE");  // Estonian (Estonia)
      langTable.put("fi", "FI");  // Finnish (Finland)
      langTable.put("fr", "FR");  // French (France)
      langTable.put("hr", "HR");  // Croatian (Croatia)
      langTable.put("hu", "HU");  // Hungarian (Hungary)
      langTable.put("is", "IS");  // Icelandic (Iceland)
      langTable.put("it", "IT");  // Italian (Italy)
      langTable.put("iw", "IL");  // Hebrew (Israel)
      langTable.put("ja", "JP");  // Japanese (Japan)
      langTable.put("ko", "KR");  // Korean (South Korea)
      langTable.put("lt", "LT");  // Lithuanian (Lithuania)
      langTable.put("lv", "LV");  // Latvian (Lettish) (Latvia)
      langTable.put("mk", "MK");  // Macedonian (Macedonia)
      langTable.put("nl", "NL");  // Dutch (Netherlands)
      langTable.put("no", "NO");  // Norwegian (Norway,B)
      langTable.put("pl", "PL");  // Polish (Poland)
      langTable.put("pt", "PT");  // Portuguese (Portugal)
      langTable.put("ro", "RO");  // Romanian (Romania)
      langTable.put("ru", "RU");  // Russian (Russian Federation)
      langTable.put("sh", "YU");  // Serbo-Croatian (Yugoslavia)
      langTable.put("sk", "SK");  // Slovak (Slovakia)
      langTable.put("sl", "SI");  // Slovenian (Slovenia)
      langTable.put("sq", "AL");  // Albanian (Albania)
      langTable.put("sr", "YU");  // Serbian (Yugoslavia)
      langTable.put("sv", "SE");  // Swedish (Sweden)
      langTable.put("th", "TH");  // Thai (Thailand)
      langTable.put("tr", "TR");  // Turkish (Turkey)
      langTable.put("uk", "UA");  // Ukrainian (Ukraine)
      langTable.put("zh", "CN");  // Chinese (China)
      langTable.put("zh", "TW");  // Chinese (Taiwan)
   }

  /**
   * Get the appropriate ResourceBundle subclass. This method uses <code>findBundle</code> method 
   * to locate the resource bundle. If it cannot the bundle, the method uses <code>getBundle</code> 
   * of <code>ResourceBundle</code> class to locate the default bundle.
   * @param baseName The fully qualified name of the properties file or class name
   * @param locale   The locale object to identify the correct resource bundle
   * @return  Appropriate ResourceBundle subclass.
   */
   public static final ResourceBundle getBundle(String baseName, Locale locale) throws MissingResourceException
   {
     StringBuffer localeName = new StringBuffer("_").append(locale.toString());
     if (locale.toString().equals(""))
       localeName.setLength(0);

     ResourceBundle bundle = findBundle(baseName,localeName,ifsSystemClassLoader,true);
     if (bundle == null) 
       bundle = ResourceBundle.getBundle(baseName,locale);

     return bundle;
   }

  /**
   * The internal routine that does the real work of finding and loading
   * the right ResourceBundle for a given name and locale.
   */
   private static ResourceBundle findBundle(String baseName, StringBuffer localeName, ClassLoader loader, boolean includeBase)
   {
        String localeStr = localeName.toString();
        String baseFileName = baseName.replace('.', '/');
        Object lookup = null;
        String searchName;
        Vector cacheCandidates = new Vector();
        int lastUnderbar;
        InputStream stream;

        searchLoop:
        while (true) {
            searchName = baseName + localeStr;
	    String cacheName =
		"["+Integer.toString(loader.hashCode())+"]" + searchName;

            // First, look in the cache.  We may either find the bundle we're
            // looking for or we may find that the bundle was not found by a
            // previous search.
            lookup = cacheList.get(cacheName);
            if( lookup == NOTFOUND ) {
//                debug("Found " + searchName + " in cache as NOTFOUND");
                localeName.setLength(0);
                break searchLoop;
            }
            if( lookup != null ) {
//                debug("Found " + searchName + " in cache");
                localeName.setLength(0);
                break searchLoop;
            }
            cacheCandidates.addElement( cacheName );

            // Next search for a class
//            debug("Searching for " + searchName );
            try {
                lookup = loader.loadClass(searchName).newInstance();
                break searchLoop;
            } catch( Exception e ){}

            // Next search for a Properties file.
            searchName = baseFileName + localeStr + ".properties";
//            debug("Searching for " + searchName );
            stream = loader.getResourceAsStream(searchName);
            if( stream != null ) {
		// make sure it is buffered
		stream = new java.io.BufferedInputStream(stream);
                try {
                    lookup = (Object)new PropertyResourceBundle( stream );
                    break searchLoop;
                } catch (Exception e) {}
            }

            //Chop off the last part of the locale name string and try again.
            lastUnderbar = localeStr.lastIndexOf('_');
            if( ((lastUnderbar==0)&&(!includeBase)) || (lastUnderbar == -1) ) {
                break;
            }
            localeStr = localeStr.substring(0,lastUnderbar);
            localeName.setLength(lastUnderbar);
        }


        if( lookup != null ) {
            // Add a positive result to the cache. The result may include
            // NOTFOUND
            for( int i=0; i<cacheCandidates.size(); i++ ) {
                cacheList.put(cacheCandidates.elementAt(i), lookup);
//                debug("Adding " + cacheCandidates.elementAt(i) + " to cache"
//                      + ((lookup == NOTFOUND)?" as NOTFOUND.":"."));
            }
        }
        else {
            // If we searched all the way to the base, then we can add
            // the NOTFOUND result to the cache.  Otherwise we can say
            // nothing.
            if( includeBase == true ) {
                for( int i=0; i<cacheCandidates.size(); i++ ) {
                    cacheList.put(cacheCandidates.elementAt(i), NOTFOUND);
//                    debug("Adding " + cacheCandidates.elementAt(i)
//                          + " to cache as NOTFOUND.");
                }
            }
        }

        if( (lookup == NOTFOUND) || (lookup == null) )
            return null;
        else
            return (ResourceBundle)lookup;
   }
}

  /**
   * The IfsSystemClassLoader loads system classes (those in your classpath).
   * This is an attempt to unify the handling of system classes and ClassLoader
   * classes.
   */
  class IfsSystemClassLoader extends java.lang.ClassLoader 
  {
    protected Class loadClass( String name, boolean resolve )
        throws ClassNotFoundException
    {
        return findSystemClass( name );
    }

    public InputStream getResourceAsStream(String name) {
        return ClassLoader.getSystemResourceAsStream(name);
    }

    public java.net.URL getResource(String name) {
        return ClassLoader.getSystemResource(name);
    }
  }

