package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsObject.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//     8/6/1999   kan   d5322R   promoted get*FromURL from IfsTxn.java
//    02/17/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    03/06/2000  rhs   d6423   Made JavaDoc additions.
//    03/14/2000  rhs   d6545   Added NLS support
//    04/14/2000  rhs   d6620   Added catching of getLocale exception.
//
// ***************************************************************************

import java.io.*;
import java.text.Format;
import java.lang.reflect.*;
import java.util.Hashtable;
import java.util.Locale;
import javax.servlet.http.*;
import javax.servlet.http.HttpSession;
import javax.servlet.http.HttpServletResponse;
import com.ibm.ifs.servlets.IfsHttpServlet;

/**
* IfsObject abstracts the common properties and behaviors among all the IFS
* objects.  This includes data validation, internationalization support, caching,
* self defining fields, session, registry, customizable methods, and
* initialization.  Every IFS object should inherit from IfsObject.
*/
public abstract class IfsObject implements Serializable, ICustomizableDataMethods
{

   //--------------------------------------------------------------------------
   // self defining field attributes/methods
   //--------------------------------------------------------------------------

   /**
   * Self defining field request flag for GsGmSelfDefiningFieldDef objects.
   */
   public static final int SD_REQUEST = 1;   // self defining request flag

   /**
   * Self defining field response flag for GsGmSelfDefiningFieldDef objects.
   */
   public static final int SD_RESPONSE = 2;  // self defining response flag

   //--------------
   // Self defining field type flags
   //
   public static final int SD_TYPE_STRUCT = 0;
   public static final int SD_TYPE_LONG = 2;  // this is a gold C "long" which is a java "int"
   public static final int SD_TYPE_DOUBLE = 3;
   public static final int SD_TYPE_CHAR = 4;
   public static final int SD_TYPE_BINARY = 5;
   public static final int SD_TYPE_CURRENCY = 9;
   public static final int SD_TYPE_DATE = 10;

   /**
   * Self defining field account type flag for GsGmSelfDefiningFieldDef objects.
   */
   public static final int SD_TYPE_ACCOUNT = 11;

   /**
   * Contains the response self defining field data.
   */
   protected GsGmSelfDefiningFieldDef[] respSelfDefData = null;

   /**
   * Contains the IfsSession object which carries references to the HTTP request,
   * HTTP response, and the session it is associated with.
   */
   protected IfsSession ifsSession;

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean  debugOn = false;

   /**
   * Gets the registry defined for this class to register customized
   * methods applicable to all IfsObjects.
   */
   private static Hashtable registry = new Hashtable();

   /**
   * Default constructor.
   */
   public IfsObject()
   {
   }

   /**
   * Constructor with an associated IfsSession object.
   */
   public IfsObject(IfsSession session)
   {
      ifsSession = session;
   }

   /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a browser, a log file, or a combination of the previous 3
   * output vehicles. The browser output is in HTML format.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param str -- The string of debug information.
   * @see IfsSystem
   */
   public void debug(String str)
   {
      if (!isDebugEnabled()) return;

      if (IfsSystem.debugInfoToStdOut)
         System.out.println(str);

      if (IfsSystem.debugInfoToHtml)
      {
         try
         {
            getIfsSession().getHttpServletResponse().getWriter().println(str + "<BR>");
         }
         catch (Exception ex)
         { /*....*/

         }
      }

      if (IfsSystem.debugInfoToFile)
      {
         try
         {
            if (IfsSystem.debugLogFile == null)
            {
               // true is for auto flush
               IfsSystem.debugLogFile =
               new PrintWriter(new FileOutputStream(IfsSystem.getDebugLogFileName()), true);
            }
            IfsSystem.debugLogFile.println((new java.util.Date()).toString() + " " + str);

         }
         catch (Exception ex)
         { /*....*/
         }
      }

   }

   /**
   * Removes the specified "methodName" from the registry hashtable.
   *
   * @param methodName -- The name of the method to be removed from the
   *    registry hashtable.
   * @return
   *    When methodName is not present in the registry hashtable, it returns null.
   *    If it is present, it returns the method associated with the methodName.
   */
   public Method deregisterMethod(String methodName)
   {
      try
      {
         Method getter = getClass().getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         return(Method)ht.remove(methodName);
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Removes the method registered under the "methodName" from the class named
   * "className" in the registry hashtable.
   *
   * @param methodName -- The name of the method to be removed from the
   *    registry hashtable.
   * @param className -- The name of the class which methodName is registered
   *    under.
   * @return
   *    When methodName is not present in the registry hashtable, it returns null.
   *    If it is present, it returns the method associated with the methodName.
   */
   public Method deregisterMethod(String methodName, String className)
   {
      try
      {
         Class theClass = Class.forName(className);
         Method getter = theClass.getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(this, new Object[]{});
         return(Method)ht.remove(methodName);
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Gets a binary value from the HTTP servlet request and returns it in a byte array.
   * <P>
   * This method does no character set encoding.  For each byte value (0x00 <-> 0xFF)
   * associated with the "name" in the request parameter, a byte is created in the
   * returned byte array.  The binary value cannot contain imbedded blank characters
   * because the javax.servlet.http.HttpServletRequest getParameterValues() method
   * assumes values are terminated by a blank character.
   *
   * @param req -- The HttpServletRequest object which contains the request
   *    from the client.
   * @param name -- The name of the request parameter which has the binary value.
   * @return
   *    The binary value in a byte array.
   */
   public byte[] getBinaryFromURL(HttpServletRequest req, String name)
   {
      String s;
      try
      {
         // Get the binary value from the input request and do no character set encoding.
         s = new String(req.getParameterValues(name)[0]);
      }
      catch (Exception e)
      {
         debug("IfsTxn.getBinaryFromURL(): Exception for Parameter " + name +
                    " is: " + e.toString() );
         s = "";
      }

      debug("Extracting binary, Parameter Name:" + name);

      int lenValues = s.length();
      byte[] binBytes = new byte[lenValues];

      // Move each byte from the request value into the byte array and do not
      // character set encoding.
      for ( int j = 0; j < lenValues; j++)
          binBytes[j] = (byte)s.charAt(j);

      return(binBytes);
   }

   /**
   * Gets a boolean value string ("true" or "false") from the HTTP servlet request and
   * returns it as a Boolean object.
   *
   * @param req -- The HttpServletRequest object which contains the request
   *    from the client.
   * @param name -- The name of the request parameter which has the boolean value.
   * @return
   *    The boolean value in a Boolean object.
   */
   public boolean getBooleanFromURL(HttpServletRequest req, String name)
   {
      String s;
      Boolean d;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         debug("IfsTxn.getBooleanFromURL():  Encoding exception for parameter " + name + ".");
         s = "0";
      }
      d = new Boolean(s);   // convert string to Boolean object

      debug("Extracting boolean, Name:"+name+", Value="+d+".");

      return(d.booleanValue());
   }

   /**
   * Gets a character value string from the HTTP servlet request and
   * returns it as a String object.
   *
   * @param req -- The HttpServletRequest object which contains the request
   *    from the client.
   * @param name -- The name of the request parameter which has the character string.
   * @return
   *    The character string in a String object.
   */
   public String getCharFromURL(HttpServletRequest req, String name)
   {
      String s;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         debug("IfsTxn.getCharFromURL():  Encoding exception for Parameter "+ name + ".");
         s = "";
      }
      s =  IfsApi.trim(s, ' ');   // remove trailing blanks from the string

      debug("Extracting char, Parameter Name:"+name);

      return(s);  // remove whitespace front/rear from string
   }

   /**
   * Gets a double value string from the HTTP servlet request and
   * returns it as a double value.
   *
   * @param req -- The HttpServletRequest object which contains the request
   *    from the client.
   * @param name -- The name of the request parameter which has the double value.
   * @return
   *    The value as a double.
   */
   public double getDoubleFromURL(HttpServletRequest req, String name)
   {
      String s;
      Double d;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         debug("IfsTxn.getDoubleFromURL():  Encoding exception for parameter " + name + ".");
         s = "0";
      }
      d = new Double(s);   // convert string to Double object

      debug("Extracting double, Name:"+name+", Value="+d+".");

      return(d.doubleValue());
   }

   /**
   * Returns the HttpSession object for this client session.
   * @see IfsSession
   */
   public HttpSession getHttpSession()
   {
      return ifsSession.getHttpSession();
   }

   /**
   * Returns the IfsSession object for this client session.
   * @see IfsSession
   */
   public  IfsSession getIfsSession()
   {
      return ifsSession;
   }

   /**
   * Returns the Locale object for this client session.
   * @see IfsSession
   */
   public Locale getLocale()
   {
      try { return(getIfsSession().getLocale()); }
      catch (Throwable e) { return(IfsSystem.SERVER_LOCALE); }
   }

   /**
   * Gets a long value string from the HTTP servlet request and
   * returns it as an integer value.
   *
   * @param req -- The HttpServletRequest object which contains the request
   *    from the client.
   * @param name -- The name of the request parameter which has the long value.
   * @return
   *    The value as a integer.
   */
   public int getLongFromURL(HttpServletRequest req, String name)
   {
      String s;
      Integer i;
      try
      {
         s = new String(IfsHttpServlet.getParameterValues(name,IfsSystem.cwsGMJCharSet,req)[0]);
      }
      catch (Exception e)
      {
         debug("IfsTxn.getLongFromURL():  Parameter not found: " +name);
         s = "0";
      }

      try
      {
         i = new Integer(s);   // convert string value to Integer object
      }
      catch (NumberFormatException e)
      {
         debug("IfsTxn.getLongFromURL():  NumberFormatException decoding string '"+s+"' from parameter "+name+" in URL.");
         i = new Integer(0);
      }

      debug("Extracting int, Parameter Name:"+name+", Value="+i+".");

      return(i.intValue());
   }

   /**
   * Returns a Hashtable object which contains the list of registered methods.
   */
   public static Hashtable getRegistry()
   {
      return registry;
   }

   /**
   * Returns the response self defining data.
   */
   public GsGmSelfDefiningFieldDef[] getRespSelfDefData() {return(respSelfDefData);}

   /**
   * Abstract method for returning the self defining data owner identification.
   */
   public abstract int getSD_OWNER();

   /**
   * Indicates whether a name in the registry has a registered method.
   *
   * @param methodName -- Name in the registry which is to be
   *    checked for registered methods.

   * @return
   *    Returns true if the registry contains the methodName, false otherwise.
   */
   public boolean hasRegisteredMethod(String methodName)
   {
      try
      {
         Method getter = getClass().getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         return ht.containsKey(methodName);
      }
      catch (Exception e)
      {
         return false;
      }
   }

   /**
   * Indicates whether a name in the registry has a registered method.
   *
   * @param methodName -- Name in the registry which is to be
   *    checked for registered methods.
   * @param withinSuperClassName -- Name of class in the inheritance hierarchy.
   * @return
   *    Returns true if one of the registries in the inheritance hierarchy
   *    within the designated super class contains the methodName, false otherwise.
   */
   public boolean hasRegisteredMethod(String methodName,
                                      String withinSuperClassName)
   {
      boolean returnCode;
      try
      {
         Class theClass = getClass();
         Method getter = theClass.getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         if (ht.containsKey(methodName)) return true;

         if (withinSuperClassName == null || withinSuperClassName.equals(""))
            return false;

         returnCode = false;
         String className = theClass.getName();

         while (!className.endsWith(withinSuperClassName) &&
                !className.endsWith(".IfsObject")         &&
                !className.equals("java.lang.Object")     &&
                returnCode==false)
         {
            theClass  = theClass.getSuperclass();
            className = theClass.getName();
            getter = theClass.getMethod("getRegistry", new Class[]{});
            ht = (Hashtable)getter.invoke(this, new Object[]{});
            returnCode = ht.containsKey(methodName);
         }
      }
      catch (Exception e)
      {
         returnCode=false;
      }
      return returnCode;
   }

   /**
   * Invokes the method registered under the "methodName" name.
   * Throws an exception if there is an error.
   *
   * @param methodName -- Name in the registry whose registered method is to be
   *    invoked.
   * @param args -- The arguments which are passed to the invoked method.
   * @return
   *    Returns the object returned by the registered method in a normal case.
   */
   public Object invokeMethod(String   methodName,
                              Object[] args)
   throws Exception
   {
      try
      {
         Method getter = getClass().getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         Method method = (Method)(ht.get(methodName));
         if (method != null)
         {
            return method.invoke(null, args);
         }
         else
            throw new Exception("Method not registered.");
      }
      catch (Exception e)
      {
         throw e;
      }
   }

   /**
   * Invokes the method registered under the "methodName" name in the
   * inheritance hierarchy within the designated super class.
   * Throws an exception if there is an error.
   *
   * @param methodName -- Name in the registry whose registered method is to be
   *    invoked.
   * @param args -- The arguments which are passed to the invoked method.
   * @param withinSuperClassName -- Name of class in the inheritance hierarchy.
   * @return
   *    Returns the object returned by the registered method in a normal case.
   */
   public Object invokeMethod(String   methodName,
                              Object[] args,
                              String   withinSuperClassName)
   throws Exception
   {
      try
      {
         Class theClass = getClass();
         Method getter = theClass.getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         Method method = (Method)(ht.get(methodName));
         if (method == null)
         {
            if (withinSuperClassName == null ||withinSuperClassName.equals(""))
               throw new Exception("Method not registered.");
            String className = theClass.getName();
            while (!className.endsWith(withinSuperClassName) &&
                   !className.endsWith(".IfsObject")         &&
                   !className.equals("java.lang.Object")     &&
                   method==null)
            {
               theClass  = theClass.getSuperclass();
               className = theClass.getName();
               getter = theClass.getMethod("getRegistry", new Class[]{});
               ht = (Hashtable)getter.invoke(null, new Object[]{});
               method = (Method)ht.get(methodName);
            }
         }
         if (method != null)
         {
            return method.invoke(null, args);
         }
         else
            throw new Exception("Method not registered");
      }
      catch (Exception e)
      {
         throw e;
      }
   }

   /**
   * Indicates whether debugging is enabled for this class.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @return
   *     Returns true when debugging is enabled, false otherwise.
   * @see IfsSystem
   */
   public boolean isDebugEnabled()
   {
      if (IfsSystem.debugMode == IfsSystem.DEBUG_OFF) return false;
      if (IfsSystem.debugMode == IfsSystem.DEBUG_ON ) return true;
      try
      {
         Class theClass = this.getClass();
         Field field = theClass.getField("debugOn");
         if (field != null) return field.getBoolean(null);
      }
      catch (Exception e)
      {

      }
      return false;
   }

   /**
   * Registers the "methodName" to the specified "method" in the registry hashtable.
   *
   * @param methodName -- The name which the "method" parameter will be registered under.
   * @param method -- Name of method which will be registered under "methodName" name.
   * @return
   *    When the methodName is already present,
   *    the old method associated with the methodName is returned.
   *    Returns null, if methodName is not previously present.
   */
   public Method registerMethod(String methodName,
                                Method method)
   {
      try
      {
         Method getter = getClass().getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         return(Method)ht.put(methodName, method);
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Registers the "methodName" to the specified "method" with the class
   * specified by "className" in the registry hashtable.
   *
   * @param methodName -- The name which the "method" parameter will be registered under.
   * @param method -- Name of method which will be registered under "methodName" name.
   * @param className -- The name of the class which the "method" is registered with.
   * @return
   *    When the methodName is already present,
   *    the old method associated with the methodName is returned.
   *    Returns null, if methodName is not previously present.
   */
   public Method registerMethod(String methodName,
                                Method method,
                                String className)
   {
      try
      {
         Class  theClass = Class.forName(className);
         Method getter = theClass.getMethod("getRegistry", new Class[]{});
         Hashtable ht = (Hashtable)getter.invoke(null, new Object[]{});
         return(Method)ht.put(methodName, method);
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Sets the IfsSession object which carries references to the HTTP request,
   * HTTP response, and the session it is associated with.
   */
   public void setIfsSession(IfsSession ifsSession)
   {
      this.ifsSession = ifsSession;
   }

   /**
   * Customizable data method to convert the IfsObject to a localized string
   * representation using the method registered to <EM>toString</EM> and the
   * format information defined in the locale of the HTTP session.
   *
   * @return
   *    Returns a String representation of this IfsObject.
   *    If the method registered to <EM>toString</EM> is not found, it returns null.
   */
   public String toString()
   {
      try
      {
         // Invoke toString method in the registry of the concrete subclass class.
         return(String)invokeMethod("toString", new Object[]{this});
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Customizable data method to convert the IfsObject to a localized string
   * representation using the method registered to <EM>toStringWithIfsSession</EM>
   * and the format information defined in the locale of the HTTP session.
   *
   * @param ifsSession -- The IfsSession object which is associated with this client
   *    session.
   * @return
   *    Returns a String representation of this IfsObject.
   *    If no method is registered or if the method registered
   *    to <EM>toStringWithIfsSession</EM> is not found, it returns null.
   */
   public String toString(IfsSession ifsSession)
   {
      try
      {
         // Invoke toStringWithIfsSession method in the registry of the concrete subclass class.
         return(String)invokeMethod("toStringWithIfsSession", new Object[]{this, ifsSession});
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Customizable data method to convert the IfsObject to a localized string
   * representation using the method registered to <EM>toStringWithString</EM>
   * and the format information defined in the locale of the HTTP session.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this IfsObject.
   *    If no method is registered or if the method registered to
   *    <EM>toStringWithString</EM> is not found, it returns null.
   */
   public String toString(String prefix)
   {
      try
      {
         // Invoke toStringWithString method in the registry of the concrete subclass class.
         return(String)invokeMethod("toStringWithString", new Object[]{this, prefix});
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Customizable data method to convert the IfsObject to a localized string
   * representation using the method registered to <EM>toStringWithFormat</EM>
   * and the format "f".
   *
   * @param f -- The Format object used in the to string conversion.
   * @return
   *    Returns a String representation of this IfsObject.
   *    If no method is registered or if the method registered to
   *    <EM>toStringWithFormat</EM> is not found, it returns null.
   */
   public String toString(Format f)
   {
      try
      {
         // Invoke toStringWithFormat method in the registry of the concrete subclass class.
         return(String)invokeMethod("toStringWithFormat", new Object[]{this, f});
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Customizable data method to convert the IfsObject to a localized string
   * representation using the method registered to <EM>toStringWithFormat</EM>
   * and the format "f".
   *
   * @param f -- The Format object used in the to string conversion.
   * @return
   *    Returns a String representation of this IfsObject.
   *    If no method is registered or if the method registered to
   *    <EM>toStringWithFormat</EM> is not found, it returns null.
   */
   public String toStringWithFormat(Format f)
   {
      try
      {
         // Invoke toStringWithFormat method in the registry of the concrete subclass class.
         return(String)invokeMethod("toStringWithFormat", new Object[]{this, f});
      }
      catch (Exception e)
      {
         return null;
      }
   }

   /**
   * Customizable data method to validate the IfsObject using the method registered
   * to <EM>validate</EM>.
   *
   * @return
   *    Returns "OK" if the IfsObject is valid.  Otherwise, it returns a message
   *    string indicating the error.
   *    If no method is registered or if the method registered to
   *    <EM>validate</EM> is not found, it returns null.
   */
   public String validate()
   {
      String msg = "";
      try
      {
         // Invoke validate method in the registry of the concrete subclass class.
         msg = (String)invokeMethod("validate", new Object[]{this});
      }
      catch (Throwable e)
      {
         msg = null;
      }
      return msg;
   }
}

