package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsSession.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  CHANGES:
//    03/06/2000  rhs   d6423   Made JavaDoc additions.
//    03/14/2000  rhs   d6545   Added support for get and set of locale in session cache.
//    02/05/2002  pdl   d7430   Remove calls to deprecated APIs
//
// ***************************************************************************

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Locale;
import javax.servlet.*;
import javax.servlet.http.*;
import com.ibm.ifs.gold.IfsException;

/**
* For this session with the client, IfsSession class encapsulates the HTTP request,
* HTTP response, session key cookie, and other session information
* for the servlet writer into one object.
*/
public class IfsSession implements Serializable
{
   private HttpServletRequest httpServletRequest;
   private HttpServletResponse httpServletResponse;
   private boolean sessionCreate = true;
   private HttpSession httpSession;
   private Cookie sessionKey;

   /**
   * Constructs an IfsSession object and sets the session key attribute to a value
   * from the session cache.
   * If object identifier, "__SessionKey", has a non-null value in the
   * session cache, the session key is set to it.
   * Otherwise, the session key is set to the Cookie object created with the
   * constant string specified by IfsSystem.cwsCookieName.
   * When the "req" parameter is null, the http session and session key attributes
   * are initialized to null.
   *
   * @param req -- An HttpServletRequest object which contains the request from
   *    the client.
   * @param res -- An HttpServletResponse object which is being used to
   *    send MIME-encoded data back to the client.
   */
   public IfsSession(HttpServletRequest req, HttpServletResponse res)
   {
     httpServletRequest = req;
     httpServletResponse = res;

     if (req != null)
     {
       httpSession = req.getSession(sessionCreate);
       sessionKey = (Cookie)httpSession.getAttribute("__SessionKey");      
       if (sessionKey == null)
          sessionKey = new Cookie(IfsSystem.cwsCookieName, null);

       Hashtable oidTable = (Hashtable)httpSession.getAttribute("__OidTable"); 
       if (oidTable == null)
                httpSession.setAttribute("__OidTable", new Hashtable());   

     }
     else
     {
        httpSession = null;
        sessionKey = null;
     }
   }

   /**
   * Constructs an IfsSession object and sets the session key attribute to the
   * "cookie" parameter.
   * When the "req" parameter is null, the http session attribute
   * is initialized to null.
   *
   * @param req -- An HttpServletRequest object which contains the request from
   *    the client.
   * @param res -- An HttpServletResponse object which is being used to
   *    send MIME-encoded data back to the client.
   * @param cookie -- A Cookie object which is used to set the session key attribute.
   */
   public IfsSession(HttpServletRequest req, HttpServletResponse res, Cookie cookie)
   {
     httpServletRequest = req;
     httpServletResponse = res;
     this.sessionKey = cookie;

     if (req != null) {
       httpSession = req.getSession(sessionCreate);
       Hashtable oidTable = (Hashtable)httpSession.getAttribute("__OidTable");  
       if (oidTable == null)
                httpSession.setAttribute("__OidTable", new Hashtable()); 

     }
   }

   /**
   * Returns the HttpServletRequest object which contains the request from the client.
   */
   public HttpServletRequest getHttpServletRequest() {return(httpServletRequest);}

   /**
   * Returns the HttpServletResponse object which is being used to
   * send MIME-encoded data back to the client.
   */
   public HttpServletResponse getHttpServletResponse() {return(httpServletResponse);}

   /**
   * Returns the HttpSession object which is associated with the request
   * from the client.
   */
   public HttpSession getHttpSession() {return(httpSession);}

   /**
   * Returns the Locale object for this session.
   */
   public Locale getLocale()
   {
      Locale locale = null;

      try { locale = (Locale) getHttpSession().getAttribute("__Locale"); } 
      catch (Throwable e) {  }

      if (locale == null)
      {
         locale = IfsSystem.SERVER_LOCALE;
         getHttpSession().setAttribute("__Locale", locale);
      }
      return(locale);
   }

   /**
   * Returns the Cookie object which is the session key for this session with the client.
   */
   public Cookie getSessionKey()
   {
      return sessionKey;
   }

   /**
   * Sets the Locale for this session in the session cache to the "locale" parameter.
   *
   * @param locale -- The Locale object used to set the "__Locale" object in the
   *    session cache.
   */
   public void setLocale(Locale locale) throws IfsException
   {
      try { httpSession.setAttribute("__Locale", locale); }
      catch (Throwable e)
      {
         throw new IfsException(e, "Unable to putValue __Locale", "");
      }
   }

   /**
   * Sets the session key attribute and the object identifier, "__SessionKey",
   * in the session cache to the "key" parameter.
   *
   * @param key -- The Cookie object used to set the session key attribute.
   */
   public void setSessionKey(Cookie key)
   {
      sessionKey = key;
      httpSession.setAttribute("__SessionKey", key);
   }

} // end-of, IfsSession class.

