// ***************************************************************************
//
//  FILE NAME:   IfsSystem.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2000 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  Changes:
//    02/17/2000  rhs   d6361    Made JavaDoc correction/addtions.
//    03/13/2000  rhs   d6545    Added NLS support
//    11/11/2001  bjb   d7361    Removed support for IfsJDBC
//    11/26/2001  bjb   d7381    Replaced IFS_WDBC_SCHEMA variable
//                               removed by defect 7361
//    01/22/2002  nc/bb d7400    Fixed Copyright problem. added code to improve 
//                               display copyright text
//    02/04/2002  bjb   d7427    Added IFS system property names for debugging
//                               and default configuration file name
//    02/10/2002  bjb   d7435    Read all static String, int and boolean 
//                               variables from properties file. Code cleanup.
//    02/20/2002  bjb   d7460    Read variables of type Object from 
//                               properties file.
//    06/24/2002  tlt   T2000163 Modified debugMode properties to get value from IfsSystem.properties; 
//                               Corrected spelling of ifs.system.response.redady.action
//    07/15/2002  dob   T2000147 Save/retrieve client address to session.
//    07/23/2002  pdl   T2000176 Split config file.
// ***************************************************************************

package com.ibm.ifs.gold;

import java.io.*;
import java.util.*;
import java.awt.Font;
import java.awt.Color;
import javax.servlet.*;
import javax.servlet.http.*;

/**
* IfsSystem class contains all the recommended default IFS system constant values.
* This class is available for the financial institution (FI) to customize.
* The FI can define FI specific constants, system tables, and their initialization
* and customized method registration.  IfsSystem is loaded before any user page
* is invoked.
*/
public class IfsSystem extends HttpServlet {

/* ----------------------------- Class Variables ---------------------------- */

  // Property file names

  /** The name of the property file containing the IFS system properties. */
  private static final String IFS_SYSTEM_PROPERTY_FILE = "IfsSystem"; // d7400

  /**  */
  private static final String THREE_TOKEN_CODES_PROPERTY_FILE = 
    "com.ibm.ifs.resource.3TkTypes";

  /**  */
  private static final String CWS_CODES_PROPERTY_FILE = 
    "com.ibm.ifs.resource.CWSTypes";

  /**  */
  private static final String BRC_CODES_PROPERY_FILE = 
    "com.ibm.ifs.resource.BRCTypes";

  // Property cache

  /**  */
  private static Hashtable systemProperties;  // d7400

  /**  */
  public static Hashtable allBrcTypesDictionary;
   
  /**  */
  public static Hashtable allCwsTypesDictionary;

  /**  */
  public static Hashtable all3tkTypesDictionary;

  // IfsSystem property names, these properties are only retrievable by using
  // the getIfsSystemProperty method (i.e. they are not stored in static variables

  /**
   * The name of the IFS system property that contains the
   * copyright text used/returned by the 
   * com.ibm.ifs.servlets.IfsHttpServlet.xxxCopyright() methods.
  */
  public static final String COPYRIGHT_PROP_NAME  = "ifs.system.copyright.info"; //d7400

  /** 
   * The name of the IFS system property that debugging 
   * statements on and off. 
  */
  public static final String DEBUG_PROP_NAME = "ifs.system.debug"; //d7427

  /** 
   * The name of the IFS system property that specifies the 
   * default application configuration file name. 
  */
  public static final String DEFAULT_APP_CONFIG_PROP_NAME =       
    "ifs.system.default.app.config";                                    //d7427 //T2000176 

  /** 
   * The name of the IFS system property that specifies the 
   * default environment configuration file name. 
  */
  public static final String DEFAULT_ENV_CONFIG_PROP_NAME = 
    "ifs.system.default.env.config";                                    //T2000176 

  /** 
   * The name of the IFS system property that specifies the WebSphere datasource
   * to use for the IfsJDBC class.
  */
  public static final String JDBC_DATASOURCE_PROP_NAME = 
    "jdbc.dataSource";

  /**
   * The name of the IFS system property that specifies the WebSphere initial
   * context used for looking up data sources.
  */
  public static final String JDBC_INITIAL_CONTEXT_PROP_NAME = 
    "jdbc.initialContext";

  // The following static variables are loaded from the IfsSystme properties 
  // file if they are defined, else, they take on a default value.
 
  /**  */
  public static String IFS_WDBC_SCHEMA = 
    getStringProperty("ifs.system.wdbc.schema", "DBTIFAGC");

  /**
   * Name of the CWS cookie.  This name must match the
   * cookie name contained in the CWS configuration file.
  */
  public static String cwsCookieName = 
    getStringProperty("ifs.system.cookie.name", "IBANK");

  /**
   * The character set used by the JDK.  The valid values are "8859_1" or "UTF-8".
   * This must be the same character set as what is specified in
   * cwsBrowserCharSet.
  */
  public static String cwsGMJCharSet = 
    getStringProperty("ifs.system.GMJ.charset", "UTF-8");

  /**
   * The character set used to pass data to/from the browsers.
   * The valid values are "8859_1" or "UTF-8".
   * This must be the same character set as what is specified in
   * cwsGMJCharSet.
  */
  public static String cwsBrowserCharSet = 
    getStringProperty("ifs.system.browser.charset", "UTF-8");
  
  /**  */
  public static  String IFS_SERVER_PATH = 
    getStringProperty("ifs.system.ifs.server.path", null);

  /**
   * Specifies the full directory path name where the autogened
   * GMOB 'strings' are stored.  If this attribute is not null, then the gold data
   * returned by a gold transaction is gotten from the strings file rather than
   * from a call to the gold manager. This is an example of how to use the stringsPath on AIX
  */
  public static String STRINGS_PATH = 
    getStringProperty("ifs.system.strings.path", null);

  /**  */
  public static  String STRINGS_PATH2 = 
    getStringProperty("ifs.system.strings.path2", "d:\\WebSphere\\AppServer\\servlets\\data\\");

  /**  */
  public static String AIX_STRINGS_PATH = 
    getStringProperty("ifs.system.aix.string.path", "/usr/WebSphere/AppServer/installedApps/test100.ear/ifs.war/docs/jcwsapi/data/");
   
  /**  */
  public static  boolean IS_VALIDATE_ACTION_REQUIRED =  
    getBooleanProperty("ifs.system.validate.action.required", true);

  /**  */
  public static  boolean IS_REQUEST_READY_ACTION_REQUIRED =  
    getBooleanProperty("ifs.system.request.ready.action.required",false);
   
  /**  */
  public static  boolean IS_RESPONSE_READY_ACTION_REQUIRED = 
    getBooleanProperty("ifs.system.response.ready.action.required", false);    // T2000163

  /**  */ 
  public static  boolean IS_SYNCHRONIZE_DATE_REQUIRED = 
    getBooleanProperty("ifs.system.synchronize.date.required", false);

  // Preferences
  
  /**  */
  public static  String  DEBIT_SIGN = 
    getStringProperty("ifs.system.debit.sign", "-[({<"); // debit indicators
   
  /**  */
  public static  boolean REMOVE_TRAILING_FRACTION_ZEROES = 
    getBooleanProperty("ifs.system.remove.trailing.fraction.zeroes", false); //remove trailing zeroes

  // Additional currency and decimal constants

  /**  */ 
  public static final int CURRENCY_TEXT_STYLE = 
    getIntProperty("ifs.system.currency.text.style", 0);
   
  /**  */
  public static final int CURRENCY_HTML_STYLE = 
    getIntProperty("ifs.system.currency.html.style", 1);
   
  /**  */
  public static int CURRENCY_STYLE = CURRENCY_HTML_STYLE;
   
  /**  */
  public static final int DECIMAL_TEXT_STYLE = 
    getIntProperty("ifs.system.decimal.text.style", 0);
   
  /**  */
  public static final int DECIMAL_HTML_STYLE = 
    getIntProperty("ifs.system.html.style", 1);
   
  /**  */
  public static int DECIMAL_STYLE = DECIMAL_HTML_STYLE;

  // Alignments/Justifications
   
  /**  */
  public static String TABLE_DATE = 
    getStringProperty("ifs.system.alignment.table.date" , "LEFT");   // alignment to use for dates in TABLE
   
  /**  */
  public static String TABLE_MONEY = 
    getStringProperty("ifs.system.alignment.table.money", "RIGHT");  // alignment to use for money/currency in TABLE
   
  /**  */
  public static String TABLE_STRING = 
    getStringProperty("ifs.system.alignment.table.string", "LEFT");  // alignment to use for string in TABLE
   
  /**  */
  public static String TABLE_DATA =
    getStringProperty("ifs.system.alignment.table.data", "LEFT");    // alignment to use for data in TABLE
   
  /**  */
  public static String TABLE_DATA2 = 
    getStringProperty("ifs.system.alignment.table.data2", "CENTER"); // alignment to use for data in TABLE (1st alternative)
   
  /**  */
  public static String TABLE_DATA3 = 
    getStringProperty("ifs.system.alignment.table.data3", "RIGHT");  // alignment to use for data in TABLE (2nd alternative)
   
  /**  */
  public static String FORM_LABEL =
    getStringProperty("ifs.system.alignment.form.label", "RIGHT");  // alignment to use for FORM labels
   
  /**  */
  public static String FORM_DATA = 
    getStringProperty("ifs.system.alignment.form.data", "LEFT");    // alignment to use for FORM data
   
  /**  */
  public static String FORM_DATA2 =
    getStringProperty("ifs.system.alignment.form.data2", "CENTER"); // alignment to use for FORM data (1st alternative)
   
  /**  */
  public static String FORM_DATA3 = 
    getStringProperty("ifs.system.alignment.form.data3", "RIGHT");  // alignment to use for FORM data (2nd alternative)
   
  /**  */
  public static String TITLE_DATA = 
    getStringProperty("ifs.system.alignment.title.data", "CENTER"); // alignment to use for Title data
   
  /**  */
  public static String SECTION_DATA  = 
    getStringProperty("ifs.system.alignment.section.data", "LEFT");   // alignment to use for Section title data

  // Font sizes
   
  /**  */
  public static String TITLE_SIZE = 
    getStringProperty("ifs.system.font.title.size", "+2");
   
  /**  */
  public static String SECTION_SIZE = 
    getStringProperty("ifs.system.font.section.size", "+1");
   
  // Table Attributes

  /**  */
  public static String TABLE_BORDER = 
    getStringProperty("ifs.system.table.border", "0"); // table border ("0" = turn borders off)
   
  /**  */
  public static String TABLE_ALIGN = 
    getStringProperty("ifs.system.table.align", "CENTER"); // table alignment
   
  /**  */
  public static String TABLE_CELLPADDING = 
    getStringProperty("ifs.system.table.cellpadding", "2"); // table cellpadding
   
  /**  */
  public static String TABLE_CELLSPACING = 
    getStringProperty("ifs.system.table.cellspacing", "1"); // table cellspacing
   
  /**  */
  public static String TABLE_WIDTH = 
    getStringProperty("ifs.system.table.width", "100%");    // table width

  // Country/Language Codes
   
  /**  */
  public static String COUNTRY_CODE = 
    getStringProperty("ifs.system.country.code", "USA");  // country code (used in logon and code-to-text)
   
  /**  */
  public static String LANGUAGE_CODE = 
    getStringProperty("ifs.system.language.code", "ENG"); // language code (used in logon and code-to-text)
   
  // Colors
   
  /**  */
  public static String PAGE_COLOR = 
    getStringProperty("ifs.system.page.color", "#FFFFFF");  // white
   
  /**  */
  public static String MSG_COLOR = 
    getStringProperty("ifs.system.msg.color", "#8EB5F0");   // Light Blue
   
  /**  */
  public static String ADDR_COLOR = 
    getStringProperty("ifs.system.addr.color", "#000066");  // blue
   
  /**  */
  public static String TTL_COLOR = 
    getStringProperty("ifs.system.ttl.color", "#000000");  // black
   
  /**  */
  public static String SUBTTL_COLOR = 
    getStringProperty("ifs.system.subttl.color", "#000080");  // dark blue
   
  /**  */
  public static String TABLE_HEADER_BGCOLOR = 
    getStringProperty("ifs.system.table.header.bgcolor", "#D9D9D9"); // light gray
   
  /**  */
  public static String TABLE_BGCOLOR = 
    getStringProperty("ifs.system.table.bgcolor", "#FFFFFF");  // white

  /**  */
  public static String COLOR_1 = 
    getStringProperty("ifs.system.color1", "#8EB5F0"); // light blue (message bar)
   
  /**  */
  public static String COLOR_2 = 
    getStringProperty("ifs.system.color2", "#ffffff"); // black
   
  /**  */
  public static String COLOR_3 = 
    getStringProperty("ifs.system.color3", "#dfdfdf"); // gray

  // error handling
   
  /**  */
  public static boolean TRAP_FATAL_ERRORS = 
    getBooleanProperty("ifs.system.trap.fatal.errors", true);

  /**
   * @depreciated The locale sensitive class methods in java.lang.Character
   * should be used to determine valid digits.
  */
  public static  String VALID_DIGITS = "0123456789";

  /**
   * @depreciated The locale sensitive class methods in java.lang.Character
   * should be used to determine valid letters, change case, etc. .
  */
  public static   String UC_CHARS     = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

  /**
   * @depreciated The locale sensitive class methods in java.lang.Character
   * should be used to determine valid letters, change case, etc. .
  */
  public static   String LC_CHARS     = "abcdefghijklmnopqrstuvwxyz";

  /**
   * @depreciated The locale sensitive class methods in java.lang.Character
   * should be used to determine valid letters, change case, etc. .
  */
  public static String VALID_CHARS  = UC_CHARS + LC_CHARS;

  /**
   * Indicates how many digits are to be "obscured" at the end of numbers.
   * It is often used for account numbers.
  */
  public static String OBSCURITY = 
    getStringProperty("ifs.system.obscurity", "0");

  /**
   * Is used as the maximum number of transactions that can be requested for inquiries.
   * Used with transfer inquiries as well as account history inquiries.
  */
  public static String MAX_REQUEST = 
    getStringProperty("ifs.system.max.requests", "15");

  /**
   * Indicates whether to include FI Product Descriptions for types of
   * accounts in the user profile.
  */
  public static boolean FI_PRODUCT_DESCRIPTION = 
    getBooleanProperty("ifs.system.fi.product.description", false);

  /**
   * Indicates whether the FI has the capability to execute transfers before the
   * end of day (EOD).
   * The FI may choose to present the consumer with the option to do so.
  */
  public static boolean FI_TRANSFER_IMMEDIATE = 
    getBooleanProperty("ifs.system.fi.transfer.immediate", true);

  // Bank or FI Identification (see help.html among other places)
   
  /**  */
  public static String APPL_NAME = 
    getStringProperty("ifs.system.appl.name", "WebBank Interactive Banking");
   
  /**  */
  public static String BANK_NAME = 
    getStringProperty("ifs.system.bank.name", "WebBank");
   
  /**  */
  public static String ABANK_NAME = 
    getStringProperty("ifs.system.abank.name", "a WebBank");
   
  /**  */
  public static String PBANK_NAME = 
    getStringProperty("ifs.system.pbank.name", "WebBank's");
   
  /**  */
  public static  String META_AUTHOR = 
    getStringProperty("ifs.system.meta.author", "IBM Corporation");
   
  /**  */
  public static  String META_APPL = 
    getStringProperty("ifs.system.meta.appl", "Customizable Web Server Reference Pages");
   
  // Directories
  
  /**  */
  public static  String IMAGE_DIRECTORY = 
    getStringProperty("ifs.system.image.directory", "/images/");

  // HINTS for date - time and others
   
  /**  */
  public static String DATE_HINT = 
    getStringProperty("ifs.system.date.hint", "mm/dd/yyyy");
   
  /**  */
  public static String TIME_HINT = 
    getStringProperty("ifs.system.time.hint", "hh:mm");
   
  /**  */
  public static  String AMPM_HINT = 
    getStringProperty("ifs.system.ampm.hint", "AM/PM");

  // Bill Presentment constants
   
  /**  */
  public static String PENDING_ACTIVATION = 
    getStringProperty("ifs.system.biller.pending.activation.delivery.state", "1");     // E BILLER DELIVERY STATE
   
  /**  */
  public static String ACTIVE = 
    getStringProperty("ifs.system.biller.active.delivery.state", "2");     // E BILLER DELIVERY STATE
   
  /**  */
  public static String REJECTED_ACTIVATION = 
    getStringProperty("ifs.system.biller.rejected.activation.delivery.state", "3");     // E BILLER DELIVERY STATE
   
  /**  */
  public static String PENDING_DEACTIVATION = 
    getStringProperty("ifs.system.biller.pending.deactivation.delivery.state", "4");    // E BILLER DELIVERY STATE
   
  /**  */
  public static String DEACTIVATED = 
    getStringProperty("ifs.system.biller.deactivated.delivery.state", "5");     // E BILLER DELIVERY STATE
   
  /**  */
  public static String REJECTED_DEACTIVATION = 
    getStringProperty("ifs.system.biller.rejected.deactivation.delivery.state", "6");     // E BILLER DELIVERY STATE
   
  /**  */
  public static String RETRIEVE_NEW = 
    getStringProperty("ifs.system.biller.retrieve.new.request.state", "1");     // E BILL REQUEST STATE ( NEW ONLY )
   
  /**  */
  public static String RETRIEVE_VIEWED = 
    getStringProperty("ifs.system.biller.retrieve.viewed.request.state", "2");     // E BILL REQUEST STATE ( VIEWED ONLY )
   
  /**  */
  public static String RETRIEVE_RETIRED = 
    getStringProperty("ifs.system.biller.retrieve.retired.request.state", "3");     // E BILL REQUEST STATE ( RETIRED ONLY )
   
  /**  */
  public static String RETRIEVE_ALL = 
    getStringProperty("ifs.system.biller.retrieve.all.request.state", "4");     // E BILL REQUEST STATE
   
  /**  */
  public static String RETRIEVE_ALL_BUT_RETIRED = 
    getStringProperty("ifs.system.biller.retrieve.all.but.retired", "5");     // E BILL REQUEST STATE
   
  /**  */
  public static String UNPAID = 
    getStringProperty("ifs.system.bill.payment.status.unpaid", "1");     // E BILL PAYMENT STATUS
   
  /**  */
  public static String PARTIALLY_PAID = 
    getStringProperty("ifs.system.bill.payment.status.partially.paid", "2");     // E BILL PAYMENT STATUS
   
  /**  */
  public static String FULLY_PAID = 
    getStringProperty("ifs.system.bill.payment.status.fully.paid", "3");     // E BILL PAYMENT STATUS
   
  /**  */
  public static String NEW = 
    getStringProperty("ifs.system.bill.state.new", "1");     // E BILL STATE
   
  /**  */
  public static String VIEWED = 
    getStringProperty("ifs.system.bill.state.viewed", "2");     // E BILL STATE
   
  /**  */
  public static String RETIRED = 
    getStringProperty("ifs.system.bill.state.retired", "3");     // E BILL STATE

  // Used in ebill_biller_dlvry_inq and ebill_biller_inq
  
  /**  */
  public static int MAX_NUMBER_BILLERS = 
    getIntProperty("ifs.system.max.number.billers", 999999);
   
  /**  */
  public static int MAX_NUMBER_PAYEES = 
    getIntProperty("ifs.system.max.number.payees", 999999);
   
  /**  */
  public static int MAX_NUMBER_BILLS = 
    getIntProperty("ifs.system.max.number.bills", 999999);
   
  /**  */
  public static int NO_INDEX_SELECTED = 
    getIntProperty("ifs.system.no.index.selected", -1);
   
  /**  */
  public static String DEFAULT_DELIVERY_STATUS_TEXT = 
    getStringProperty("ifs.system.default.delivery.status.text", "No text value found for this tag/code");
   
  /**  */
  public static boolean INCLUDE_LOGO = 
    getBooleanProperty("ifs.system.include.logo", true);
   
  /**  */
  public static boolean DISPLAY_PRESENTMENT_TABS = 
    getBooleanProperty("ifs.system.display.presentment.tabs", true);
   
  /**  */
  public static final String PRESENTMENT_ENABLED = 
    getStringProperty("ifs.system.presentment.enabled", "1");
   
  /**  */
  public static final String PRESENTMENT_DISABLED = 
    getStringProperty("ifs.system.presentment.disabled", "0");

  // Bill Presentment GoldError Message constants
   
  /**  */
  public static final String BILLER_NOT_FOUND = 
    getStringProperty("ifs.system.gold.error.message.biller.not.found", "500");
   
  /**  */
  public static final String UNKNOWN_CONSUMER = 
    getStringProperty("ifs.system.gold.error.message.unknown.consumer", "501");

  /**  */
  public static final String BILLER_DLVRY_ALREADY_ACTIVE = 
    getStringProperty("ifs.system.gold.error.message.biller.dlvry.already.active", "502");

  /**  */
  public static final String ACCT_NUMBER_ALREADY_ACTIVE = 
    getStringProperty("ifs.system.gold.error.message.acct.number.already.active", "503");

  /**  */
  public static final String BILLER_DLVRY_DEACT_IN_PROGRESS = 
    getStringProperty("ifs.system.gold.error.message.biller.dlvry.deact.in.progress", "504");

  /**  */
  public static final String BILLER_DLVRY_ACT_IN_PROGRESS = 
    getStringProperty("ifs.system.gold.error.message.biller.dlvry.act.in.progress", "505");
   
  /**  */
  public static final String BILL_AMOUNT_INVALID = 
    getStringProperty("ifs.system.gold.error.message.bill.amount.invalid", "506");
   
  /**  */
  public static final String BILL_NOT_FOUND = 
    getStringProperty("ifs.system.gold.error.message.bill.not.found", "507");
   
  /**  */
  public static final String BILLER_DLVRY_ALREADY_DEACTIVATED = 
    getStringProperty("ifs.system.gold.error.message.biller.dlvry.already.deactivated", "508");
   
  /**  */
  public static final String BILL_STATE_INVALID = 
    getStringProperty("ifs.system.gold.error.message.bill.state.invalid", "509");
   
  /**  */
  public static final String PAYMENT_STATUS_INVALID = 
    getStringProperty("ifs.system.gold.error.message.payment.status.invalid", "510");
   
  /**  */
  public static final String INCLUDE_LOGO_INVALID = 
    getStringProperty("ifs.system.gold.error.message.include.logo.invalid", "511");
   
  /**  */
  public static final String BILLER_DLVRY_NOT_FOUND = 
    getStringProperty("ifs.system.gold.error.message.biller.dlvry.not.found", "512");
   
  /**  */
  public static final String BILL_COUNTS_ONLY_NOT_FOUND = 
    getStringProperty("ifs.system.gold.error.message.bill.counts.only.not.found", "513");

  //Additional GoldError Message constants
   
  /**  */
  public static String SUCCESS = 
    getStringProperty("ifs.system.gold.error.message.success", "000");
   
  /**  */
  public static String DUPLICATE_PAYMENT = 
    getStringProperty("ifs.system.gold.error.message.duplicate.payment", "269");

  // ADD ADDITIONAL CUSTOM CONSTANTS HERE:
  
  /**  */
  public static String FREEFORM_PAYEE_TYPE = 
    getStringProperty("ifs.system.freeform.payee.type", "F");
   
  /**  */
  public static String CLIENT_SIDE_JS_ENABLED = 
    getStringProperty("ifs.system.client.side.js.enabled", "Set");

  //Page Identifier Constants (note: the following are examples)

  /**  */ 
  public static String ServerPath = 
    getStringProperty("ifs.system.pages.server.path", "/servlet/");
   
  // Misc. Pages
  
  /**  */
  public static String LOGON = 
    getStringProperty("ifs.system.pages.logon", "logon.jsp");
   
  /**  */
  public static String LOGOFF = 
    getStringProperty("ifs.system.pages.logoff", "logoff.jsp");
   
  /**  */
  public static String IFS_ERROR_PAGE = 
    getStringProperty("ifs.system.pages.ifs.error.page", "IfsErrorPage.jsp");
   
  /**  */
  public static String ErrorPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.error.page", "ErrorPage");
   
  /**  */
  public static String InfoPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.info.page", "InfoPage");
   
  /**  */
  public static String RedoPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.redo.page", "RedoPage");
   
  /**  */
  public static String HelpPage = 
    getStringProperty("ifs.system.pages.help.page", "/logon/help.html");
   
  /**  */
  public static String LogoffPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.logoff.page", "Logoff");
   
  /**  */
  public static  String LogonPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.logon.page", "Logon");

  /**  */ 
  public static String PageDisplayedFollowingLogon = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.following.logon", "Menu");
   
  /**  */
  public static String ReauthenticationPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.reauth.page", "Reauth");
   
  /**  */
  public static String SelfRegistrationPage = IfsSystem.ServerPath + 
    getStringProperty("ifs.system.pages.self.reg.page", "SelfRegistrationPage");
   
  /**  */
  public static String BankInfoPage = 
    getStringProperty("ifs.system.pages.bank.info.page", "/logon/bankinfo.html");

  // Accounts Pages
   
  /**  */
  public static String ACCT_SUMM = 
    getStringProperty("ifs.system.account.pages.acct.summ", "acctsum.jsp");
   
  /**  */
  public static String ACCT_DTL_CD = 
    getStringProperty("ifs.system.account.pages.acct.dtl.cd", "acctdtlcd.jsp");
   
  /**  */
  public static String ACCT_DTL_CHK = 
    getStringProperty("ifs.system.account.pages.acct.dtl.chk", "acctdtlch.jsp");
   
  /**  */
  public static String ACCT_DTL_IRA = 
    getStringProperty("ifs.system.account.pages.acct.dtl.ira", "acctdtlir.jsp");
   
  /**  */
  public static String ACCT_DTL_SAV = 
    getStringProperty("ifs.system.account.pages.acct.dtl.sav", "acctdtlsv.jsp");
   
  /**  */
  public static String ACCT_DTL_ILN = 
    getStringProperty("ifs.system.account.pages.acct.dtl.iln", "acctdtlil.jsp");
   
  /**  */
  public static String ACCT_DTL_MLN = 
    getStringProperty("ifs.system.account.pages.acct.dtl.mln", "acctdtlml.jsp");
   
  /**  */
  public static String ACCT_DTL_LOC = 
    getStringProperty("ifs.system.account.pages.acct.dtl.loc", "acctdtllc.jsp");
   
  /**  */
  public static String ACCT_DTL_EQL = 
    getStringProperty("ifs.system.account.pages.acct.dtl.eql", "acctdtleq.jsp");
   
  /**  */
  public static String ACCT_HST = 
    getStringProperty("ifs.system.account.pages.acct.hst", "accthist.jsp");
   
  /**  */
  public static String ACCT_HST_DT = 
    getStringProperty("ifs.system.account.pages.acct.hst.dt", "accthistdtl.jsp");
   
  /**  */
  public static String ACCT_HST_SRCH = 
    getStringProperty("ifs.system.account.pages.acct.hst.srch", "accthistsch.jsp");

  /**
   * @depreciated The IfsSystem.ReauthBRC constant should be used instead.
  */
  public static  String WSP_REAUTHENTICATION_TIMEOUT = "600";

  /**
   * The business response code from a transaction which determines when the browser
   * should reroute page flow to the reauthorization page.  When the transaction code
   * value matches this value, rerouting occurs.
  */
  public static String ReauthBRC = 
    getStringProperty("ifs.system.reauth.brc", "600");

  /**
   * The business response code from a transaction which determines when the browser
   * should reroute page flow to the logon page.  When the transaction code
   * value matches this value, rerouting occurs.
  */
  public static String ReLogonBRC = 
    getStringProperty("ifs.system.relogon.brc", "1000");

  /**
   * The URL the webserver redirects the user to when it detects a VERIFY failure.
  */
  public static String WSP_VERIFY_FAILURE_URL = 
    getStringProperty("ifs.system.wsp.verify.failure.url", null);

  /**
   * The URL the webserver redirects the user to when
   * it detects that reauthentication is required.
  */
  public static String WSP_REAUTH_REQUIRED_URL = 
    getStringProperty("ifs.system.wsp.reauth.required.url", "/logon/reauth.html");

  /**
   * The URL the webserver redirects the user to when
   * it logoffs a user from the webserver.
  */
  public static String WSP_LOGOFF_URL = 
    getStringProperty("ifs.system.wsp.logoff.url", "/logon/exit.html");

  /**
   * The URL the webserver redirects the user to when it detects a failure in
   * SELFREG transaction.
  */
  public static String WSP_SELFREG_FAILURE_URL = 
    getStringProperty("ifs.system.wsp.selfreg.failure.url", "/selfreg/selfreg.html");

  /**
   * The URL the webserver redirects the user to when it detects a failure in
   * CHGPWPIN transaction.
  */
  public static String WSP_CHGPWPIN_FAILURE_URL = 
    getStringProperty("ifs.system.wsp.chgpwpin.failure.url", "/bank/chgpwpin.html");

  /**
   * This is the URL that the web server redirects the user to
   * when the reauthentication transaction was successful.
   * <P>
   * Note: WSP_REAUTH_SUCCESSFUL_URL should be a page that requires no
   *   parameters on the URL such as the initial page after logon
   *  (e.g. the account summary page in the reference page set).
  */
  public static String WSP_REAUTH_SUCCESSFUL_URL = 
    getStringProperty("ifs.system.wsp.reauth.successful.url", "/bank/");

  /**
   * Specifies the maximum number of web users who may concurrently
   * log on to this web server.
  */
  public static int WSP_MAX_SESSIONS= 
    getIntProperty("ifs.system.wsp.max.sessions", 50);

  /**  */ 
  public static boolean isSessionBasedLocalSupported = 
    getBooleanProperty("ifs.system.is.session.based.local.supported", false);  // or true.  Bank can decide.

  /**  */
  public static String MAX_CURRENCY_VALUE = 
    getStringProperty("ifs.system.max.currency.value", "999999999999.99");
   
  /**  */
  public static String MIN_CURRENCY_VALUE = 
    getStringProperty("ifs.system.min.currency.value", "-999999999999.99");
   
  /**  */
  public static String MAX_DECIMAL_VALUE = 
    getStringProperty("ifs.system.max.decimal.value", "999999999999.99");
   
  /**  */
  public static String MIN_DECIMAL_VALUE = 
    getStringProperty("ifs.system.min.decimal.value", "-999999999999.99");
   
  /**  */ 
  public static String NO_SUCH_METHOD_EXCEPTION    = null; //should be set to "getMessage" in init method if
                                                            //a "NoSuchMethodException" is thrown.
  /**  */ 
  public static String GENERIC_EXCEPTION           = null;
   
  /**  */
  public static boolean SHOW_DEBUG_CODE = 
    getBooleanProperty("ifs.system.show.debug.code", false);

  // new defines for the GtLogon info
   
  /**  */
  public static String DefaultCountryCode = 
    getStringProperty("ifs.system.default.country.code", "USA");
   
  /**  */
  public static String DefaultLanguage = 
    getStringProperty("ifs.system.default.language", "ENG");
   
  /**  */
  public static String DefaultValueMissing = 
    getStringProperty("ifs.system.default.value.missing", "ValueMissing");

  //this is checked in the initIfs method
  /**  */
  private static boolean initialized = false;

  /**  */ 
  public static String GOLD_DATE_FORMAT = 
    getStringProperty("ifs.system.gold.date.format", "USA");
   
  /**  */
  public static String GOLD_TIME_FORMAT= 
    getStringProperty("ifs.system.gold.time.format", "USA");

  /**
   * Flag to turn OFF generation of debugging output for all classes.
  */
  public static final int DEBUG_OFF = 
    getIntProperty("ifs.system.debug.off", 0);           // No deugging output

  /**
   * Flag to turn ON generation of debugging output for all classes.
  */
  public static final int DEBUG_ON = 
    getIntProperty("ifs.system.debug.on", 1);          // With debugging output

  /**
   * Flag to turn ON generation of debugging output for those classes which have
   * their debugOn flag set to true.
  */
  public static final int DEBUG_SOME_CLASSES = 
    getIntProperty("ifs.system.debug.some.classes", 2);  // Debugging output only for the classes
                                                         // whoes debugOn flag is set to true.
  /**
   * Controls the generation of debugging output.
   * It can have the values of DEBUG_OFF, DEBUG_ON, or DEBUG_SOME_CLASSES.
  */
  public static int debugMode =                         // T2000163 DEBUG_OFF;
    getIntProperty("ifs.system.debug", DEBUG_OFF);      // T2000163

  /**
   * When true, send debug information to the standard output device.
  */
  public static boolean debugInfoToStdOut = 
    getBooleanProperty("ifs.system.debug.to.std.out", false);

  /**
   * When true, generates debug information in HTML format to the browser.
  */
  public static boolean debugInfoToHtml = 
    getBooleanProperty("ifs.system.debug.to.html", false);

  /**
   * When true, send debug information to the file specified by debugLogFileName.
  */
  public static boolean debugInfoToFile = 
    getBooleanProperty("ifs.system.debug.to.file", false);

  /**
   * The name of the debug output file.
   * Debug output is appended to this file when debugInfoToFile flag is true.
  */
  private static String debugLogFileName = 
    getStringProperty("ifs.system.debug.log.file.name", null);

  // Exception handling
   
  /**  */
  public static boolean exceptionInfoToHtml = 
    getBooleanProperty("ifs.system.exception.info.to.html", false);
   
  /**  */
  public static String exceptionLogFileName = 
    getStringProperty("ifs.system.exception.log.file.name", "/tmp/default.exception.log");

  /**  */ 
  public static final String NO_ENCRYPTION = 
    getStringProperty("ifs.system.no.encryption", "0");
   
  /**  */
  public static final String RSA_ENCRYPTION = 
    getStringProperty("ifs.system.rsa.encryption", "1");
   
  /**  */
  public static int MAX_PIN_LENGTH = 
    getIntProperty("ifs.system.max.pin.length", 128);
   
  /**  */
  public static int MAX_PASSWORD_LENGTH = 
    getIntProperty("ifs.system.max.password.length", 128);
   
  /**  */
  public static int MAX_USER_ID_LEN = 
    getIntProperty("ifs.system.max.user.id.len", 30);

  /**  */
  public static String PUBLIC_ENCRYPTION_KEY = 
    getStringProperty("ifs.system.public.encryption.key", "pubkey.cbsf");

  /**
   * Specifies that the server's locale is to be used to set
   * this session's locale.
  */
  public static final int USE_SERVER_LOCALE = 
    getIntProperty("ifs.system.use.server.locale", 0);

  /**
   * Specifies that the browser's accept-language attribute from the logon
   * page is to be used to set this session's locale.
   * If the logon page does not contain a valid accept-language attribute
   * the server's locale is used to set the locale for this session.
  */
  public static final int USE_BROWSER_LANG_LOGON = 1;

  /**
   * Specifies that the browser's accept-language attribute is checked on
   * each page.  If the browser's accept-language attribute is valid
   * and it is different from this session's locale, this session's locale
   * is updated to the new locale.
   * Whenever, the browser page does not contain a valid accept-language
   * attribute and the session locale is not initialized,
   * the server's locale is used to set the locale for this session.
  */
  public static final int USE_BROWSER_LANG_ALWAYS = 2;

  /**
   * Flag which controls the source for setting of this session's locale.
   * It can have values of USE_SERVER_LOCALE, USE_BROWSER_LANG_LOGON, or
   * USE_BROWSER_LANG_ALWAYS.
  */
  public static int  browserLocaleUsage = 
    getIntProperty("ifs.system.browser.locale.usage", USE_SERVER_LOCALE);

  /**
   * Specifies format for Gold date strings.
  */
  public static GsDateFormat gsDateFormat = new GsDateFormat();
   
  /**
   * Specifies format of Gold decimal data number.
  */
  public static  GsDecimalFormat gsDecimalFormat =
    getGsDecimalFormatProperty("ifs.system.gs.decimal.format",
      new GsDecimalFormat(
        "",        // positivePrefix,
        "",        // positiveSuffix,
        "-",       // negativePrefix,
        "",        // negativeSuffix,
        "-({<",    // inputNegativeChars,
        ',',       // groupingSeparator,
        0,         // groupingSize,
        '.',       // decimalSeparator,
        false,     // decimalSeparatorAlwaysShown,
        0,         // minimumItegerDigits,
        0,         // minimumFractionDigits,
        999,       // maximumFractionDigits,
        false));    // removeTrailingFractionZerosP

  /**
   * Specifies the currency type for input currency text strings and
   * output format for unknown ISO currency codes.
  */
  public static  GsCurrencyFormat gsCurrencyFormat =
    getGsCurrencyFormatProperty("ifs.system.gs.currency.format", 
      new GsCurrencyFormat(
        "USD",     // ISO currency code,
        "$",       // positivePrefix,
        "",        // positiveSuffix,
        "-$",      // negativePrefix,
        "",        // negativeSuffix,
        "-({<",    // inputNegativeChars,
        ',',       // groupingSeparator,
        0,         // groupingSize,
        '.',       // decimalSeparator,
        true,      // decimalSeparatorAlwaysShown,
        1,         // minimumItegerDigits,
        2,         // minimumFractionDigits,
        2,         // maximumFractionDigits,
        false));    // removeTrailingFractionZerosP

  /**
   * Specifies format of Gold currency strings.
   * This is an array of formats, where there may be different
   * formats for different gold currency types.
  */
   public static  GsCurrencyFormat[] gsCurrencyFormatArray = { gsCurrencyFormat };


  /**
   *
  */
  public static  GsCurrencyFormat QIFCurrencyFormat =
    getGsCurrencyFormatProperty("ifs.system.qif.currecny.format", 
      new GsCurrencyFormat(
        "USD",     // ISO currency code,
        "",        // positivePrefix,
        "",        // positiveSuffix,
        "-",       // negativePrefix,
        "",        // negativeSuffix,
        "-({<",    // inputNegativeChars,
        ',',       // groupingSeparator,
        0,         // groupingSize,
        '.',       // decimalSeparator,
        true,      // decimalSeparatorAlwaysShown,
        1,         // minimumItegerDigits,
        2,         // minimumFractionDigits,
        2,         // maximumFractionDigits,
        false));    // removeTrailingFractionZerosP

  /**  */ 
  public static  Font CURRENCY_DEBIT_FONT = 
    getFontProperty("ifs.system.currency.debit.font", null);
   
  /**  */
  public static  Font CURRENCY_CREDIT_FONT = 
    getFontProperty("ifs.system.currency.credit.font", null);

  /**  */
  public static  Color CURRENCY_DEBIT_COLOR = 
    getColorProperty("ifs.system.currency.debit.color", Color.red);
   
  /**  */
  public static  Color CURRENCY_CREDIT_COLOR = 
    getColorProperty("ifs.system.currency.credit.color", Color.black);

  /**  */ 
  public static  Font DECIMAL_POSITIVE_FONT = 
    getFontProperty("ifs.system.decimal.positive.font", null);
  
  /**  */
  public static  Font DECIMAL_NEGATIVE_FONT = 
    getFontProperty("ifs.system.decimal.negative.font", null);
   
  /**  */
  public static  Color DECIMAL_POSITIVE_COLOR = 
    getColorProperty("ifs.system.decimal.positive.color", Color.black);
   
  /**  */
  public static  Color DECIMAL_NEGATIVE_COLOR = 
    getColorProperty("ifs.system.decimal.negative.color", Color.red);

  /**
   * The Locale default value.
   * Note: It is assumed that during WebSphere configuration, the administrator will
   * set the default locale for the server and that will in turn set the default
   * locale in class Locale.
  */
   public static  Locale SERVER_LOCALE = 
     getLocaleProperty("ifs.system.server.locale", Locale.getDefault());  // SEE EXAMPLES SECTION FOR
                                                                          // USAGE OF THIS CONSTANT.

  /**  */
  public static  PrintWriter debugLogFile;

  /**
   * The list of business response codes for Logon transaction requests
   * that should be considered as successful by the web server.
   * The codes are 3 character String objects.
  */
  public static  String[] WSP_OK_LOGON_BRC = 
    getStringArrayProperty("ifs.system.wsp.ok.logon.brc", 
                           new String[] {"000", "067", "029"});

  /**
   * The list of business response codes for PINPSWD transaction requests
   * that should be considered as successful by the web server.
   * The codes are 3 character String objects.
  */
  public static  String[] WSP_OK_PINPSWD_BRC = 
    getStringArrayProperty("ifs.system.wsp.ok.pinpswd.brc", 
                           new String[] {"000"});

  /**
   * The list of business response codes for REAUTH transaction requests
   * that should be considered as successful by the web server.
   * The codes are 3 character String objects.
  */
  public static  String[] WSP_OK_REAUTH_BRC = 
    getStringArrayProperty("ifs.system.wsp.ok.reauth.brc", 
                           new String[] {"000"});

  /**
   * The key by which the client remote address is saved in the session. (See PTR2000147)
  */
  public static final String CLIENT_ADDR = "client.remoteAddr";


/* -------------------------- Static Initializers --------------------------- */

  // Load properties files
  static {
    
    boolean success1 = populate3tkTypesDictionary();
    boolean success2 = populateBrcTypesDictionary();
    boolean success3 = populateCwsTypesDictionary();
    initialized = success1 && success2 && success3;

  }

/* ---------------------------- Class Methods ------------------------------- */

  /**
   * When debugging is enabled the debug information is sent to the standard
   * output, a log file, or a combination of the previous 2 output vehicles.
   * IfsSystem debug attributes control the enabling and disabling of debug
   * output and to where the debug information is outputted.
   *
   * @param str -- The string of debug information.
  */
  public static void debug(String str) {
    if (IfsSystem.debugMode == IfsSystem.DEBUG_OFF) 
      return;

    if (IfsSystem.debugInfoToStdOut)
       System.out.println(str);

    if (IfsSystem.debugInfoToFile) {
      try {
        if (IfsSystem.debugLogFile == null) {
          // true is for auto flush
          IfsSystem.debugLogFile =
            new PrintWriter(new FileOutputStream(IfsSystem.getDebugLogFileName()), true);
        }
        IfsSystem.debugLogFile.println(str);
      }
      catch (Exception ex) { 
        /*....*/
      }
    }
  }

  /**
   * Returns the Hashtable which contains Three Token (3tk) errors and
   * it indicates if the code is a FATAL, REDO, or INFO type of completion.
   * Most of these are fatal errors.
  */
  public static Hashtable get3tkTypesDictionary() {
    if (null == all3tkTypesDictionary) {
      all3tkTypesDictionary = new Hashtable ();
      populate3tkTypesDictionary ();
    }

    return all3tkTypesDictionary;
  }

  /**
   * Returns the Hashtable which contains business response codes (BRC) and
   * it indicates if the code is a FATAL, REDO, or INFO type of completion.
   * The BRC are completion codes for Core Banking Business, Bill Payment Business,
   * Bill Presentment, REDO from Three Token Errors, and others.
  */
  public static Hashtable getBrcTypesDictionary() {
     if (null == allBrcTypesDictionary) {
       allBrcTypesDictionary = new Hashtable();
       populateBrcTypesDictionary ();
     }

     return allBrcTypesDictionary;
  }

  /**
   * Returns the Hashtable which contains CWS types completion codes and
   * it indicates if the code is a FATAL, REDO, or INFO type of completion.
  */
  public static Hashtable getCwsTypesDictionary() {
    if (null == allCwsTypesDictionary) {
      allCwsTypesDictionary = new Hashtable ();
      populateCwsTypesDictionary ();
    }

    return allCwsTypesDictionary;
  }

  /**
   * Returns the name of the debug output file.
   * Debug output is appended to this file when debugInfoToFile flag is true.
  */
   public static String getDebugLogFileName() { 
     return debugLogFileName;
   }

   /**
    *
   */
   public void init(ServletConfig config) throws ServletException {
      //initIfs();
   }

  /**
   * This method is used by the servlet writer to initialize IfsSystem.
   * If IfsSystem class has not already been initialized,
   * it is initialized and methods are also registered for the various classes.
   * Always returns true.
  */
  public static boolean initIfs() {
    if ( !initialized ) {
       //populate Type Dictionaries

       getBrcTypesDictionary();
       getCwsTypesDictionary();
       get3tkTypesDictionary();

       //---------------
       // Example of how to register methods for various classes.
       //
       // ------------ register validator and fromSting for a gold primitive.
       //  GsDate date = new GsDate();
       //  Class theClass = date.getClass();
       //  try
       //  {
       //     date.registerMethod("validate",
       //            theClass.getMethod("validateGsDate",
       //                          new Class[]{GsDate.class}));
       //     date.registerMethod("fromString",
       //              theClass.getMethod("gsDateFromString",
       //                    new Class[]{GsDate.class, String.class}));
       //  }
       //  catch(NoSuchMethodException e) {NO_SUCH_METHOD_EXCEPTION = e.getMessage();}
       //
       // ------------- register validator and fromSting for a transaction
       //  GtAddPayee aPayee = new GtAddPayee();
       //  theClass = aPayee.getClass();
       //  try
       //  {
       //     date.registerMethod("validate",
       //              theClass.getMethod("validateGtAddPayee",
       //                            new Class[]{GtAddPayee.class}));
       //     date.registerMethod("fromString",
       //              theClass.getMethod("gtAddPayeeFromString",
       //                    new Class[]{GtAddPayee.class, String.class}));
       //  }
       //  catch(NoSuchMethodException e) {NO_SUCH_METHOD_EXCEPTION = e.getMessage();}
       //
       // ------------------------

       initialized = true;
     }
     return initialized;
   }

   /**
   * This private method is used by the service method when IfsSystem servlet is first
   * started by WebSphere.  If IfsSystem class has not already been initialized,
   * it is initialized and methods are also registered for the various classes.
   * Always returns true.
   */
   private static boolean initIfs(HttpServletRequest req, HttpServletResponse resp)
   {
      if (! initialized)
      {

         //populate Type Dictionaries

         getBrcTypesDictionary();
         getCwsTypesDictionary();
         get3tkTypesDictionary();

         //---------------
         // Example of how to register methods for various classes.
         //
         // ------------ register validator and fromSting for a gold primitive.
         //  GsDate date = new GsDate();
         //  Class theClass = date.getClass();
         //  try
         //  {
         //     date.registerMethod("validate",
         //            theClass.getMethod("validateGsDate",
         //                          new Class[]{GsDate.class}));
         //     date.registerMethod("fromString",
         //              theClass.getMethod("gsDateFromString",
         //                    new Class[]{GsDate.class, String.class}));
         //  }
         //  catch(NoSuchMethodException e) {NO_SUCH_METHOD_EXCEPTION = e.getMessage();}
         //
         // ------------- register validator and fromSting for a transaction
         //  GtAddPayee aPayee = new GtAddPayee();
         //  theClass = aPayee.getClass();
         //  try
         //  {
         //     date.registerMethod("validate",
         //              theClass.getMethod("validateGtAddPayee",
         //                            new Class[]{GtAddPayee.class}));
         //     date.registerMethod("fromString",
         //              theClass.getMethod("gtAddPayeeFromString",
         //                    new Class[]{GtAddPayee.class, String.class}));
         //  }
         //  catch(NoSuchMethodException e) {NO_SUCH_METHOD_EXCEPTION = e.getMessage();}
         //
         // ------------------------

         initialized = true;
      }
      return initialized;
  }

  /**
   * Three Token (3tk) errors, most of which are fatal.
  */
  private static boolean populate3tkTypesDictionary() {
    boolean ret = true;
    all3tkTypesDictionary = new Hashtable();
    try {
      loadProperties(THREE_TOKEN_CODES_PROPERTY_FILE, all3tkTypesDictionary);
    }
    catch (MissingResourceException e) {
      e.printStackTrace();
      System.err.println("ERROR could not load 3 token codes from property " +
        "file " + THREE_TOKEN_CODES_PROPERTY_FILE + " the codes must be "    + 
        "loaded for CWS to function properly: " + e.getMessage());
      ret = false;
      all3tkTypesDictionary = null;
    }
    return ret;
  }

  /**
   * Populate the business response code dictionary with completion codes for
   * Core Banking Business, Bill Payment Business, Bill Presentment,
   * REDO from Three Token Errors, and others.
  */
  private static boolean populateBrcTypesDictionary() {
    boolean ret = true;
    allBrcTypesDictionary = new Hashtable();
    try {
      loadProperties(BRC_CODES_PROPERY_FILE , allBrcTypesDictionary);
    }
    catch (MissingResourceException e) {
      e.printStackTrace();
      System.err.println("ERROR could not load BRC codes from property file " +
        BRC_CODES_PROPERY_FILE + " the codes must be loaded for CWS to "      +
        "function properly: " + e.getMessage());
      ret = false;
      allBrcTypesDictionary = null;
    }
    return ret;

  }

  /**
   * Populate the cws return types dictionary with completion codes.
  */
  private static boolean populateCwsTypesDictionary() {
    boolean ret = true;
    allCwsTypesDictionary = new Hashtable();
    try {
      loadProperties(CWS_CODES_PROPERTY_FILE, allCwsTypesDictionary);
    }
    catch (MissingResourceException e) {
      e.printStackTrace();
      System.err.println("ERROR could not load CWS codes from property file " +
        CWS_CODES_PROPERTY_FILE + " the codes must be loaded for CWS to "     +
        "function properly: " + e.getMessage());
      ret = false;
      allCwsTypesDictionary = null;
    }
    return ret;

  }

  /**
   * This service method is used when IfsSystem servlet is first
   * started by WebSphere.
  */
  public void service(HttpServletRequest  request,
    HttpServletResponse response) throws ServletException {

    ResourceBundle aBundle;
    Locale locale = null;
    locale =  IfsSystem.SERVER_LOCALE;
    String bundleText = null;

    initIfs(request,response);
    // Display system acknowledgement and status in this page
    // ...
    OutputStream outputStream = null;
    boolean exceptionFlag = false;
    PrintWriter out = null;
    try {
      response.setContentType("text/html; charset=" + IfsSystem.cwsBrowserCharSet);
      out = response.getWriter();
    }
    catch (IOException exception) {
      out = new PrintWriter(outputStream);
      out.println("An IOException has occured: " + exception.getMessage() );
    }

    aBundle = ResourceBundle.getBundle("IfsScreenTextBundle", locale);
    bundleText =  aBundle.getString("Welcome");
    out.println("<html>");
    out.println("<head>");
    out.println("<title>");
    out.println("IfsSystem Start-Up Page");
    out.println("</TITLE>");
    out.println("</HEAD>");

    out.println("<BODY BGCOLOR='#FFFFFF' BACKGROUND=''>");
    out.println("<TABLE WIDTH='100%' BORDER='0' CELLSPACING='0' CELLPADDING='0'>");
    out.println("<TR><TD><IMG SRC='/images/3dlogo.gif' ALIGN='TOP' BORDER='0'></TD></TR></TABLE>");
    out.println("<BR>");
    out.println("<TABLE WIDTH='100%' BORDER='0' CELLSPACING='0' CELLPADDING='0'>");
    out.println("<TR><TD><IMG SRC='/images/3dbanner.gif' ALIGN='TOP' BORDER='0'></TD></TR></TABLE>");
    out.println("<BR>");

    out.println("<TABLE>");
    out.println("<TR><TD>");
    //   out.println("WELCOME TO THE IFSSYTEM SERVLET STARTUP PAGE");
    out.println(bundleText);
    out.println("</TD></TR>");
    out.println("</TABLE>");
    out.println("<BR><BR><BR><BR><BR><BR><BR>");

    if (NO_SUCH_METHOD_EXCEPTION != null) {
      out.println("<TABLE>");
      out.println("<TR><TD>");
      out.println("A \"NoSuchMethodException\" was thrown while registering a method: \n");
      out.println("NO_SUCH_METHOD_EXCEPTION");
      out.println("</TD></TR>");
      out.println("</TABLE>");
      exceptionFlag = true;
    }
    else {
      out.println("<BR><BR><BR><BR><BR><BR><BR><BR><BR><BR><BR><BR><BR><BR>");
    }

    if (!exceptionFlag) {
      out.println("<TABLE>");
      out.println("<TR><TD>");
      bundleText =  aBundle.getString("Welcome");
      //      out.println("<A href=Logon>Please Click Here To Enter Webbank's Online Banking Service</A>");
      out.println("<A href=Logon>" + bundleText + "</A>");
      out.println("</TD></TR>");
      out.println("</TABLE>");
    }

    out.println("</BODY> </HTML>");
  }

  /**
   * Sets the name of the debug output file.
   * Debug output is appended to this file when debugInfoToFile flag is true.
  */
  public static void setDebugLogFileName(String logFileName) {
    if (null != debugLogFile) {
      debugLogFile.flush();
      debugLogFile.close();
      debugLogFile = null;
    }
    debugLogFileName = logFileName;
 }

  /**
   *
  */
  private static void loadProperties(String fileName, Hashtable cache) 
    throws MissingResourceException {

    ResourceBundle bundle = ResourceBundle.getBundle(fileName);
    Enumeration e = bundle.getKeys();
    while (e.hasMoreElements()) {
      String key = (String)e.nextElement();
      cache.put(key, bundle.getString(key));
    }
  }
  
  /**
   * This method will read the copyright text stored in the 
   * properties file IfsSystem.properties.
   *
   * @return String The property value for the key <I>propertyName</I>
   *         or null if the IfsSystem.properties file could not be found
   *         or the property with the key <I>propertyName</I> was not
   *         defined in the IfsSystem.properties file.
  */
  public static String getIfsSystemProperty(String propertyName){
    return (String)systemProperties.get(propertyName);
  }	// d7400

  /**
   * Reads a String from an IFS System property.
   *
   * @param property The property name.
   * @param defaultValue The String to return if the property is not defined.
   * @return String The <CODE>property</CODE> or <CODE>defaultValue</CODE> if the
   *         property is not defined.
  */
  public static String getStringProperty(String property, String defaultValue) {
    if (systemProperties == null)
      initSystemProperties();

    String value = (String)systemProperties.get(property);
    return (value == null ? defaultValue : value);
  }

  /**
   * Reads a boolean from an IFS System property.
   *
   * @param property The property name.
   * @param defaultValue The String to return if the property is not defined.
   * @return boolean True if the property is defined and has the value "true"
   *         ignoring case, false if the property is defined and has a value 
   *         other than true, or <CODE>defaultValue</CODE> if the property is
   *         not defined.
  */
  public static boolean getBooleanProperty(String property, boolean defaultValue) {
    if (systemProperties == null)
      initSystemProperties();

    String value = (String)systemProperties.get(property);
    return (value == null ? defaultValue : Boolean.valueOf(value).booleanValue());
  }

  /**
   * Reads an int from an IFS System property.
   *
   * @param property The property name.
   * @param defaultValue The int to return if the property is not defined
   *        or is not a valid integer.
   * @return int The int specified by <CODE>property</CODE> or
   *         <CODE>defaultValue</CODE> if the property is not defined or
   *         is not a valid integer.
  */
  public static int getIntProperty(String property, int defaultValue) {
    if (systemProperties == null)
      initSystemProperties();

    String value = (String)systemProperties.get(property);
    int ret = defaultValue;
    if (value != null) {
      try {
        ret = Integer.parseInt(value);
      }
      catch (NumberFormatException e) {
        System.err.println("ERROR parsing system property: " + property +
          " Value " + value + " must be a valid integer, using default "+   
          " value of " + defaultValue);
      }
    }
    return ret;
  }

  /**
   * Initializes the internal cache of IFS system properties.
   *
   * @return void
  */
  private static void initSystemProperties() {
    try {
      systemProperties = new Hashtable();
      loadProperties(IFS_SYSTEM_PROPERTY_FILE, systemProperties);
    } 
    catch (MissingResourceException e) {
      e.printStackTrace();
      System.err.println("ERROR could not load properties file "           +
        IFS_SYSTEM_PROPERTY_FILE + " this file must be loaded for CWS to " +
        "function properly: " + e.getMessage());
    }
  }

  /**
   * Constructs a com.ibm.ifs.gold.GsDecimalFormat Object bases on an IFS System property.
   * The property value is a color ':' delimited list of the parameters to pass to the 
   * GsDecimalFormat constructor.
   *
   * @param property The name of the property to read.
   * @param defaultFormat The GsDecimalFormat to return if the property is not defined
   *        or has an invalid format.
   * @return GsDecimalFormat The GsDecimalFormat Object constructed from the <CODE>property</CODE>
   *         specification or <CODE>defaultFormat</CODE> if the property is not defined or has an
   *         invalid form.
  */
  public static GsDecimalFormat getGsDecimalFormatProperty(String property, GsDecimalFormat defaultFormat) {
    if (systemProperties == null)
      initSystemProperties();

    GsDecimalFormat ret = defaultFormat;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        StringTokenizer tokenizer = new StringTokenizer(value, ":");
        int i = 0;
        String[] params = new String[13];
        while (tokenizer.hasMoreTokens() && i < params.length) {
          params[i++] = tokenizer.nextToken();
        }

        // Construct the GsDecimalFormat Object
        ret = new GsDecimalFormat( params[0].trim(),
                                   params[1].trim(),
                                   params[2].trim(),
                                   params[3].trim(),
                                   params[4].trim(),
                                   params[5].charAt(0),
                                   Integer.parseInt(params[6]),
                                   params[7].charAt(0),
                                   Boolean.valueOf(params[8]).booleanValue(),
                                   Integer.parseInt(params[9]),
                                   Integer.parseInt(params[10]),
                                   Integer.parseInt(params[11]),
                                   Boolean.valueOf(params[12]).booleanValue());
        
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property     +
          " Value " + value + " must be a ':' delimited list of parameters" +
          "to pass to the GsDecimalFormat class. Using default format "     +   
          defaultFormat);
      }
    }
    return ret;
  }

  /**
   * Constructs a com.ibm.ifs.gold.GsCurrencyFormat Object bases on an IFS System property.
   * The property value is a color ':' delimited list of the parameters to pass to the 
   * GsCurrencyFormat constructor.
   *
   * @param property The name of the property to read.
   * @param defaultFormat The GsCurrecnyFormat to return if the property is not defined
   *        or has an invalid format.
   * @return GsCurrencyFormat The GsCurrencyFormat Object constructed from the <CODE>property</CODE>
   *         specification or <CODE>defaultFormat</CODE> if the property is not defined or has an
   *         invalid form.
  */
  public static GsCurrencyFormat getGsCurrencyFormatProperty(String property, GsCurrencyFormat defaultFormat) {
    if (systemProperties == null)
      initSystemProperties();

    GsCurrencyFormat ret = defaultFormat;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        StringTokenizer tokenizer = new StringTokenizer(value, ":");
        int i = 0;
        String[] params = new String[14];
        while (tokenizer.hasMoreTokens() && i < params.length) {
          params[i++] = tokenizer.nextToken();
        }

        // Construct the GsCurrencyFormat Object
        ret = new GsCurrencyFormat( params[0].trim(),
                                    params[1].trim(),
                                    params[2].trim(),
                                    params[3].trim(),
                                    params[4].trim(),
                                    params[5].trim(),
                                    params[6].charAt(0),
                                    Integer.parseInt(params[7]),
                                    params[8].charAt(0),
                                    Boolean.valueOf(params[9]).booleanValue(),
                                    Integer.parseInt(params[10]),
                                    Integer.parseInt(params[11]),
                                    Integer.parseInt(params[12]),
                                    Boolean.valueOf(params[13]).booleanValue());
        
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property     +
          " Value " + value + " must be a ':' delimited list of parameters" +
          "to pass to the GsCurrencyFormat class. Using default format "     +   
          defaultFormat);
      }
    }
    return ret;
  }


  /**
   * Constructs a java.util.Locale Object based on an IFS System property.
   * The property value's form is <I>language_country_variant</I> where
   * <I>language</I> is a lowercase two -letter ISO639 language code, <I>country</I>
   * is a uppercase two-letter ISO-3166 country code and <I> variant</I> is
   * optional and provides more detailed locale information. Example:
   * <CODE>en_US_NY</CODE>.
   *
   * @param property The name of the property to read.
   * @param defaultLocale Returned if the specified property is not present or has
   *        an invalid form.
   * @return Locale The Locale Object constructed from the <CODE>property</CODE> 
   *         specification or <CODE>defaultLocale</CODE> if the property is not
   *         specified or has an invalid format.
  */
  public static Locale getLocaleProperty(String property, Locale defaultLocale) {
    if (systemProperties == null)
      initSystemProperties();
    
    Locale ret = defaultLocale;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        String[] params = new String[3];
        StringTokenizer tokenizer = new StringTokenizer(value, "_");
        int i = 0; 
        while (tokenizer.hasMoreTokens() && i < params.length) {
          params[i++] = tokenizer.nextToken();
        }
        ret = params[2] == null ? new Locale(params[0], params[1]) : new Locale(params[0], params[1], params[2]);
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property                  +
          " Value " + value + " Must be in the form <language>_<country>_<variant> "     +
          "where <language> is the lowercae two-letter ISO639 langauge code, <country> " +
          "is the uppercase two-letter ISO-3166 country code and <variant> is optional " +
          "and provides more detailed locale information. For example en_US_NY.");
      }
    }
    return ret;
  }

  /**
   * Contructs a String array Object based on an IFS System property.
   * The property value is a space seperated list of Strings.
   *
   * @param property The name of the property to read.
   * @param defaultValue The String array to return if the property is not
   *        defined or if it has an invalid format.
   * @return String[] The String array constructed using the specification defined
   *         by <CODE>property</CODE> or <CODE>defaultValue</CODE> if the property
   *         is not defined or has an invalid format. 
  */
  public static String[] getStringArrayProperty(String property, String[] defaultValue) {
    if (systemProperties == null)
      initSystemProperties();

    String[] ret = defaultValue;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        Vector v = new Vector();
        StringTokenizer tokenizer = new StringTokenizer(value, " ");
        while (tokenizer.hasMoreTokens()) {
          v.add(tokenizer.nextToken().trim());
        }
        ret = new String[v.size()];
        for (int i = 0; i < ret.length; ++i) {
          ret[i] = (String)v.elementAt(i);
        }
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property +
          " Value " + value + " must be a space delimited list of values");
      }
    }
    return ret;
  }

  /**
   * Contructs a java.awt.Color Object based on an IFS System property.
   * The property value is three integers <I>x</I>, such that -1 &lt; 
   * <I>x</I> &gt; 256 that specify the Red, Green and Blue components
   * of the color respectivley.
   *
   * @param property The name of the property that contains the specification
   *        for the Color.
   * @param defaultColor The Color to return if the property is not defined
   *        or has an invalid format.
   * @return Color The Color specified by <CODE>property</CODE> or
   *         <CODE>defaultColor</CODE> if the property is not defined or has
   *         an invalid format.
  */
  public static Color getColorProperty(String property, Color defaultColor) {
    if (systemProperties == null)
      initSystemProperties();

    Color ret = defaultColor;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        StringTokenizer tokenizer = new StringTokenizer(value, " ");
        ret = new Color(Integer.parseInt(tokenizer.nextToken()), 
                        Integer.parseInt(tokenizer.nextToken()), 
                        Integer.parseInt(tokenizer.nextToken()));
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property +
          " Value " + value + " must be a space separated list of RGB values." +
          " For example \"255 255 255\" is white");
      }
    }
    return ret;
  }

  /**
   * Constructs a java.awt.Font Object based on an IFS System property.
   * The form of the property is <I>name</I>:<I>style</I>:<I>size</I> where
   * <I>name</I> is the name of the font, <I>style</I> is either <CODE>
   * PLAIN, BOLD, ITALIC</CODE> or <CODE>BOLD|ITALIC</CODE> and <I>size</I>
   * the point size of the Font.
   *
   * @param property The name of the property that contains the specification
   *        for the Font.
   * @param defaultFont The Font to use if the property is not defined or
   *        if there was a problem parsing the property.
   * @return Font The Font specified by <CODE>property</CODE> or <CODE>
   *         defaultFont</CODE> if the property is not defined or is
   *         not in the correct format.
  */
  public static Font getFontProperty(String property, Font defaultFont) {
    if (systemProperties == null)
      initSystemProperties();

    Font ret = defaultFont;
    String value = (String)systemProperties.get(property);
    if (value != null) {
      try {
        StringTokenizer tokenizer = new StringTokenizer(value, ":");
        String face = tokenizer.nextToken();
        String styleSpec = tokenizer.nextToken();
        int size = Integer.parseInt(tokenizer.nextToken());
        int style = Font.PLAIN;
        if (styleSpec.equals("BOLD")) {
          style = Font.BOLD;
        }
        else if (styleSpec.equals("ITALIC")) {
          style = Font.ITALIC;
        }
        else if (styleSpec.equals("BOLD|ITALIC")) {
          style = Font.BOLD | Font.ITALIC;
        }
        ret = new Font(face, style, size);
      }
      catch (Exception e) {
        e.printStackTrace();
        System.err.println("ERROR parsing system property: " + property +
          " Value " + value + " must be of the form \"<name>:<style>:<size>\" " +
          "where <name> is the name of the font, style is either 'PLAIN', 'BOLD' " +
          "'ITALIC' or 'BOLD|ITALIC' and size is the point size of the font. " +
          "For example: 'Times New Roman:BOLD|ITALIC:22'");
      }
    }
    return ret;
  }

} // End com.ibm.ifs.gold.IfsSystem class
