package com.ibm.ifs.gold;

// ***************************************************************************
//
//  FILE NAME:   IfsTxn.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//     US Government Users Restricted Rights - Use, duplication
//     or disclosure restricted by GSA ADP Schedule Contract
//     with IBM
//
//  REVISION INFO:
//
//    SID:       1.2
//    Timestamp: 02/07/17 17:54:39
//
//  CHANGES:
//     8/6/99  kan   d5322R   promoted get*FromURL to IfsObject.java
//     8/9/99  kan   d5324 added check similar to javascript for BRC checking
//     8/23/99   larock   d5449 disable Gold Manager cacheing
//     9/21/99    srs     d5582   reenabled Gold Manager cacheing
//    02/17/2000  rhs   d6361   Made JavaDoc correction/addtions.
//    03/20/2000  rhs   d6545   Added NLS support & JavaDoc comments.
//    05/08/2000  rhs   d6682   Added check in packNlsChar to assure that the
//                              string length is not greater than specified width.
//    06/01/2000  rhs   d6850   Added support for null input strings in packChar,
//                              packNlsChar, and packVarChar.
//    06/09/2000  rhs   d6868   Changed support for null input strings in packChar,
//                              packNlsChar, and packVarChar.
//    06/13/2000  rhs   d6884   Changed handling of NLS fields so they are not
//                              dependent on IfsApi.MAX_FIXED_STRING_LENGTH.  Instead
//                              they are always treated as variable length fields.
//    01/30/2001  tlt   d7233   Port sla support to v1r6, also included T0962113, 7172   
//    02/12/2001  tlt  T0979766 use truncated string size not vlen
//    02/12/2001  mbc  T0962907 use reason code instead of comp code when
//                              displaying error messages
//    02/02/2002  tlt  T1000102 allow GMJI buffer to grow to 131072
//    07/15/2002  dob   T2000147 Save/retrieve client address to session.
// ***************************************************************************

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Hashtable;

/**
* IfsTxn class abstracts the common properties and behaviors of all the
* IFS transaction objects.  The common features include GMJI initialization,
* data convertion, logging, execution, error handling, etc..
* Every transaction class should inherit from IfsTxn.
*/
public abstract class IfsTxn extends IfsObject implements ICustomizableTxnMethods
{
   //---------------------------------------------------------------------------------------
   // public static final variables
   //---------------------------------------------------------------------------------------

   /**
   * Controls the generation of debugging output for this class when
   * IfsSystem.debugMode equals DEBUG_SOME_CLASSES.
   */
   public static boolean debugOn = false;

   /**
   * Specifies that the transaction is to be executed by sending it
   * to the Financial Instition (FI).
   * This constant is used as an input parameter to the execute method.
   */
   public static final int FI = 1;

   /**
   * Specifies that the transaction data is to be obtained from the Gold Manager cache
   * and the transaction is not executed by sending it to the Financial Instition (FI).
   * This constant is used as an input parameter to the execute method.
   */
   public static final int CACHE = 0;

   /**
   * Specifies that the transaction is to be executed by sending it
   * to the Financial Instition and the MTI bit stream message
   * is to be obtained from the undelivered message transaction.
   * This constant is used as an input parameter to the execute method.
   */
   public static final int UNDELIVERED_FLAG = 4;

   /**
   * Specifies that the transaction data is to be obtained from the Gold Manager cache.
   * When the transaction data is not available in the Gold Manager cache,
   * the transaction is executed by sending it to the Financial Instition (FI).
   * This constant is used as an input parameter to the execute method.
   */
   public static final int CACHEFI = 8;

   /**
   * Specifies that the transaction returns a response code.
   * ContainsRespCode field can be set to this constant.
   */
   public static final int RETURNS_RESPONSE_CODE = 1;

   /**
   * Specifies that the transaction does not return a response code.
   * ContainsRespCode field can be set to this constant.
   */
   public static final int NO_RESPONSE_CODE = 0;

   /**
   * Specifies that the data (pin, password, etc.) is to be
   * RSA encrypted when true and not encrypted when false.
   */
   /** 10/18/00 PSB :used to be 'final' and true, but HSBC finds this
   * problematic - their test env's do not include RSA so
   * they cannot test with 'always encrypt'
   **/
   public static boolean ENCRYPT = true;

   /**
   * Specifies the Business Response Code value that is
   * to be considered successul.
   */
   public static final String VALIDBRC = "000";

   /**
   * Specifies a Core Controller Warning
   */
   public static final String IFSCC_WARNING = "2";

   /**
   * Specifies a Core Controller Application error
   */
   public static final String IFSCC_APPLICATION = "4";

   /**
   * Specifies a Core Controller Internal error
   */
   public static final String IFSCC_INTERNAL = "6";

   //---------------------------------------------------------------------------------------
   // private static final variables
   //---------------------------------------------------------------------------------------

   // The following data types correspond to values used in Gold Manager C code.
   // They also correspond to the NS LiveWire callC interface values.
   private static final int LW_NULL=1;
   private static final int LW_INTEGER=2;
   private static final int LW_DOUBLE=3;
   private static final int LW_STRING=4;
   private static final int LW_BOOLEAN=5;

   private static final int GMOB_ACRO_OFFSET = 0;
   private static final int GMOB_ACRO_OFFSET_LEN = 4;
   private static final int GMOB_CODE_OFFSET = 4;
   private static final int GMOB_CODE_OFFSET_LEN = 4;

   private static final String GM_TXN_OK = "0000";
   private static final String GM_TXN_3_TOKEN = "!CC!";
   private static final String GM_TXN_CWS = "CWS!";

   private static final int initGMJISize = 4096;
   private static final int biggerGMJISize = 65536;
   private static final int largestGMJISize = 131072;  // T1000102 
   private static final int toStringIncr = 4096; // T1000201

   private static final String ERROR_BAD_GMJO_ACRONYM = "2007";
   private static final String ERROR_BAD_GMJO_ERROR_CODE = "2008";

   //---------------------------------------------------------------------------------------
   // protected instance variables
   //---------------------------------------------------------------------------------------

   protected boolean logonTxn = false;
   protected boolean selfRegTxn = false;
   protected boolean changePinPasswordTxn = false;
   protected boolean reauthTxn = false;
   protected boolean logoffTxn = false;
   protected boolean slaTxn = false;
   protected boolean oboLogonTxn = false;

   /**
   * The character set used in the Gold manager data streams (GMJI & GMJO)
   * to pass data to/from the CWS.
   * <P>
   * Browsers expect the character set name to be "UTF-8" while the JDK uses
   * "UTF8".  IANA (Internet Assigned Numbers Authority) has declared "UTF-8"
   * to be the preferred name.  JDK 1.1.6 supposedly has added an alias for
   * "UTF-8" but for maximum portability across Java versions,
   * "UTF8" is being used.
   */
   private static  String GMJ_IO_CHARSET = "UTF8";

   /**
   * Gold Manager Java Input buffer.
   * This buffer is used to pass data from java code to Gold Manager.
   */
   protected byte[] gmji;

   /**
   * This gmjiCursor indexes to position of next byte to add, not last byte used.
   * It is also the number of bytes used in the GMJI.
   */
   protected int gmjiCursor;

   /** Specifies the number of arguments in the GMJI. */
   protected int gmjiArgCount;

   /** Size of a Gold long variable in the GMJO. */
   public static final int GMJO_SIZEOF_LONG   = 12;

   /** Size of a Gold double variable in the GMJO. */
   public static final int GMJO_SIZEOF_DOUBLE = 24;

   /**
   * Size of the length field at beginning of variable length data in the GMJO.
   * This length of the data is expressed in hex characters.
   */
   public static final int GMJO_VLEN_LENGTH   = 4;

   /** Maximum length of variable length data in the GMJO. */
   public static final int GMJO_MAX_VARCHAR_LENGTH     = 65535;

   /**
   * Gold Manager Java Output buffer.
   * This buffer is used to pass data to java code from Gold Manager.
   */
   protected byte[] gmjo;

   /**
   * This gmjoCursor indexes to position of next byte to read from GMJO.
   */
   protected int gmjoCursor;

   /**
   * When true the GMJO has returned Gold data that needs to be parsed.
   * When false the GMJO does not contain Gold data.
   */
   protected boolean goldData = false;

   /**
   * When true the transaction input data is validated via registered methods
   * or default methods (when no validation methods are registered).
   * It is initially set to IfsSystem.IS_VALIDATE_ACTION_REQUIRED.
   */
   protected boolean validateActionRequired = IfsSystem.IS_VALIDATE_ACTION_REQUIRED;

   /**
   * When true the method registered to <EM>requestReadyAction</EM> is called
   * to process bank specific logic.  This call is made after the GMJI has been
   * created and before the transaction is sent to the Gold Manager for execution.
   * It is initially set to IfsSystem.IS_REQUEST_READY_ACTION_REQUIRED.
   */
   protected boolean requestReadyActionRequired =
                IfsSystem.IS_REQUEST_READY_ACTION_REQUIRED;

   /**
   * When true the method registered to <EM>responseReadyAction</EM> is called
   * to process bank specific logic.  This call is made after the GMJO has been
   * received from the Gold Manager and the GMJO has been parsed into the java objects.
   * It is initially set to IfsSystem.IS_RESPONSE_READY_ACTION_REQUIRED.
   */
   protected boolean responseReadyActionRequired =
                IfsSystem.IS_RESPONSE_READY_ACTION_REQUIRED;

   /**
   * Flag for setting transaction executionMode to no execution.
   */
   public static final int NOT_EXECUTE   = 0;

   /**
   * Flag for setting transaction executionMode to always execute.
   */
   public static final int EXECUTE       = 1;

   /**
   * Flag for setting transaction executionMode to not execute a transaction
   * again when it has previously been executed successfully in this session.
   * That is, the previous transaction status was OK.
   */
   public static final int EXECUTE_SMART = 2;

   /**
   * This attribute controls the transaction execution mode.
   * It can have values of NOT_EXECUTE, EXECUTE, and EXECUTE_SMART.
   */
   protected int executionMode = EXECUTE;

   /** One of possible return/error codes from transaction exection. */
   public static final int NO_EXECUTION = 0;

   /** One of possible return/error codes from transaction exection. */
   public static final int OK = 1;

   /** One of possible return/error codes from transaction exection. */
   public static final int INFO_ERROR = 2;

   /** One of possible return/error codes from transaction exection. */
   public static final int REDO_ERROR = 3;

   /** One of possible return/error codes from transaction exection. */
   public static final int FATAL_ERROR = 4;

   /** One of possible return/error codes from transaction exection. */
   public static final int VALIDATION_ERROR = 5;

   /** One of possible return/error codes from transaction exection. */
   public static final int REQUEST_READY_ACTION_ERROR = 6;

  /** One of possible return/error codes from transaction exection. */
  public static final int RESPONSE_READY_ACTION_ERROR = 7;

   /** One of possible return/error codes from transaction exection. */
   public static final int THROWABLE_ERROR = 8;

   /** One of possible return/error codes from transaction exection. */
   public static final int INVALID_ERROR = 9;

   /** One of possible return/error codes from transaction exection. */
   public static final int FORMAT_ERROR = 10;

   /** One of possible return/error codes from transaction exection. */
   public static final int ENCRYPTION_FAILURE_ERROR = 11;

   /** One of possible return/error codes from transaction exection. */
   public static final int SUBCLASS_ERROR = 12;

   /** The 3 token error code indicator value. */
   public static final int TXN_3_TOKEN = 1;

   /** The CWS error code indicator value. */
   public static final int TXN_CWS = 2;

   /** The Business Response Code indicator value. */
   public static final int TXN_BRC = 3;

   /** Contains the transaction execution status code. */
   protected int status;

   /** Contains the type of error (3TK, CWS, FI). */
   protected int errorCode;

   /** Contains the transaction Business Response Code. */
   protected String brc;

   /**
   * Contains a translatable message about the error.
   */
   protected String message;

   /**
   * Text obtained from resource or properties files for return codes from a gold message.
   */
   protected String gmMessage;

   /** Transaction completion code. Contains 3TK or CWS codes. */
   protected String compCode;

   /** Reason code returned from  3TK or CWS (linked to compCode). */
   protected String reasonCode;

   /** The name of the failing function. */
   protected String errorFunctionName;

   /** Module in which the function failed. */
   protected String errorModuleName;

   /** For 3TK, value from the FI reflecting details of the error. */
   protected String senseData;

   /** Contains the length of the sense data string. */
   protected int senseDataLength;

   /** The self defining fields for the request being sent in the GMJI. */
   protected GsGmSelfDefiningFieldDef[] reqSelfDef = null;

   /**
   * The packed version of self defining fields for the request being
   * sent in the GMJI.
   */
   protected String reqSelfDef_packed = null;

   /** The self defining fields for the response being received in the GMJO. */
   protected GsGmSelfDefiningFieldDef[] respSelfDefData = null;

   // Contains the methods registered for this object.
   private static Hashtable registry = new Hashtable();

   /**
   * This flag indicates whether the transaction returns a response code.
   * It can have values of RETURNS_RESPONSE_CODE or NO_RESPONSE_CODE .
   */
   public int ContainsRespCode = RETURNS_RESPONSE_CODE;


   //-----------------------------------------------------------
   // Start of the code.
   //-----------------------------------------------------------

   /** Default constructor. */
   public IfsTxn()
   {
   }

   /** Constructor with IfsSession object. */
   public IfsTxn(IfsSession session)
   {
      super(session);
   }

   /**
   * Adds null argument to Gold Manager Java Input (GMJI) buffer.
   */
   public void addArgToGMJI()
   {
      addToGMJI(LW_NULL);
      gmjiArgCount++;
   }

   /**
   * Adds binary argument to Gold Manager Java Input (GMJI) buffer.
   */
   public void addArgToGMJI(byte[] buf)
   {
      if (buf == null)
         addArgToGMJI();

      else   // create string containing length and nibblized data.
         addArgToGMJI(nibblize(buf));

      return;
   }

   /**
   * Adds integer argument to Gold Manager Java Input (GMJI) buffer.
   */
   public void addArgToGMJI(int i)
   {
      addToGMJI(LW_INTEGER);
      addToGMJI(i);
      gmjiArgCount++;
   }

   /**
   * Adds String argument to Gold Manager Java Input (GMJI) buffer.
   */
   public void addArgToGMJI(String s)
   {
      if (s == null)
      {
         addToGMJI(LW_NULL);
      }
      else
      {
         addToGMJI(LW_STRING);
         addToGMJI(s);
      }
      gmjiArgCount++;
   }

   /**
   * Adds Gold message (transaction data) to Gold Manager Java Input (GMJI) buffer.
   * This method needs to be implemented by every gold transaction.
   */
   public void addGoldToGMJI(int argFlags, String cacheTxnName) {
   status = SUBCLASS_ERROR;
   return;
   }

   /**
   * Parses the self defining fields returned from the Gold Manager in the GMJO buffer.
   */
   protected void addSelfDefProperties()
   {
      int i, j, type, sdOwner, sdCount, repeatCounter;
      GsGmSelfDefiningFieldDef reqSD, respSD;

      if (reqSelfDef == null)
      {
         return;
      }

      sdOwner = getSD_OWNER();  // get the owner ID for this object

      sdCount = 0;

      // count number of self defining data items are needed for this object

      for (i=0; i < reqSelfDef.length; ++i)
      {
         reqSD = reqSelfDef[i];
         if (reqSD.getFlags() == SD_RESPONSE  &&    // is this a response self def item?
             reqSD.getOwner() == sdOwner)           // and is it for my object?
         {
            ++sdCount;
         }
      }

      if (sdCount == 0)       // if there are self def items for my object, return.
      {
         return;
      }

      // allocate array of GsGmSelfDefiningFieldDef objects to hold the response
      // self def data for my object.

      respSelfDefData = new GsGmSelfDefiningFieldDef[sdCount];

      sdCount = 0;

      for (i=0; i < reqSelfDef.length; ++i)
      {
         reqSD = reqSelfDef[i];
         if (reqSD.getFlags() == SD_RESPONSE  &&    // is this a response self def item?
             reqSD.getOwner() == sdOwner)           // and is it for my object?
         {
            respSelfDefData[sdCount] = new GsGmSelfDefiningFieldDef(); // create a new object
            // to hold the data
            respSD = respSelfDefData[sdCount];

            // copy needed fields from request object to response object

            respSD.setFlags(reqSD.getFlags());
            respSD.setTagID(reqSD.getTagID());
            respSD.setTagIDChain(reqSD.getTagIDChain());
            respSD.setOwner(reqSD.getOwner());
            respSD.setDataType(reqSD.getDataType());
            respSD.setLength(reqSD.getLength());
            respSD.setStructureInstanceCount(reqSD.getStructureInstanceCount());
            respSD.setPropertyName(reqSD.getPropertyName());
            repeatCounter = (int)getNextLongFromGMJO("self-def long");
            if (repeatCounter > 1)
            {
               //We have multiple items
               GsGmSelfDefiningFieldData[] multiData = new GsGmSelfDefiningFieldData[repeatCounter];
               for (j=0; j < repeatCounter; j++)
               {
                  multiData[j] = new GsGmSelfDefiningFieldData();
                  type = respSD.getDataType();         // get type of the output self def data

                  if (type == SD_TYPE_LONG)
                  {
                     multiData[j].setSdLong((int)getNextLongFromGMJO("self-def long"));
                  }
                  else if (type == SD_TYPE_DOUBLE)
                  {
                     multiData[j].setSdDouble(getNextDoubleFromGMJO("self-def double"));
                  }
                  else if (type == SD_TYPE_CHAR)
                  {
                     multiData[j].setSdChar(getNextCharFromGMJO(respSD.getLength(), "self-def char"));
                  }
                  else if (type == SD_TYPE_BINARY)
                  {
                     multiData[j].setSdBinary(getNextBinaryItemFromGMJO("self-def binary"));
                  }
                  else if (type == SD_TYPE_CURRENCY)
                  {
                     multiData[j].setSdCurrency(new GsCurrency(this,"self-def currency"));
                  }
                  else if (type == SD_TYPE_DATE)
                  {
                     multiData[j].setSdDate(new GsDate(this,"self-def date"));
                  }
                  else if (type == SD_TYPE_ACCOUNT)
                  {
                     multiData[j].setSdAccount(new GsAccount(this,"self-def account"));
                  }
                  else
                  {  //unknown type
                     if (isDebugEnabled())
                        debug("IfsTxn.addSelfDefProperties():  Unknown self defining datatype=" +
                              type + " at reqSelfDef array index=" + i + ".");
                  }
               }
               respSD.setMulti(multiData);

            }
            else
            {
               type = respSD.getDataType();         // get type of the output self def data

               if (type == SD_TYPE_LONG)
               {
                  respSD.setSdLong((int)getNextLongFromGMJO("self-def long"));
               }
               else if (type == SD_TYPE_DOUBLE)
               {
                  respSD.setSdDouble(getNextDoubleFromGMJO("self-def double"));
               }
               else if (type == SD_TYPE_CHAR)
               {
                  respSD.setSdChar(getNextCharFromGMJO(respSD.getLength(), "self-def char"));
               }
               else if (type == SD_TYPE_BINARY)
               {
                  respSD.setSdBinary(getNextBinaryItemFromGMJO("self-def binary"));
               }
               else if (type == SD_TYPE_CURRENCY)
               {
                  respSD.setSdCurrency(new GsCurrency(this,"self-def currency"));
               }
               else if (type == SD_TYPE_DATE)
               {
                  respSD.setSdDate(new GsDate(this,"self-def date"));
               }
               else if (type == SD_TYPE_ACCOUNT)
               {
                  respSD.setSdAccount(new GsAccount(this,"self-def account"));
               }
               else
               {  //unknown type
                  if (isDebugEnabled())
                     debug("IfsTxn.addSelfDefProperties():  Unknown self defining datatype=" +
                           type + " at reqSelfDef array index=" + i + ".");
               }
            }
            ++sdCount;
         }
      }
   }

   /**
   * Adds integer value to Gold Manager Java Input (GMJI) buffer.
   */
   protected void addToGMJI(int i)
   {
      if (gmji.length < gmjiCursor+4)
         allocateBiggerGMJI();

      gmji[gmjiCursor++] = (byte)((i&0xFF000000)>>24);
      gmji[gmjiCursor++] = (byte)((i&0x00FF0000)>>16);
      gmji[gmjiCursor++] = (byte)((i&0x0000FF00)>>8);
      gmji[gmjiCursor++] = (byte)( i&0x000000FF);
   }

   /**
   * Adds String object to Gold Manager Java Input (GMJI) buffer.
   */
   protected void addToGMJI(String str)
   {
      byte[] b;
      int i, len;
      try
      {
         // Get array of bytes containing the GMJ_IO_CHARSET characters for the input string.
         // Note the length of byte[] b is may not be the same as length of input string str
         // because the input string str is in unicode characters which don't map
         // one to one to bytes.
         b=str.getBytes(GMJ_IO_CHARSET);
      }
      catch (UnsupportedEncodingException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.addToGMJI(String): unsupportedEncodingException caught");
         b = new byte[1];   // create a 'null' one byte array to be put into
         b[0] = 0;          // the GMJI.
      }

      // put number of UTF bytes into GMJI
      len = b.length;
      //if (isDebugEnabled())
      //   debug("adding to GMJI, Offset:"+gmjiCursor+", Len:"+len+", Value="+str+".");
      addToGMJI(len+1);   // add 1 to account for null terminator

      debug("Adding " + len + " chars to GMJI, current cursor at " + gmjiCursor);

      // then put the character bytes into GMJI
      if (gmji.length < (gmjiCursor + len + 1) )   // if string does not fit in current GMJI
      {
         allocateBiggerGMJI();                     // allocate a bigger one
         if (gmji.length < (gmjiCursor + len + 1) ) // T1000102 if still to small allocate largest one
            allocateBiggerGMJI();
      }

      for (i=0; i<len; ++i)
      {
         gmji[gmjiCursor++] = b[i];
      }
      gmji[gmjiCursor++] = 0;  // null terminate the string
   }

   /**
   * Reallocates the Gold Manager Java Input (GMJI) buffer to the maximum size,
   * when the GMJI is less than the maximum size,
   */
   protected void allocateBiggerGMJI()
   {
      byte[] newGMJI;
      int i;

       /* T1000102 - added additional gmji size of 131072 (largestGMJISize),
                     GMJI starts with 4096 (initGMJISize),
                     incremenst to 65536 (biggerGMJISize),
                     and finally grows to 131072 (largestGMJISize) */ 

      if (gmji.length >= largestGMJISize)  // have we already made it as big as it can get?
         return;        // yes, simply return and let Java exception happen

      if (gmji.length >= biggerGMJISize)
      {
         newGMJI = new byte[largestGMJISize];
      }
      else
      {
         newGMJI = new byte[biggerGMJISize];
      }

      for (i=0; i<gmji.length; ++i)     // copy bytes from old GMJI to new
      {
         newGMJI[i] = gmji[i];
      }

      gmji = newGMJI;
      return;
   }

   /**
   * Converts a String object to a byte array and returns the byte array.
   *
   * @param in -- A String object which is expected to contain characters A to P
   *    which represent values 0 to 15.
   * @return
   *    A byte array which contains the binary value of <EM>in</EM> parameter and
   *    is half the length of <EM>in</EM>. Two sequential characters represent 1 byte.
   */
   public static byte[] denibblize(String in)
   {
      String code = "ABCDEFGHIJKLMNOP";
      String inString;
      byte[] outByte;
      int i, outSize, nibHi, nibLo;

      inString = in;

      outSize = inString.length()/2;

      if (inString.length() % 2 != 0)
      {
         outSize++;       // add one to size of output byte array
         inString = inString + "A";  // add a '0' nibble value to inString
      }

      outByte = new byte[outSize];

      for (i=0; i < outSize; i++)
      {
         nibHi = (code.indexOf(inString.charAt(i*2)))*16;
         if (nibHi < 0) nibHi = 0;  // if invalid char, set to 0

         nibLo = code.indexOf(inString.charAt(i*2+1));

         if (nibLo < 0) nibLo = 0;  // if invalid char, set to 0
         outByte[i] = (byte)(nibHi + nibLo);
      }
      return(outByte);
   }

   /**
   * Executes the transaction by sending it to the Financial Instition (FI) and
   * the response is not saved in the Gold Manager transaction cache.
   * The transaction execution mode is controlled by the executionMode attribute.
   *
   */
   public void execute() throws IfsException
   {
      execute(FI, null);  // Send txn to the FI and don't save in txn cache
   }

   /**
   * Executes the transaction and where it is to be sent for execution
   * is controlled by the argument flag ("argFlags") parameter.
   * The response is not saved in the Gold Manager transaction cache.
   * The transaction execution mode is controlled by the executionMode attribute.
   *
   * @param argFlags -- Controls were the transaction is sent for execution.
   *    It can contain values of IfsTxn.FI, IfsTxn.CACHE, or IfsTxn.CACHEFI.
   *
   */
   public void execute(int argFlags) throws IfsException
   {
      execute(argFlags, null);
   }

   /**
   * Executes the transaction and where it is to be sent for execution
   * is controlled by the argument flag ("argFlags") parameter.
   * The transaction execution mode is controlled by the executionMode attribute.
   *
   * @param argFlags -- Controls were the transaction is sent for execution.
   *    It can contain values of IfsTxn.FI, IfsTxn.CACHE, or IfsTxn.CACHEFI.
   * @param cacheTxnName -- Gold Manager transaction cache name.
   *
   */
   public void execute(int argFlags, String cacheTxnName) throws IfsException
   {
      if (isDebugEnabled())
      {
          debug("<BR>Entry to executeTxn.");
          debug("<BR>Executing transaction of name " + this.getClass().getName());
      }
      // check if executeTxn has previously been done, set it if not.
      // When returning DONE, the original execute output data and error
      // information is not changed. The original data can still be
      // obtained from the object.

      String msg;

      try
      {
         // Check the execution mode.
         if (executionMode == NOT_EXECUTE)
         {
            if (isDebugEnabled()) debug("<BR>ExecutionMode is set to NOT_EXECUTE.");
            status = NO_EXECUTION;
            return;
         }

         if (executionMode == EXECUTE_SMART && status == OK)
         {
            if (isDebugEnabled())
               debug("<BR>ExecutionMode is set to EXECUTE_SMART and the status is OK.");
            return;
         }

         // Perform validate action if required.
         if (isValidateActionRequired())
         {
            msg = validate();
            if (msg != null && !msg.equals("OK"))
            {
               status = VALIDATION_ERROR;
               message = msg;
               if (isDebugEnabled())
                  debug("<BR>Validation Failed. status = " + String.valueOf(status) +
                        " message = " + message );
               return;
            }
         }

         // build the GMJI request byte buffer
         if (isDebugEnabled()) debug("<BR>Before initGMJI().");
         initGMJI(ifsSession);

         // Initialize/reset "status"
         status = OK;

         // add the gold data to the GMJI
         if (isDebugEnabled()) debug("<BR>Calling addGoldToGMJI().");
         addGoldToGMJI(argFlags, cacheTxnName);

         //Check to see if there were any errors from adding Gold to GMJI.  This is not the preferred way to
         //check.  In v1r6 we will throw exceptions.
         if (!(NO_EXECUTION==status || OK==status))
            return;

         // Perform requestReadyAction action if required.
         if (isRequestReadyActionRequired())
         {
            msg = requestReadyAction();
            if (msg != null && !msg.equals("OK"))
            {
               if (isDebugEnabled()) debug("<BR>Request Ready Action Failed.");
               status = REQUEST_READY_ACTION_ERROR;
               message = msg;
               return;
            }
         }

         // call Gold Manager via JNI, the GMJI is input, a GMJO is returned.
         if (isDebugEnabled()) debug("<BR>Calling invokeGoldManage().");
         invokeGoldManager();

         // we now have a GMJO, examine for errors, extract fields, and put into Java attributes.
         if (isDebugEnabled()) debug("<BR>Calling setErrorStatus().");
         setErrorStatus();
         if (isDebugEnabled()) debug("<BR>After setErrorStatus().");

         // check if there is gold data to extract from the GMJO, if so, get it.
         if (isGoldData())
         {
            if (isDebugEnabled()) debug("<BR>Calling getGoldFromGMJO()");
            getGoldFromGMJO();
         }

         //Invoke SLAEvent to signify end of cwsapi
         markSLAEvent(GtSLAEvent.END_CWS_API);

         // Perform responseReadyAction action if required.
         if (isResponseReadyActionRequired())
         {
            msg = responseReadyAction();
            if (msg != null && !msg.equals("OK"))
            {
               if (isDebugEnabled())
                  debug("<BR>Response Ready Action Failed.");
               if (status == OK) status = RESPONSE_READY_ACTION_ERROR;
               message = message + "  responseReadyActionErrorMassage = " + msg;
            }
         }

         if (isDebugEnabled()) debug("<BR>Calling executeTxnCleanup().");
         executeTxnCleanup();

         if (isDebugEnabled()) debug("Exit from executeTxn.");
      }
      catch (Throwable e)
      {
         status = THROWABLE_ERROR;
         if (message == null)
            message = e.toString();

         else
            message = message + "\n\t" + e.toString();
      }
   }

   /**
   * Frees internal working objects (GMJI, GMJO) so they are not preserved
   * by the caller when the transaction object is saved in the servlet session cache.
   */
   protected void executeTxnCleanup()
   {
      gmji = null;
      gmjo = null;

   }

   /**
   * Sends a SLAEvent to the CWS Gold Manager.
   */
   public void markSLAEvent(int state) {
   debug("IfsTxn.markSLAEvent() - state is " + Integer.toString(state));
   GtSLAEvent sla = new GtSLAEvent(this.getIfsSession());
   sla.setState(state);
   sla.execute();
   }

   /**
   * Returns the business response code.
   */
   public String getBrc()
   {
      return brc;
   }

   /**
   * Gets a string of characters from the GMJO.
   *
   * @param len -- Number of bytes to get from the GMJO.
   * @param offset -- The offset in the GMJO were the data starts.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object of "len" length or less.  Trailing blanks are trimmed
   *    from the String.
   */
   protected String getCharFromGMJO(int len, int offset, String name)
   {
      String s;
      try
      {
         s = new String(gmjo, offset, len, GMJ_IO_CHARSET);
      }
      catch (UnsupportedEncodingException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.getCharFromGMJO():  Encoding exception at GMJO Offset "+ offset + ".");
         s = "UnsupportedEncodingException";
      }
      s =  IfsApi.trim(s, ' ');   // remove trailing blanks from the string

      if (isDebugEnabled())
         debug("Extracting char, Offset:"+offset+", Name:"+name+", Len:"+len+", Value="+s+".");

      return(s);
   }

   /**
   * Returns the completion code.
   */
   public String getCompCode()
   {
      return compCode;
   }

   /**
   * The getter method for the ContainsRespCode field.
   * This field indicates whether the transaction returns a response code.
   */
   public int getContainsRespCode() { return ContainsRespCode;}

   /**
   * Gets a Gold double value from the GMJO and returns a double value.
   *
   * @param offset -- The offset in the GMJO were the data starts.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    The double value.
   */
   protected double getDoubleFromGMJO(int offset, String name)
   {
      String s;
      Double d;
      try
      {
         s = new String(gmjo, offset, GMJO_SIZEOF_DOUBLE, GMJ_IO_CHARSET); // get varchar length from GMJO
      }
      catch (UnsupportedEncodingException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.getDoubleFromGMJO():  Encoding exception at GMJO offset " + offset + ".");
         s = "0";
         if (OK==status)
            status = FORMAT_ERROR;

         if (null==message)
            message = e.toString();

         else
            message = message + "\n" + e.toString();
      }
      d = new Double(s);   // convert string to Double object

      if (isDebugEnabled())
         debug("Extracting double, Offset:"+offset+", Name:"+name+", Value="+d+".");

      return(d.doubleValue());
   }

   /**
   * The getter method for the errorCode field.
   */
   public int getErrorCode()
   {
      return errorCode;
   }

   /**
   * The getter method for the errorFunctionName field.
   */
   public String getErrorFunctionName()
   {
      return errorFunctionName;
   }

   /**
   * The getter method for the errorModuleName field.
   */
   public String getErrorModuleName()
   {
      return errorModuleName;
   }

   /**
   * The getter method for the executionMode field.
   * It can have values of IfsTxn.NOT_EXECUTE, IfsTxn.EXECUTE, and IfsTxn.EXECUTE_SMART.
   */
   public    int getExecutionMode()
   {
      return executionMode;
   }

   /**
   * The getter method for the gmMessage (Gold Manager message) field.
   */
   public String getGmMessage()
   {
      return gmMessage;
   }

   /**
   * Returns the Gold Message request ID.
   * <P>
   * It is expected that every gold transaction (Gtxxxx) will implement this
   * method.  However, it is implemented here rather than being abstract to allow
   * non-gold transactions (such as ClientFetch) to extend IfsTxn and not have to
   * implement this method.
   */
   public String getGMRequestId() {return("IfsTxn");}

   /**
   * Gets the Gold message result by parsing the data returned from the Gold Manager
   * in the GMJO.
   * This method needs to be implemented by every gold transaction.
   */
   public void getGoldFromGMJO() {
   status = SUBCLASS_ERROR;
   return;
   }

   /**
   * Gets a Gold long value from the GMJO and returns a integer value.
   *
   * @param offset -- The offset in the GMJO were the data starts.
   * @param name -- Name of data.  Used for debug output identification
   * @return
   *    The integer value.
   */
   public int getLongFromGMJO(int offset, String name)
   {
      String s;
      Integer i;
      if (isDebugEnabled()) debug("Beginning of getLongFromGMJO");
      try
      {
         s = new String(gmjo, offset, GMJO_SIZEOF_LONG, GMJ_IO_CHARSET); // get 'int string' from GMJO
      }
      catch (UnsupportedEncodingException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.getLongFromGMJO():  Encoding exception at GMJO offset " + offset + ".");
         s = "0";
         if (OK==status)
            status = FORMAT_ERROR;

         if (null==message)
            message = e.toString();

         else
            message = message + "\n" + e.toString();
      }

      try
      {
         i = new Integer(s);   // convert string value to Integer object
      }
      catch (NumberFormatException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.getLongFromGMJO():  NumberFormatException decoding string '"+s+"' at offset "+offset+" in GMJO.");
         i = new Integer(0);
         if (OK==status)
            status = FORMAT_ERROR;

         if (null==message)
            message = e.toString();

         else
            message = message + "\n" + e.toString();
      }

      if (isDebugEnabled())
      {
          debug("Extracting int, Offset:"+offset+", Name:"+name+", Value="+i+".");
          debug("End of getLongFromGMJO");
      }
      return(i.intValue());
   }

   /**
   * The getter method for the message field.
   */
   public String getMessage()
   {
      return message;
   }

   /**
   * Converts BRC or Completion Code to text (Language/Locale dependent).
   *
   * @param bundleName -- Name of the resource bundle which contains the text.
   * @param codeToConvert -- The code which is to be converted to text.
   * @param return
   *    The String object which describes the code.
   */
   public String getMessageText (String bundleName, String codeToConvert)
   {
      String msg;
      try
      {
         if (isDebugEnabled())
            debug("<BR>getMessageText() Trying the try expression.<BR>");
         Locale locale = getIfsSession().getLocale();
         ResourceBundle messageBundle = ResourceBundle.getBundle(bundleName, locale);
         msg = messageBundle.getString(codeToConvert);
      }
      catch (Exception e)
      {
         if (isDebugEnabled())
            debug("<BR>getMessageText() - Developer, there was no data in bundle, so Unknown error type.<BR>");
         msg = "Undefined Resource Bundle Text Message for CODE : " + codeToConvert;
      }
      return msg;
   }

   /**
   * Gets the next binary item from the GMJO and returns a byte array.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A byte array which contains the binary item.
   */
   public byte[] getNextBinaryItemFromGMJO(String name)
   {
      // Binary items in the GMJO are encoded as a long containing the length of the
      // char encoded (nibblized) string following by the nibbilized string.
      // In otherwords, the nibbilized string is a variable length GMJO string.
      //
      //The length represents the overall length of the data.  When the binary data
      //was nibbilized, the size doubled.  Therefore, use twice the actual length to
      //obtain the characters, then denibbilize.
      //
      int len = 2 * getNextLongFromGMJO(name);

      return(denibblize(getNextFixLengthFromGMJO(len, name)));
   }

   /**
   * Gets the next fixed length character string from the GMJO and returns it.
   * <P>
   * When "len" value is greater than IfsApi.MAX_FIXED_STRING_LENGTH,
   * the character string is assumed to be a variable length string.
   *
   * @param len -- Length of the fixed length data.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   public String getNextCharFromGMJO(int len, String name)
   {
      String s;

      if (len > IfsApi.MAX_FIXED_STRING_LENGTH)
         s = getNlsVarCharFromGMJO(len, name);
      else
      {
         s = getCharFromGMJO(len, gmjoCursor, name);
         gmjoCursor += len;
      }

      return(s);
   }

   /**
   * Gets the next variable length character string from the GMJO.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   public String getNextCharItemFromGMJO(String name)
   {
      return(getNextVarCharFromGMJO(name));
   }

   /**
   * Gets the next 2 variable length strings from the GMJO and
   * creates a Cookie object from them.  The GMJO cookie representation
   * consists of a variable length GMJO string for the cookie name
   * followed by a variable length GMJO string for the cookie value.
   *
   * @param cookieName -- Cookie name for debug purposes.
   * @param cookieValue -- Cookie value name for debug purposes.
   * @return
   *    A Cookie object representing the name and value from the GMJO.
   */
   protected Cookie getNextCookieFromGMJO(String cookieName, String cookieValue)
   {
      if (isDebugEnabled()) debug("<DEBUG> in getNextCookieFromGMJO");

      String thisCookieName  = getNextVarCharFromGMJO(cookieName);
      if (isDebugEnabled()) debug("<DEBUG> after getting cookieName");

      String thisCookieValue = getNextVarCharFromGMJO(cookieValue);
      if (isDebugEnabled()) debug("<DEBUG> after getting cookieValue");

      return( new Cookie(thisCookieName, thisCookieValue));

   }

   /**
   * Gets the next Gold double value from the GMJO and returns it.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A double value.
   */
   public double getNextDoubleFromGMJO(String name)
   {
      double d;
      d = getDoubleFromGMJO(gmjoCursor, name);
      gmjoCursor = gmjoCursor + GMJO_SIZEOF_DOUBLE;
      return(d);
   }

   /**
   * Returns the next specified number of bytes from the GMJO as a String object.
   *
   * @param len -- The number of bytes to get from the GMJO.
   * @param name -- Name associated with the data for debug data identification.
   * @param result
   *    The "len" number of GMJO bytes are encoded to a String object using the
   *    IfsTxn.GMJ_IO_CHARSET character set and then returned.
   *    Trailing blanks are removed.
   */
   public String getNextFixLengthFromGMJO(int len, String name)
   {
      String s = getCharFromGMJO(len, gmjoCursor, name);
      gmjoCursor += len;

      return(s);
   }

   /**
   * Gets the next Gold long value from the GMJO and returns an integer.
   * The 'Long' in this name refers to the C data type in the Gold MTI structure.
   * These are converted to Java 'int' data type.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    An integer value.
   */
   public int getNextLongFromGMJO(String name)
   {
      int i;
      i = getLongFromGMJO(gmjoCursor,name);
      gmjoCursor = gmjoCursor + GMJO_SIZEOF_LONG;
      return(i);
   }

   /**
   * Gets the next null terminated character string from the GMJO.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   public String getNextNullCharFromGMJO(String name)
   {
      String s;
      int i, len;

      // Search from current gmjoCursor position until next null terminator byte.
      for (i = gmjoCursor; i < gmjo.length; ++i)
      {
         if (gmjo[i] == 0x00)
            break;
      }

      // Determine actual length of string and get data from GMJO.
      len = i - gmjoCursor;
      s = getCharFromGMJO(len, gmjoCursor, name);

      // Update cursor by adding length of string and one to skip over the null.
      gmjoCursor += (len + 1);
      return(s);
   }

   /**
   * Gets the next variable length character string from the GMJO.
   *
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   public String getNextVarCharFromGMJO(String name)
   {
      // Get the data length (12 decimal characters).
      int len = getNextLongFromGMJO(name);

      // Get and return the actual data.
      return(getNextFixLengthFromGMJO(len, name));
   }

   /**
   * Gets the next variable length NLS character string from GMJO and returns it.
   *
   * @param len -- Maximum expected length of the variable length data.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   private String getNlsVarCharFromGMJO(int len, String name)
   {
      String  s, hexstr;
      int     vlen = 0;
      try
      {
         // Get the data length (4 hex characters).
         hexstr = new String(gmjo, gmjoCursor, GMJO_VLEN_LENGTH, GMJ_IO_CHARSET);
         vlen = Integer.parseInt(hexstr,16);
         gmjoCursor += GMJO_VLEN_LENGTH;

         // Get the actual data.
         s = getCharFromGMJO(vlen, gmjoCursor, name);
         gmjoCursor += vlen;

         // Assure the returned string is not longer than maximum expected length.
         // if (vlen > len)
         if (s.length() > len)                   // T0979766 use truncated string size not vlen
            s = s.substring(0,len);
      }
      catch (UnsupportedEncodingException e)
      {
         if (isDebugEnabled())
            debug("IfsTxn.getNlsVarCharFromGMJO: " + name + ": " + e.getMessage());
         s = new String("");
      }

      return(s);
   }

   /**
   * Extracts an integer from the GMJO which is in the form of a 32 bit big
   * endian format.
   *
   * @param offset -- The offset within the GMJO where the data starts.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    An integer value.
   */
   public int getRealIntFromGMJO(int offset, String name)
   {
      int i;
      i = ( ((gmjo[offset]<<24)&0xFF000000) |
            ((gmjo[offset+1]<<16)&0x00FF0000) |
            ((gmjo[offset+2]<< 8)&0x0000FF00) |
            (gmjo[offset+3]&0x000000FF));

      if (isDebugEnabled())
         debug("Extracting real int, Offset:"+offset+", Name:"+name+", Value="+i+".");

      return(i);
   }

   /**
   * The getter method for the reasonCode field.
   * The reason code is returned from execution of Gold Message.
   */
   public String getReasonCode()
   {
      return reasonCode;
   }

   /**
   * The getter method for the registry hash table which
   * contains the methods registered for this object.
   */
   public static Hashtable getRegistry()
   {
      return registry;
   }

   /**
   * The getter method for the reqSelfDef (self defining) field
   * of the request being sent in the GMJI.
   */
   public  GsGmSelfDefiningFieldDef[] getReqSelfDef() {return(reqSelfDef);}

   /**
   * The getter method for the reqSelfDef_packed field.
   * It is the packed version of the self defining fields of the
   * request being sent in the GMJI.
   */
   public  String getReqSelfDef_packed() {return(reqSelfDef_packed);}

   /**
   * The getter method for the respSelfDefData field.
   * It is the self defining fields for the response being received in the GMJO.
   */
   public GsGmSelfDefiningFieldDef[] getRespSelfDefData() {return(respSelfDefData);}

   // It is expected that every gold transaction (Gtxxxx) will implement the following
   // method.  However, it is implemented here rather than being abstract to allow
   // non-gold transactions (such as ClientFetch) to extend IfsTxn and not have to
   // implement this method.
   /**
   * The getter method for the owner ID.
   */
   public int getSD_OWNER() {return(-1);}

   /**
   * The getter method for the senseData field which is returned in the GMJO.
   */
   public String getSenseData()
   {
      return senseData;
   }

   /**
   * The getter method for the senseDataLength field which contains
   * the length of the sense data returned in the GMJO.
   */
   public int getSenseDataLength()
   {
      return senseDataLength;
   }

   /**
   * The getter method for the status field which
   * status of the Gold message that was executed.
   */
   public int getStatus()
   {
      return status;
   }

   /**
   * Returns a variable length field from the GMJO.
   *
   * @param offset -- The offset within the GMJO where the data starts.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which contains the variable length field.
   */
   protected String getVarCharFromGMJO(int offset, String name)
   {
      int len;
      len = getLongFromGMJO(offset,name);
      return(getCharFromGMJO(len, offset + GMJO_SIZEOF_LONG, name));
   }

   /**
   * Native method for passing a change pin password transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaChangePinPasswordReq(byte[] gmji);

   /**
   * Native method for passing a logoff transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaLogoffReq(byte[] gmji);

   /**
   * Native method for passing a logon transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaLogonReq(byte[] gmji);

   /**
   * Native method for passing a reauthenication transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaReauthenticationReq(byte[] gmji);

   /**
   * Native method for passing a general transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaReq(byte[] gmji);

   /**
   * Native method for passing a self registration transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaSelfRegReq(byte[] gmji);

   /**
   * Native method for passing a sla transaction request to the Gold Manager.
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected native byte[] gmJavaSLAReq(byte[] gmji);

   /**
   * Creates the Gold Manager Java Input (GMJI) buffer and initializes the GMJI header.
   *
   * @param ses -- An IfsSession object which contains the HTTP request parameters
   *    HTTP response parameters, etc. for the session.
   */
   protected void initGMJI(IfsSession ses)
   {
      int i;
      Cookie[] cookies = null;
      Cookie cookie = null;
      HttpServletRequest httpServletRequest;
      HttpServletResponse httpServletResponse;

      httpServletRequest = ses.getHttpServletRequest();
      httpServletResponse = ses.getHttpServletResponse();

      gmji = new byte[initGMJISize];

      gmji[0]=(byte) 0x47; // ASCII/UTF8 'G'
      gmji[1]=(byte) 0x4D; // ASCII/UTF8 'M'
      gmji[2]=(byte) 0x4A; // ASCII/UTF8 'J'
      gmji[3]=(byte) 0x49; // ASCII/UTF8 'I'

      if (isDebugEnabled()) debug("<BR>DEBUG: Done with signature GMJI");

      gmjiCursor=4;
      addToGMJI(0);        // Add space for number of bytes actually used
                           //   in this GMJI. Will be filled in later.
      addToGMJI(0);        // Add space for number of arguments
      gmjiArgCount=0;

      if (isDebugEnabled())
         debug("<BR>DEBUG: Done putting two spaces for number of bytes and arguments");

      //------------------
      // add the URL string, IBANK cookie value string, and user's IP address as first
      // parameters in the GMJI argument list.
      //
      if (httpServletRequest != null)   // if being run as a servlet, then get the data
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: The httpServletReques is not null ");

         // Add the URL string to the GMJI
         if (httpServletRequest.getServletPath() == null ) {
               if (isDebugEnabled()) debug("<BR>DEBUG: ServletPath is null, adding ./unknown, as path in GMJI");
               addArgToGMJI("./unknown");
         }
         else {
              addArgToGMJI(httpServletRequest.getServletPath());
         }

         if (isDebugEnabled()) debug("<BR>DEBUG: Done putting the ServletPath");

         // Add the cookie
         // Get sessionKey (cookie) from the IfsSession object
         cookie = ses.getSessionKey();

         if (cookie == null || cookie.getValue() == null)
         {
            // The cookie is not in the ifsSession.
            // If the cookie is null means the customer is not using
            // the IfsSession and the GoldManager cookie should be
            // in the httpSevletRequest object.
            // So try to get the cookie from the HTTP request.

            if (isDebugEnabled()) debug("<BR>DEBUG: The cookie is null ");
            cookies = httpServletRequest.getCookies();
            if (isDebugEnabled()) debug("<BR>DEBUG: Done with getCookies()");

            // find the cws cookie and add it's value to the GMJI
            cookie = null;
            if (cookies != null)
            {
               for (i=0; i<cookies.length; ++i)
               {
                  if (isDebugEnabled()) debug("<BR>DEBUG: We got into the for loop");
                  if ( (cookies[i].getName()).equals(IfsSystem.cwsCookieName) )   // if found the cws cookie name
                  {
                     cookie = cookies[i];
                     break;    // exit from the search loop
                  }
               }
            }
         }
         if (cookie == null)
            addArgToGMJI(IfsSystem.cwsCookieName+"="+ "NoCookie");
         else
            addArgToGMJI(IfsSystem.cwsCookieName+"="+ cookie.getValue());

         // Add the caller's IP address to the GMJI
         
         // In some cases the remote address is not available from the
         // httpServletRequest. Here we first search the session for the
         // remote address. (see PTR 2000147).
         String remoteAddr = "";
         HttpSession httpSession = ses.getHttpSession();
         if ( httpSession != null ) {
            remoteAddr = (String) httpSession.getAttribute( IfsSystem.CLIENT_ADDR );
            if (remoteAddr == null) 
               remoteAddr = httpServletRequest.getRemoteAddr();
         } // if

         addArgToGMJI( remoteAddr );
         if (isDebugEnabled()) debug("<BR>DEBUG: Done with puting getRemoteAddr");

         // Add the caller's language and country codes to the GMJI
         addArgToGMJI(ses.getLocale().getLanguage());
         addArgToGMJI(ses.getLocale().getCountry());
         if (isDebugEnabled())
            debug("<BR>DEBUG: GMJI now contains language and country codes.");
      }
      else      // else not being run as a servlet, simply add nulls as values.
      {
         if (isDebugEnabled())
            debug("<BR>DEBUG: We got into the else case.  Something is wrong...");
         addArgToGMJI();       // URL
         addArgToGMJI();       // cookie
         addArgToGMJI();       // IP address
         addArgToGMJI();       // Language code
         addArgToGMJI();       // Country code
      }
   }

   /**
   * Executes the transaction by invoking the corresponding native Gold Manager method.
   *
   * @param gmji -- The Gold Manager Java Input transaction data buffer.
   * @return Gold Manager Java Output transaction response buffer.
   */
   protected byte[] invokeGmJavaReq(byte[] gmji)
   {
      if (logonTxn)
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: We are calling the gmJavaLogonReq");
         logonTxn = false;
         return gmJavaLogonReq(gmji);
      }
      else if (logoffTxn)
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: Calling gmJavaLogoffReq()");
         logoffTxn = false;
         return gmJavaLogoffReq(gmji);
      }
      else if (selfRegTxn)
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: Calling gmJavaSelfRegReq()");
         selfRegTxn = false;
         return gmJavaSelfRegReq(gmji);
      }
      else if (changePinPasswordTxn)
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: Calling gmJavaChangePinPasswordReq()");
         changePinPasswordTxn = false;
         return gmJavaChangePinPasswordReq(gmji);
      }
      else if (reauthTxn)
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: Calling gmJavaReauthenticationReq()");
         reauthTxn = false;
         return gmJavaReauthenticationReq(gmji);
      }
      else if (slaTxn)
      {
         debug("<BR>DEBUG: Calling gmJavaSLAReq()");
         slaTxn = false;
         return gmJavaSLAReq(gmji);
      }
      else if (oboLogonTxn)
      {
         if (isDebugEnabled())
            debug("<BR>DEBUG: We are calling the gmJavaLogonReq for OBO Logon");
         oboLogonTxn = false;
         return gmJavaLogonReq(gmji);
      }
      else
      {
         if (isDebugEnabled()) debug("<BR>DEBUG: We are calling the gmJavaReq");
         return gmJavaReq(gmji);
      }
   }

   /**
   * Inserts the GMJI length and number of arguments into the GMJI header
   * and then executes the transaction by passing the GMJI to the Gold Manager.
   * The GMJO receives the transaction response.
   */
   protected void invokeGoldManager()
   {
      int saveCursor, retCode;

      // Put the number of bytes used in the GMJI in the GMJI.
      // This is different than gmji.length because we allocate
      // gmji byte buffer space in chunks.

      saveCursor = gmjiCursor;
      gmjiCursor = 4;            // put total amount of GMJI used into buffer
      addToGMJI(saveCursor);

      gmjiCursor = 8;
      addToGMJI(gmjiArgCount);  // put number of arguments into the GMJI
      gmjiCursor = saveCursor;  // restore the cursor position

      if (isDebugEnabled()) debug(toStringFromGMJI());

      gmjo=null;
      gmjoCursor=0;

      if (IfsSystem.STRINGS_PATH == null)   // should 'strings' be used?
      {
         // no, call gold manager
         if (isDebugEnabled())
         {
             debug("\nIfsTxn:invokeGoldManager calling gmJavaReq native interface.$$$");
             debug("\n LogoffTxn = " + logoffTxn);
         }
         gmjo = invokeGmJavaReq(gmji);
         if (isDebugEnabled())
            debug("\nIfsTxn:invokeGoldManager return from gmJavaReq native interface.");
      }
      else     // get gmjo data from 'strings'
      {
         File file;
         FileInputStream fileS;
         String fileName;

         fileName = IfsSystem.STRINGS_PATH + "!" + getGMRequestId() + ".str";

         file = new File(fileName);

         if (file.canRead())
         {
            try
            {
               fileS = new FileInputStream(file); // get FileInputStream for the file

               gmjo = new byte[(int)file.length()];    // allocate buffer size of file
               fileS.read(gmjo);                  // read all bytes from file into buffer
               fileS.close();
            }
            catch (Exception e)
            {
               if (isDebugEnabled())
                  debug("IfsTxn.involeGoldManager(): Exception accessing string file.");
            }

         }
         else
         {
            if (isDebugEnabled())
               debug("IfsTxn.invokeGoldManager(): Unable to read string file '" + fileName + "'.");
            try
            {
               String err = "GMOBCWS!999999999999000000000000IfsTxn  IfsTxn  UNABLE TO OPEN STRING FILE.\0";
               gmjo = err.getBytes(GMJ_IO_CHARSET);
            }
            catch (UnsupportedEncodingException e)
            {
               if (isDebugEnabled())
                  debug("IfsTxn.invokeGoldManager(): unsupportedEncodingException caught");
               gmjo = null;  // let exception happen
            }
         }
      }  // end of strings support

      if (isDebugEnabled()) debug(toStringFromGMJO());
   }

   /**
   * Returns true when the GMJO has Gold data that needs to be parsed,
   * false otherwise.
   */
   public boolean isGoldData() {return(goldData);}

   /**
   * Returns the boolean value of requestReadyActionRequired field.
   */
   public boolean isRequestReadyActionRequired()
   {
      return requestReadyActionRequired;
   }

   /**
   * Returns the boolean value of responseReadyActionRequired field.
   */
   public boolean isResponseReadyActionRequired()
   {
      return responseReadyActionRequired;
   }

   /**
   * Returns the boolean value of validateActionRequired field.
   */
   public boolean isValidateActionRequired()
   {
      return validateActionRequired;
   }

   /**
   * Encodes a byte array into a String object and returns the String object.
   * Each byte results in 2 characters in the output.
   */
   public static String nibblize(byte[] byteArray)
   {
      String code = "ABCDEFGHIJKLMNOP";
      int i, nibHi, nibLo;

      // Changed from String objects to StringBuffer objects for performance reasons.
      StringBuffer outString = new StringBuffer(byteArray.length);

      for (i=0; i<byteArray.length; i++)
      {
         // note:  the following ANDing is needed to avoid problems with
         // java treating bytes as signed.  The ANDing causes them
         // to be treated as unsigned (which is what we need).
         nibHi =  (0x000000F0 & byteArray[i]) / 16;
         nibLo =  0x0000000F & byteArray[i];
         outString = outString.append(code.charAt(nibHi));
         outString = outString.append(code.charAt(nibLo));
      }
      return(outString.toString());
   }

   /**
   * Customizable Txn method which processes bank specific logic after all the
   * transaction request parameters are set and before the transaction is executed.
   * It returns "OK" if the action is successful.  Otherwise, it returns a message string
   * indicating the error.
   * If a requestReadyAction method is not registered, a null is returned..
   */
   public String requestReadyAction()
   {
      String msg = "";
      try
      {
         // Invoke requestReadyAction method in the registries whichever is found first
         // from the concrete transaction class up to the IfsTxn class.
         msg = (String)invokeMethod("requestReadyAction", new Object[]{this}, "IfsTxn");
      }
      catch (Exception e)
      {
         msg = null;
         /*
           if (e.getMessage().equals("Method not registered."))
                msg = null;
           else
                msg = e.getMessage();
         */
      }

      return msg;
   }

   /**
   * Customizable Txn method which processes bank specific logic after the transaction is
   * executed and all the transaction response objects are set.
   * It returns "OK" if the action is successful.  Otherwise, it returns a message string
   * indicating the error.
   * If a responseReadyAction method is not registered, a null is returned.
   */
   public String responseReadyAction()
   {
      String msg = "";
      try
      {
         // Invoke responseReadyAction method in the registries whichever is found first
         // from the concrete transaction class up to the IfsTxn class.
         msg = (String)invokeMethod("responseReadyAction", new Object[]{this}, "IfsTxn");
      }
      catch (Exception e)
      {
         msg = null;

         /*
         if (e.getMessage().equals("Method not registered."))
              msg = null;
         else
              msg = e.getMessage();
         */
      }
      return msg;
   }
   public void setContainsRespCode(int responseCodeFlag) { ContainsRespCode = responseCodeFlag;}
   public void setDefaultFlags(int     executionMode,
                               boolean validateActionRequiredFlag,
                               boolean requestReadyActionRequiredFlag,
                               boolean responseReadyActionRequiredFlag)
   {
      this.executionMode = executionMode;
      this.validateActionRequired = validateActionRequiredFlag;
      this.requestReadyActionRequired = requestReadyActionRequiredFlag;
      this.responseReadyActionRequired = responseReadyActionRequiredFlag;
   }

   /**
   * Parses Gold Manager Output Buffer and sets associated status values.
   * For example, error code, completion code, reason code, message, sense data, etc. .
   */
   private void setErrorStatus()
   {

      String s;
      String tempStatus;
      int wsp_ok_logon = -1;

      // Initialize "results" data

      errorCode = 0;
      compCode = null;
      reasonCode = null;
      errorFunctionName = null;
      errorModuleName = null;
      gmMessage = null;
      message = null;
      senseData = null;
      senseDataLength = 0;

      goldData = false;  // Assume there is no gold data to parse initially
      if (isDebugEnabled()) debug("<BR>In IfsTxn setErrorStatus()");
      // verify the response has a valid GMOB string tag at front

      s = getCharFromGMJO(GMOB_ACRO_OFFSET_LEN, GMOB_ACRO_OFFSET, "GMOB_Acronym");
      if (!(s.equals("GMOB")))    // something bad happened
      {
         if (isDebugEnabled()) debug("<BR>setErrorStatus() Bad GMOB acronym");

         errorCode = TXN_CWS;
         status = FATAL_ERROR;
         compCode = ERROR_BAD_GMJO_ACRONYM;
         reasonCode = "0";
         errorFunctionName = "UNKNOWN";
         errorModuleName = "IfsTxn";
         gmMessage = getMessageText("com.ibm.ifs.resource.IfsCwsErrorTextBundle", "BADGMOB");
         return;
      }

      // The Gold Message has valid "GMOB" acronym, get next 4 bytes to determine if
      // if this is an OK messare or a gold manager error code

      s = getCharFromGMJO(GMOB_CODE_OFFSET_LEN, GMOB_CODE_OFFSET, "GMOB_Return_Code");
      gmjoCursor=8;
      if (s.equals(GM_TXN_OK))
      {
         if (isDebugEnabled()) debug("<BR>setErrorStatus() BRC");
         brc = getNextFixLengthFromGMJO(3,"brc");
         if (this.getContainsRespCode()==0) {
            brc = "000";
         }
         else  {  /* T0962113 */
             if( brc.length() == 0 || brc.equals("   ") ) {
                 brc = "000";
             }
         }

         for (int i=0;i<IfsSystem.WSP_OK_LOGON_BRC.length;i++)
         {
            if (brc.equals(IfsSystem.WSP_OK_LOGON_BRC[i])) wsp_ok_logon = i;
         }
         if (wsp_ok_logon > -1)
         {
            status = OK;
         }
         else
         {
            errorCode = TXN_BRC;
            if (isDebugEnabled())
               debug("<BR>setErrorStatus() before getting brc tempStatus  Brc== " + brc);
            tempStatus = (String)IfsSystem.allBrcTypesDictionary.get(brc);
            if (tempStatus == null)
            {
               if (isDebugEnabled())
                  debug("<BR>Developer, this BRC not in hashtable, setting status to FATAL_ERROR!<BR>");
               tempStatus = "FATAL";
            }

            if (isDebugEnabled()) debug("<BR>setErrorStatus() before getting brc message");
            message = getMessageText("com.ibm.ifs.resource.IfsBrcErrorTextBundle", brc);
            if (isDebugEnabled()) debug("<BR>setErrorStatus() after getting brc message: " + message);
            setStatus(tempStatus);
         }
         // "gmjoCursor" points to the first character after the "GMOB" and
         // the 4 byte error code.  Subsequent characters will be parsed appropriately.
         gmjoCursor=GMOB_ACRO_OFFSET_LEN + GMOB_CODE_OFFSET_LEN;
         goldData = true;
         return;
      }

      // This was a bad transaction, so parse it accordingly, otherwise the GMOB did not
      // contain anything.  The variable "gmjoCursor" should be set to a value placing it
      // after the "GMOB" and any "GMOB return code".  Any characters after this should
      // be valid data to parse for normal transactions.

      gmjoCursor = GMOB_ACRO_OFFSET_LEN + GMOB_CODE_OFFSET_LEN;

      if (s.equals(GM_TXN_3_TOKEN))
      {
         if (isDebugEnabled())
            debug("<BR>I am in the setErrorStatus() Three Token IF case");

         compCode =  "" + getNextLongFromGMJO("compCode");  // get 3 token completion code from GMJO
         reasonCode = "" + getNextLongFromGMJO("reasonCode");          // get 3 token reason code from GMJO
         senseData = getNextVarCharFromGMJO("senseData");         // get 3 token sense data from GMJO
         senseDataLength = senseData.length();
         processCCErrors();
         return;
      }

      if (s.equals(GM_TXN_CWS))
      {
         if (isDebugEnabled())
         {
            debug("<BR>s = " + s);
            debug("<BR>setErrorStatus() CWS ");
         }
         errorCode = TXN_CWS;
         compCode = "" + getNextLongFromGMJO("compCode");
         tempStatus = (String)IfsSystem.allCwsTypesDictionary.get(compCode);
         if (tempStatus == null)
         {
            if (isDebugEnabled())
               debug("<BR>Developer, this CWS error not in hashtable, setting status to FATAL_ERROR!<BR>");
            status = FATAL_ERROR;
         }
         else
         {
            setStatus(tempStatus);
         }
         message = getMessageText("com.ibm.ifs.resource.IfsCwsErrorTextBundle", compCode);
         reasonCode = "" + getNextLongFromGMJO("reasonCode");
         errorFunctionName = getNextFixLengthFromGMJO(8,"errorFunctionName");
         errorModuleName = getNextFixLengthFromGMJO(8,"errorModuleName");
         if (isDebugEnabled()) debug("<BR>setErrorStatus() CWS Before set gmMessage ");

         gmMessage = getNextNullCharFromGMJO("gmMessage");            // note null terminated string support
         if (isDebugEnabled()) debug("<BR>setErrorStatus() CWS After set gmMessage ");
         return;
      }

      // Not a known valid GMOB type.  Return CWS error code and information.
      if (isDebugEnabled()) debug("<BR>setErrorStatus() Unknown error type ");

      errorCode = TXN_CWS;
      status = FATAL_ERROR;
      compCode = ERROR_BAD_GMJO_ERROR_CODE;
      reasonCode = "0";
      errorFunctionName = "UNKNOWN";
      errorModuleName = "IfsTxn";
      gmMessage = getMessageText("com.ibm.ifs.resource.IfsCwsErrorTextBundle", "NOGMOB");
      if (isDebugEnabled()) debug("<BR>setErrorStatus() Unknown error type before return");
      return;
   }

   /**
   * Handles processing of Core Controller Errors.
   * <P>.Core Controller Error types of warning, application error, or an internal error, return Business Response Codes(BRC).  The BRC
   * value can be retrieved with the getBrc() method and any text associated with the BRC can be retrieved via the getMessage() method.
   * <P>.If the BRC value is successful, the transaction status is set to valid and the returning data is parsed by the transaction object.
   * <P>.For unsuccessful BRC values or any other Core Controller Error type, the transaction status will be set to either a fatal,
   * information, or redo state and the returning data is not parsed by the transaction object.  The getCompCode()
   * method returns the specific Core Controller Error, and the getGmMessage() method returns any text associated with this error.
   *
   * Ticket T0962907 - HSBC requested change to use reasonCode instead of compCode when looking up error message.
   */
   public void processCCErrors()
   {
      String tempStatus = null;

      if (compCode.equals(IFSCC_WARNING) || compCode.equals(IFSCC_APPLICATION) ||
          compCode.equals(IFSCC_INTERNAL))
      {
         brc = getNextFixLengthFromGMJO(3,"brc");
         message = getMessageText("com.ibm.ifs.resource.IfsBrcErrorTextBundle", brc);
         if (isDebugEnabled())
            debug("<BR>Three Token BRC Error Message = " + message + "<BR>");
         if (isBRCValid())
         {
            status = OK;
            goldData = true;
         }
         else
         {
            errorCode = TXN_3_TOKEN;
            gmMessage = getMessageText("com.ibm.ifs.resource.Ifs3tkErrorTextBundle", reasonCode); /* T0962907 */
            if (isDebugEnabled())
            {
               debug("<BR>Three Token Error Message = " + gmMessage + "<BR>");
               debug("<BR>setErrorStatus() before getting 3TK brc tempStatus  Brc== " + brc);
            }
            tempStatus = (String)IfsSystem.allBrcTypesDictionary.get(brc);
            if (tempStatus == null)
            {
               if (isDebugEnabled())
                  debug("<BR>Developer, this BRC not in hashtable, setting status to FATAL_ERROR!<BR>");
               status = FATAL_ERROR;
            }
            else
               setStatus (tempStatus);
         }
      }
      else
      {
         errorCode = TXN_3_TOKEN;
         gmMessage = getMessageText("com.ibm.ifs.resource.Ifs3tkErrorTextBundle", reasonCode); /* T0962907 */
         if (isDebugEnabled())
            debug("<BR>Three Token Error Message = " + gmMessage + "<BR>");
         tempStatus = (String)IfsSystem.all3tkTypesDictionary.get(reasonCode); /* T0962907 */

         if (tempStatus == null)
         {
            if (isDebugEnabled())
               debug("<BR>Developer, this 3TK error not in hashtable, setting status to FATAL_ERROR!<BR>");
            status = FATAL_ERROR;
         }
         else
            setStatus(tempStatus);
      }
      return;
   }

   /**
   * Process any special BRC handling for transactions.
   */
   public boolean isBRCValid()
   {
      return (getBrc().equals(VALIDBRC));
   }

   /**
   * Sets the execution mode for this transaction.
   *
   * @param mode -- Valid values for the execution mode are
   *    NOT_EXECUTE, EXECUTE, and EXECUTE_SMART.
   */
   public    void setExecutionMode(int mode)
   {
      executionMode = mode;
   }

   /**
   * The setter method for reqSelfDef field.
   * These are self defining fields for the request being sent in the GMJI.
   */
   public  void setReqSelfDef(GsGmSelfDefiningFieldDef[] o) {reqSelfDef = o;}

   /**
   * The setter method for reqSelfDef_packed field.
   * These are packed self defining fields for the request being sent in the GMJI.
   */
   public  void   setReqSelfDef_packed( String o ) {reqSelfDef_packed = o;}

   /**
   * The setter method for requestReadyActionRequired field.
   *
   * @param b -- It should be true when the method registered to
   *    <EM>requestReadyAction</EM> is to be called to process bank specific logic.
   *    When set to true, this call is made after the GMJI has been created and
   *    before the transaction is sent to the Gold Manager for execution.
   */
   public void setRequestReadyActionRequired(boolean b)
   {
      requestReadyActionRequired = b;
   }

   /**
   * The setter method for responseReadyActionRequired field.
   *
   * @param b -- It should be true when the method registered to
   *    <EM>responseReadyAction</EM> is to be called to process bank specific logic.
   *    When set to true, this call is made after the GMJO has been received from
   *    the Gold Manager and the GMJO has been parsed into the java objects.
   */
   public    void    setResponseReadyActionRequired(boolean b)
   {
      responseReadyActionRequired = b;
   }

   /**
   * The setter method for the status field.
   *
   * @param tmpStatus -- This parameter can be "REDO", "INFO", or "FATAL".
   *    All other values are treated as invalid error status.
   */
   public int setStatus(String tmpStatus)
   {
      if (tmpStatus.equals("REDO"))
         status = REDO_ERROR;
      else
      {
         if (tmpStatus.equals("INFO"))
            status = INFO_ERROR;
         else
         {
            if (tmpStatus.equals("FATAL"))
               status = FATAL_ERROR;
            else
               status = INVALID_ERROR;
         }
      }
      return status;
   }

   /**
   * The setter method for validateActionRequired field.
   *
   * @param b -- It should be true when true the transaction input data is
   *    to be validated via registered methods
   *    or default methods (when no validation methods are registered).
   */
   public    void    setValidateActionRequired(boolean b)
   {
      validateActionRequired = b;
   }

   /**
   * Converts the IfsTxn object to a localized string representation.
   * When a method is registered to <EM>toString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @return
   *    Returns a String representation of this object.
   */
   public String toString()
   {
      if (hasRegisteredMethod("toString"))
      {
         return super.toString();
      }
      else
         return toString("\n");
   }

   /**
   * Converts the IfsTxn object to a localized string representation.
   * When a method is registered to <EM>toStringWithString</EM> for this class,
   * it is used.  Otherwise, a default conversion to string is done.
   *
   * @param prefix -- The String object which is appended to the beginning of the
   *    returned string.
   * @return
   *    Returns a String representation of this object.
   */
   public String toString(String prefix)
   {
      if (hasRegisteredMethod("toStringWithString"))
      {
         return super.toString(prefix);
      }
      else
      {
         StringBuffer buffer = new StringBuffer(256);
         // buffer.append(super.toString(prefix));

         buffer.append(prefix);
         buffer.append("gmjiArgCount = ");
         buffer.append(gmjiArgCount);

         buffer.append(prefix);
         buffer.append("gmjiCursor = ");
         buffer.append(gmjiCursor);

         buffer.append(prefix);
         buffer.append("gmjoCursor = ");
         buffer.append(gmjoCursor);

         buffer.append(prefix);
         buffer.append("validateActionRequired = ");
         buffer.append(validateActionRequired);

         buffer.append(prefix);
         buffer.append("requestReadyActionRequired = ");
         buffer.append(requestReadyActionRequired);

         buffer.append(prefix);
         buffer.append("responseReadyActionRequired = ");
         buffer.append(responseReadyActionRequired);

         buffer.append(prefix);
         buffer.append("executionMode = ");
         buffer.append(executionMode);

         buffer.append(prefix);
         buffer.append("status = ");
         buffer.append(status);

         buffer.append(prefix);
         buffer.append("errorCode = ");
         buffer.append(errorCode);

         buffer.append(prefix);
         buffer.append("brc = ");
         buffer.append(brc);

         buffer.append(prefix);
         buffer.append("message = ");
         buffer.append(message);

         buffer.append(prefix);
         buffer.append("gmMessage = ");
         buffer.append(gmMessage);

         buffer.append(prefix);
         buffer.append("compCode = ");
         buffer.append(compCode);

         buffer.append(prefix);
         buffer.append("reasonCode = ");
         buffer.append(reasonCode);

         buffer.append(prefix);
         buffer.append("errorFunctionName = ");
         buffer.append(errorFunctionName);

         buffer.append(prefix);
         buffer.append("errorModuleName = ");
         buffer.append(errorModuleName);

         buffer.append(prefix);
         buffer.append("senseData = ");
         buffer.append(senseData);

         buffer.append(prefix);
         buffer.append("senseDataLength = ");
         buffer.append(senseDataLength);

         return buffer.toString();
      }
   }

   /**
   * Returns a formatted string of the GMJI when debug is turned ON.
   * Returns a null when debug is turned OFF.
   */
   public String toStringFromGMJI()
   {
      if (!isDebugEnabled()) return null;

      int i, totalBytes;                            // T1000102

      String str = "******** GMJI ********\n";
      if (gmji==null)
      {
         str += "The GMJI reference is null.\n";
      }
      else
      {
         /* T1000102 - larger gmji size causing toString to become painfully slow so breaking
                       call to toString up into toStringIncr (4096) chuncks */

         str += "Size of gmji buffer is "+ gmji.length + " bytes.\n" +
                "There are " + gmjiCursor + " bytes used in the buffer.\n"; 

         totalBytes = gmjiCursor;
         for(i = 0; i<gmjiCursor; i += toStringIncr)
         {
            if (totalBytes > toStringIncr)
            {
               str += IfsApi.toString(gmji, i, toStringIncr);
               totalBytes -= toStringIncr;
            }
            else
              str += IfsApi.toString(gmji, i, totalBytes);
         }
         str += "******** END OF GMJI ********\n";
      }
      return str;
   }

   /**
   * Returns a formatted string of the GMJO when debug is turned ON.
   * Returns a null when debug is turned OFF.
   */
   protected String toStringFromGMJO()
   {
      if (!isDebugEnabled()) return null;

      int i, totalBytes;                        // T1000102

      String str = "******** GMJO ********\n";
      if (gmjo==null)
      {
         str += "The GMJO reference is null.\n";
      }
      else
      {
          /* T1000102 - larger gmji size causing toString to become painfully slow so breaking
                       call to toString up into toStringIncr (4096) chuncks - applying same
                       code change to handle large gmjo buffers */

         str += "Size of gmjo buffer is "+ gmjo.length + " bytes.\n"; 

         totalBytes = gmjo.length;
         for(i = 0; i<gmjo.length; i += toStringIncr)
         {
            if (totalBytes > toStringIncr)
            {
               str += IfsApi.toString(gmjo, i, toStringIncr); 
               totalBytes -= toStringIncr;
            }
            else
              str += IfsApi.toString(gmjo, i, totalBytes);
         }
         str += "******** END OF GMJO ********\n";
      }
      return str;
   }

   /**
   * Gets the next fixed length character string (NLS sensitive field)
   * from the GMJO and returns it.
   *
   * @param len -- Length of the fixed length data. Since this is a NLS sensitive
   *    field, the maximum length for this field is 3 times this value.
   * @param name -- Name of data.  Used for debug output identification
   * @param return
   *    A String object which has its trailing blank characters removed.
   */
   public String getNextNlsCharFromGMJO(int len, String name)
   {
      if (isDebugEnabled())
         debug("This is a NLS field. We entered getNextNlsCharFromGMJO");

      return( getNlsVarCharFromGMJO(len * 3, name));  // NLS fields are 3 times longer.
   }


   // larockc -varying length encoding for strings
   // The pack() methods are used for repeat items

   /**
   * Creates a packed variable length String object.
   *
   * @param s -- String object which is to be packed as a varying length field.
   * @param width -- Maximum length of the string data.
   * @param return
   *    A variable length String object which has a 4 character hex length
   *    followed by the string data.
   */
   public static String packVarChar(String s, int width)
   {
      if (s == null)
         s = new String("");

      int len = s.length();
      if (len > width)
         len = width;    // Truncate if longer than field length

      if (len > GMJO_MAX_VARCHAR_LENGTH)
         len = GMJO_MAX_VARCHAR_LENGTH;

      StringBuffer packed = new StringBuffer(len + GMJO_VLEN_LENGTH);

      // Insert length (hex padded with leading zeros) in buffer
      String hexlen = Integer.toHexString(len);
      int hlen = hexlen.length();
      while(hlen < GMJO_VLEN_LENGTH)
      {
         packed.append('0');
         hlen++;
      }
      packed.append(hexlen);

      // Append string, truncated to max length
      packed.append(s.substring(0,len));

      return packed.toString();
   }

   /**
   * Creates a packed fixed or variable length String object.
   *
   * When the field width is less than or equal to IfsApi.MAX_FIXED_STRING_LENGTH,
   * the resulting packed string is a fixed-length field, left-justified, and
   * padded with blanks.  Otherwise, it is encoded as variable length.
   *
   * @param s -- String object which is to be packed.
   * @param width -- Maximum length of the string data.
   * @param return
   *    A packed fixed or variable length String object.
   */
   public static String packChar(String s, int width)
   {
      if (s == null)
         s = new String("");

      if (width > IfsApi.MAX_FIXED_STRING_LENGTH)
         return packVarChar(s,width);    // Make it a varying length string.

      int len = s.length();
      if (len > width)
         return s.substring(0,width);    // Truncate to specified width

      // Pad the string with blanks
      StringBuffer packed = new StringBuffer(width);
      packed.append(s);

      while(len < width)
      {
         packed.append(' ');
         len++;
      }

      return packed.toString();
   }

   // UTF-8 strings are actually 3* longer than the specified length
   public static String packNlsChar(String s, int width)
   {
      if (s == null)
         s = new String("");

      if (s.length() > width)
         return packVarChar(s.substring(0,width), width * 3); // Truncate to specified width
      else
         return packVarChar(s, width * 3);
   }

   /**
   * Creates a fixed length String object from a Gold LONG (integer) value.
   *
   * @param v -- The Gold LONG value.
   * @param return
   *    A String object which is GMJO_SIZEOF_LONG characters in length.  Leading
   *    zeros are padded to the string to make it GMJO_SIZEOF_LONG characters long.
   */
   public static String packLong(int v)
   {
      String vstr = Integer.toString(Math.abs(v));
      StringBuffer packed = new StringBuffer(GMJO_SIZEOF_LONG);

      // Append leading zeros to make a total length of GMJO_SIZEOF_LONG.
      int len = vstr.length();
      while (len < GMJO_SIZEOF_LONG)
      {
         packed.append('0');
         len++;
      }
      packed.append(vstr);

      return packed.toString();
   }

} // end-of, IfsTxn class

