// ***************************************************************************
//  FILE NAME:   IfsConnection.java
//
//  COPYRIGHT:
//
//    Licensed Materials - Property of IBM
//    5648-A06
//    (C) Copyright IBM Corp. 1998, 2002 All Rights Reserved
//    US Government Users Restricted Rights - Use, duplication
//    or disclosure restricted by GSA ADP Schedule Contract
//    with IBM
//
//  CMVC Info:
//
//     timestamp:     02/08/27-18:56:22
//     cmvc info:     src/cws/jcwsapi/com/ibm/ifs/servlets/IfsConnection.java, cws.jcwsapi, cws.v1r6m1.mmp, v1r6m1_020828
//     revision:      1.3
//     sccs filename: /cmvc/cmvccws/vc/0/0/0/4/s.85
//
//  CHANGES:
//    10/26/2001  nc/dob  d7359    Created
//    11/07/2001  dob     d7365    Improve exception handling. Implement initConnection.
//    02/04/2002  bjb     d7427    Load IFS native library using absolute path
//    07/23/2002  pdl     T2000176 split config file
//    08/27/2002  pdl     T2000187 deduce bin directory from environment config file.
// ***************************************************************************

package com.ibm.ifs.servlets;

import java.io.*;
import javax.servlet.*;
import com.ibm.ifs.gold.IfsException;
import com.ibm.ifs.gold.IfsSystem;

/** 
 *  IfsConnection.java
 *  <p>
 *  IfsConnection is a class that will manage the initialization
 *  and termination of the connection to the back end process.
 *  It is a singleton class (i.e., It will permit only one instance
 *  of itself to be instantiated).
 *  <p>
 *  When it is instantiated. It will invoke a JNI method, initConnection()
 *  implemented in msgclient.c to initialize communication to the back end.
 *  The name of the configuration file, obtained from java system property 
 *  ifs.config.file will be passed as a parameter to initConnection().
 *  <p>
 *  IfsConnection will register a shutdown hook so that the back end
 *  can be commanded to terminate when the WebSphere JVM terminates.
 *  The shutdown hook will call JNI method dropConnection, which is 
 *  also implemented in msgclient.c, will issue an IPC to the back end
 *  with action code ACTION_SHUTDOWN which will cause the back end to
 *  terminate.
*/
public class IfsConnection implements Runnable {

/* ----------------------------- Class Variables ---------------------------- */
   
  /**  Native CWS library. Path is relative to the CWS installation root.  */
  protected static final String LIBRARYNAME = "bin/libwsp.so";

  /**  Debugging information is turned on if true. */
  protected static final boolean debug = 
    Boolean.valueOf(IfsSystem.getIfsSystemProperty(IfsSystem.DEBUG_PROP_NAME)).booleanValue();

  /**  
   * The JVM system property name whose value is the name of the CWS 
   * application configuration file. 
  */
  protected static final String APP_CONFIG_PROP_NAME = "ifs.app.config.file";   //T2000176

  /**  
   * The JVM system property name whose value is the name of the CWS 
   * environment configuration file. 
  */
  protected static final String ENV_CONFIG_PROP_NAME = "ifs.env.config.file";   //T2000176

  /**  
   * The default name of the CWS application configuration file if the JVM system 
   * property was not set. 
  */
  protected static final String DEFAULT_APP_CONFIG_NAME =
    IfsSystem.getIfsSystemProperty(IfsSystem.DEFAULT_APP_CONFIG_PROP_NAME);     //T2000176

  /**  
   * The default name of the CWS environment configuration file if the JVM system 
   * property was not set. 
  */
  protected static final String DEFAULT_ENV_CONFIG_NAME =
    IfsSystem.getIfsSystemProperty(IfsSystem.DEFAULT_ENV_CONFIG_PROP_NAME);     //T2000176

  /**  The singleton IfsConnection Object. */ 
  protected static IfsConnection singleConnection  = null;
   
  /**  
   * Thrown by the getConnection method if there is a problem creating the 
   * singleton IfsConnection Object. 
  */
  protected static IfsException  singleException   = null;

/* --------------------------------- Constructors --------------------------- */

  /**
   * Creates a new IfsConnection Object. During creation the back-end websrvr
   * process is started. If this constructor does not throw an IfsException
   * then the back-end process should be loaded and ready to recieve requests.
   * 
   * @param servlet The Servlet instance that requested the back-end websrvr
   *                process startup. This instance is used for logging and to
   *                obtain information about the environment.
   * @exception IfsException If there is a problem starting the back-end
   *                         websrvr process.
  */ 
  private IfsConnection( GenericServlet servlet ) throws IfsException {
    if (debug) 
      System.out.println( "IfsConnection cons servlet=" + servlet );
    
    // Retrieve the CWS installation root for loading of CWS resources  
    String deploymentPath = servlet.getServletContext().getRealPath( "/" );
    if (deploymentPath == null) deploymentPath = "";
    
    // The config file is either specified by a JVM system property or defaults
    // to an IFS system property. 
    String appConfigFileName = System.getProperty(APP_CONFIG_PROP_NAME, DEFAULT_APP_CONFIG_NAME );  //T2000176
    String envConfigFileName = System.getProperty(ENV_CONFIG_PROP_NAME, DEFAULT_ENV_CONFIG_NAME );  //T2000176
    if (! envConfigFileName.startsWith("/")) {
        envConfigFileName = deploymentPath + "/" + envConfigFileName;
    }

    if (debug)
        System.out.println("IfsConnections cons, envConfigFileName = " + envConfigFileName);  //pdltest

    // Deduce the environment directory from the wsp_exe_name parameter in the environment
    // config file                                        - T2000187
    String environmentPath = deploymentPath;  // initialize to java deployment path
    boolean foundWspExeName = false;          // set to true when line starting with "wsp_exe_name=" is found
    String exekey="wsp_exe_name=";            // string to search for
    try {
        BufferedReader in = new BufferedReader (new FileReader(envConfigFileName));
        String line;
        while (in.ready() && !foundWspExeName) {
            line = in.readLine();
            if (line.startsWith(exekey)) {
                String exe_name = line.substring(exekey.length());
                if( ! exe_name.startsWith("/")) {
                    exe_name = deploymentPath + "/" + exe_name;
                }
                File exeFile = new File(exe_name);
                environmentPath = exeFile.getParentFile().getParent();
                foundWspExeName = true;
            }
        }
        in.close();
    } catch(Throwable t) {
        IfsException fileReadException = new IfsException("IfsConnection", "Error reading environment configuration file, " + envConfigFileName);
        t.printStackTrace( System.err );
        throw fileReadException;
    }

    // Confirm we found the "wsp_exe_name" in the config file                                     //T2000187
    if (!foundWspExeName) {
        IfsException nolibwspException = new IfsException("IfsConnection", "Required parameter " + exekey + " not found in " + envConfigFileName);
        throw nolibwspException;
    }

    // Confirm environmentPath is good and the libwsp.so file exists.                             //T2000187
    File libwspFile = new File(environmentPath + "/" + LIBRARYNAME);
    if (!libwspFile.exists()) {
        IfsException nolibwspException = new IfsException("IfsConnection", "Cannot find library file \"" + libwspFile.getPath() + "\".  Path information was retrieved from " + exekey + " value from environment config file " + envConfigFileName + ".");
        throw nolibwspException;
    }

    if (debug) { 
      System.out.println( "IfsConnection cons appConfigFileName \"" + appConfigFileName + "\"" );   //T2000176
      System.out.println( "IfsConnection cons envConfigFileName \"" + envConfigFileName + "\"" );   //T2000176
      System.out.println( "IfsConnection cons deploymentPath \"" + deploymentPath + "\"" );         //T2000187
      System.out.println( "IfsConnection cons environmentPath \"" + environmentPath + "\"");        //T2000187
    }
  
    // Load the IFS native library using it's absolute path avoiding the need
    // to change the java.library.path JVM system property
    System.load( libwspFile.getPath() );                                                            //T2000187
      
    if (debug) { 
      System.out.println( "IfsConnection cons loadLibrary \"" + LIBRARYNAME + "\" done" );
      System.out.println( "IfsConnection cons initConnection attempt" );
    }
 
    // Load the back-end websrvr process     
    initConnection( servlet, deploymentPath, environmentPath, appConfigFileName, envConfigFileName );                //T2000176
      
    if (debug) 
      System.out.println( "IfsConnection cons done." );
   
  } 

/* ----------------------------- Class Methods ------------------------------ */

  /**
   * Returns the singleton IfsConnection Object. 
   *
   * @param servlet The Servlet requesting the IfsConnection Object. The Servlet
   *                instance is used for logging.
   * @exception IfsException If there is a problem creating the singleton
   *                         IfsConnection Object.
   * @return IfsConnection The singleton IfsConnection Object.
  */ 
  public static synchronized IfsConnection getConnection(GenericServlet servlet)
    throws IfsException {
    
    if (debug) 
      System.out.println( "IfsConnection.getConnection servlet=" + servlet );

    // If we have not initiailzed this singleton, get a connection.
    if (singleConnection == null) {
      // If servlets keep asking for a bad connection, continue complaining.
      if (singleException != null) 
        throw singleException;

      try {
       
        if (debug) 
          System.out.println( "IfsConnection.getConnection instantiating IFSConnection" );
        
        singleConnection = new IfsConnection( servlet );
      // Runtime.getRuntime().addShutdownHook(new Thread(singleConnection, "IfsConnection Shutdown Hook")); 
        
        if (debug) 
          System.out.println( "IfsConnection.getConnection instantiation done" );
         
      } 
      catch (IfsException e) {
        // Warning. Other exceptions such as bad args, out of memory are not handled here.
        // Only errors exclusively dealing with IFS connections are recorded here.
        e.printStackTrace( System.err );
        singleException = e;
        throw e;
      }

    } // End if 

    if (debug) 
      System.out.println( "IfsConnection.getConnection done" );
      
     return singleConnection;
   
  }

/* ----------------------------- Instance Methods --------------------------- */   

  /**
   * JVM shutdown hook Thread. This method will be called before the JVM
   * terminates and will initiate a shutdown of the back-end websrvr process.
   *
   * @return void
  */ 
  public void run() {
    markDropped();
    dropConnection();
  } 

  /**
   * Internal cleanup of singleton instance variables after the back-end
   * websrvr process has been terminated.
   *
   * @return void
  */ 
  private static synchronized void markDropped() {
    singleException = new IfsException("IfsConnection", "Connection dropped");
    singleConnection = null;
  } 

  /**
   * Starts the back-end websrvr process.
   *
   * @param servlet The Servlet instance requesting a back-end startup.
   *                The Servlet is used for logging and obtaining
   *                information about the environment.
   * @param deploymentPath The absolute path to the CWS installation base
   *                       directory.
   * @param appConfigFileName The path to the CWS application configuration file.  If 
   *                       it begins with a '/' it is considered an absolute path
   *                       else it is considered relative to 
   *                       <CODE>deploymentPath</CODE>.
   * @param envConfigFileName The path to the CWS environment configuration file.  If
   *                       it begins with a '/' it is considered an absolute path
   *                       else it is considered relative to 
   *                       <CODE>deploymentPath</CODE>.
   * @return void 
  */ 
  private native void initConnection(GenericServlet servlet, 
                                     String deploymentPath, 
                                     String environmentPath,        //pdlpack
                                     String appConfigFileName,
                                     String envConfigFileName );    //T2000176
   
  /**
   * Terminates the back-end websrvr process.
   *
   * @return void
  */
  private native void dropConnection();

} // End com.ibm.ifs.servlets.IfsConnection class
